/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#ifndef __CDispenser_H__
#define __CDispenser_H__

#include <new.h>
#include "FXElement.h"
#include "FXArray.h"

class CProperty : public IProperty
{
protected:
	CProperty(IDispenser* pDispenser) {m_pDispenser = pDispenser; m_bEnabled = TRUE;}

public:
	void enable(void) {m_bEnabled = TRUE;}
	void disable(void) {m_bEnabled = FALSE;}
	virtual FXbool isEnabled(void) {return m_bEnabled;}

	virtual const char* name(void) = 0;
	virtual ePropertyType type(void) = 0;
	virtual const char* description(void) = 0;
	virtual FXint style(void) {return PROPERTY_MULTIPLE|PROPERTY_UPDATEEVERYKEY;}
	virtual const char* tooltip(void) {return "";}

	virtual void set(FXWindow* pWindow, FXString strValue) = 0;
	virtual FXString get(FXWindow* pWindow) = 0;

	// This version of validate is used to update options within this property
	// -- See CPropertyLayout for an example.
	virtual void validate(IConnector* pConnector, FXString& strNewValue) {}
	
	// This version of validate is used to update other properties
	// -- See CPropertyLayout for an example.
	virtual void validate(IConnector* pConnector) {}

	virtual OPTION* option(void) {return 0;}
	virtual void getOptions(FXWindow* pWindow, FXString& strOptions);

	virtual FXString getImplementationCode(FXWindow* pWidget, FXString& widget) {return "";}

	FXbool getProperty(const FXchar* pszName, IProperty** ppProperty);

private:
	FXbool m_bEnabled;
	IDispenser* m_pDispenser;
};

// Implemented in CAppDialog.cpp if you're interested
extern "C" void toggleAttribute(FXint& nOldValue, FXint& nNewValue, FXint attributeA, FXint attributeB);

///////////////////////////////////////////////////////////
//

class CDispenser : public IDispenser
{
public:
	virtual ~CDispenser() {for (FXint i = 0; i < m_Properties.no(); i++) delete m_Properties[i];}

	virtual void addProperty(IProperty* property) {m_Properties.insert(0, property);}

	// Adding a widget to toolbox
	virtual const char* getWidgetName(void) = 0;
	virtual const void* getWidgetIcon(void) = 0;
	virtual const char* getWidgetCategory(void) = 0;
	virtual const int getWidgetGrabbers(void) = 0;

	// Creating a widget (user has clicked on the toolbar)
	virtual FXWindow* createWidget(FXComposite* p, FXObject* tgt=NULL, FXSelector sel=0) = 0;

	virtual FXint getDefaultWidth(void)  = 0;
	virtual FXint getDefaultHeight(void) = 0;
	virtual FXint getMinimumWidth(void)  = 0;
	virtual FXint getMinimumHeight(void) = 0;
	virtual FXint getMaximumWidth(void)  = 0;
	virtual FXint getMaximumHeight(void) = 0;

	// widget properties
	FXint getPropertyCount(void) {return m_Properties.no();}
	IProperty* getProperty(FXint index) {return m_Properties[index];}
	IProperty& getProperty(const FXchar* pszName);

	//	Validate Parent/Child relationships
	virtual const FXMetaClass* getClassInfo(void) = 0;
	virtual FXbool canHaveChildren(void) = 0;
	virtual FXbool allowChild(const FXMetaClass* pChildClass) = 0;
	virtual FXbool allowParent(FXWindow* pParent) = 0;
	virtual FXWindow* getCreateParent(FXWindow* pParent) = 0;
	virtual FXbool IsValidWidget(FXWindow** ppWindow) = 0;
	virtual FXbool widgetDeletesChildren(void) {return FALSE;}

	// Another instance of this component is required
	virtual IDispenser* clone(void) = 0;

	// Return the code snippet necessary to create this component
	virtual FXString getChildSuffix(void) = 0;
	virtual void getDefinitionCode(ITextStream& stream, FXWindow* pWidget, FXString& widget, FXString& parent) = 0;
	virtual void getImplementationCode(ITextStream& stream, FXWindow* pWidget, FXString& widget, FXString& parent) = 0;

protected:
	void getPropertyCode(ITextStream& stream, FXWindow* pWindow, FXString& widget)
	{
		FXint nCount = getPropertyCount();
		for (FXint i = 0; i < nCount; i++)
		{
			IProperty* pProperty = getProperty(i);

			if ( pProperty )
			{
				stream << pProperty->getImplementationCode(pWindow, widget);
			}
		}
	}

protected:
	FXArray<IProperty*> m_Properties;
};

#endif // __CDispenser_H__
