/**************************************************************************  
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#ifndef __CAppDialog_H__
#define __CAppDialog_H__

class CAppDialog : public CApp
{
	FXDECLARE(CAppDialog)

	friend class CTrackManager;

// Services
public:
	CAppDialog(const FXchar *name="Application",const FXchar *vendor="Default");
	virtual ~CAppDialog();

// Operations
public:
	void addWidget(CConnector& connector);
	void clearWidget(CConnector& connector);
	void clearWidgets(FXbool bSelectDialog = TRUE);

	virtual FXbool isValidWidget(FXWindow* wnd);
	FXbool isValidWidget(FXWindow* wnd, CConnector** ppConnector);
	FXint getWidgetIndex(FXWindow* wnd, CConnector** ppConnector);

	CTrackManager* getTrackManager(void) {return &m_TrackManager;}

	FXbool isDirty(void) {return m_bDirty;}
	void setDirty(void) {m_bDirty = TRUE;}
	FXbool doFileSave(void);

	FXbool isRunning(void) {return m_bRunning;}

	void refreshWidgetTree(void);

	void addDispenser(IDispenser* pDispenser);
	IDispenser* findDispenser(FXString strName);
	FXint getDispenserCount(IDispenser* pDispenser);
	void zeroDispenserCount(void);

// Operations
protected:
	void deleteWidget(FXint nIndex, FXbool bForceDelete);
	FXbool getWidget(FXWindow* wnd, CConnector** ppConnector, FXint& nIndex);

	virtual void doResetAction(void)
		{m_TrackManager.resetAction();}

	virtual void doPaint(FXWindow* window)
		{m_TrackManager.onPaint(window);}
	virtual FXbool doKeyDown(FXWindow* window, FXuint key, FXuint code)
		{return m_TrackManager.OnKeyDown(window, key, code);}
	virtual FXbool doKeyUp(FXWindow* window, FXuint key, FXuint code)
		{return m_TrackManager.OnKeyUp(window, key, code);}
	virtual FXbool doLButtonDown(FXWindow* window, FXint x, FXint y, FXbool bCtrl)
		{return m_TrackManager.onLButtonDown(window, x, y, bCtrl);}
	virtual FXbool doMouseMove(FXWindow* window, FXint x, FXint y)
		{return m_TrackManager.onMouseMove(window, x, y);}
	virtual FXbool doLButtonUp(FXWindow* window, FXint x, FXint y)
		{return m_TrackManager.onLButtonUp(window, x, y);}

// Events
public:
	enum
	{
		IDM_FILENEW = FXApp::ID_LAST,
		IDM_FILEOPEN,
		IDM_FILESAVE,
		IDM_FILESAVEAS,
		ID_LAST
	};

	long onCmdQuit(FXObject* o,FXSelector s,void* ptr);
	long onFileNew(FXObject* o,FXSelector s,void* ptr);
	long onFileOpen(FXObject* o,FXSelector s,void* ptr);
	long onFileSave(FXObject* o,FXSelector s,void* ptr);
	long onFileSaveAs(FXObject* o,FXSelector s,void* ptr);

	long onUpdateFileNew(FXObject* o,FXSelector s,void* ptr);

// Attributes
private:
	FXArray<CConnector*> m_Widgets;

	FXWindow* m_pAppFrame;
	FXWindow* m_pFrame;
	FXWindow* m_pPane;

	CTrackManager m_TrackManager;

	FXString m_strFileName;

	FXbool m_bDirty;
	FXbool m_bRunning;

private:
	struct DISPENSERDATA
	{
		IDispenser* pDispenser;
		FXint       nCount;
	};
	FXArray<DISPENSERDATA> m_Dispensers;
};

extern CAppDialog* application;

#endif // __CAppDialog_H__
