/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#ifndef __CMainFrame_H__
#define __CMainFrame_H__

class CMainFrame : public FXMainWindow 
{
	FXDECLARE(CMainFrame)

// Services
public:
	CMainFrame() {}
	CMainFrame(FXApp* a);
	virtual ~CMainFrame();

// Operations
public:
	virtual void layout();
	virtual void create();

	void createIcons(void);
	void createMenu(void);
	void createToolbar(void);
	CConnector createDialog(FXbool bShow = FALSE);

// Events
public:
  enum {
    ID_ADDWIDGET = FXMainWindow::ID_LAST,
	IDM_ALIGNLEFT,
	IDM_ALIGNRIGHT,
	IDM_ALIGNTOP,
	IDM_ALIGNBOTTOM,
	IDM_SIZEWIDTH,
	IDM_SIZEHEIGHT,
	IDM_SIZEBOTH,
	IDM_CENTREHORIZONTAL,
	IDM_CENTREVERTICAL,
	IDM_SNAPTOGRID,
	IDM_DRAGOUTLINE,
    ID_LAST
    };

	long onAlignLeft(FXObject*,FXSelector,void* ptr);
	long onUpdateAlignLeft(FXObject*,FXSelector,void* ptr);

	long onAlignRight(FXObject*,FXSelector,void* ptr);
	long onUpdateAlignRight(FXObject*,FXSelector,void* ptr);

	long onAlignTop(FXObject*,FXSelector,void* ptr);
	long onUpdateAlignTop(FXObject*,FXSelector,void* ptr);

	long onAlignBottom(FXObject*,FXSelector,void* ptr);
	long onUpdateAlignBottom(FXObject*,FXSelector,void* ptr);

	long onSizeWidth(FXObject*,FXSelector,void* ptr);
	long onUpdateSizeWidth(FXObject*,FXSelector,void* ptr);

	long onSizeHeight(FXObject*,FXSelector,void* ptr);
	long onUpdateSizeHeight(FXObject*,FXSelector,void* ptr);

	long onSizeBoth(FXObject*,FXSelector,void* ptr);
	long onUpdateSizeBoth(FXObject*,FXSelector,void* ptr);

	long onCentreHorizontal(FXObject*,FXSelector,void* ptr);
	long onUpdateCentreHorizontal(FXObject*,FXSelector,void* ptr);

	long onCentreVertical(FXObject*,FXSelector,void* ptr);
	long onUpdateCentreVertical(FXObject*,FXSelector,void* ptr);

	long onCmdExit(FXObject*,FXSelector s,void*);

	long onSnapToGrid(FXObject*,FXSelector s,void*);
	long onUpdateSnapToGrid(FXObject*,FXSelector s,void*);

	long onDragOutline(FXObject*,FXSelector s,void*);
	long onUpdateDragOutline(FXObject*,FXSelector s,void*);

// Attributes
private:
	FXHorizontalFrame *toolbar;
	FXVerticalFrame *buttonFrame;
	FXVerticalFrame* properties;
	FXTreeList      *tree;
	
	FXComposite* m_pDialog;

	FXMenuBar* menubar;
	FXMenuPane* filemenu;

	bool m_bInitialSize;

	FXLabel*	   m_pDescription;
	CNameComboBox*	   m_pNames;

	FXVerticalFrame*   m_pFrame;
	FXSplitter*        m_pSplitter1; // new
	FXSplitter*        m_pSplitter2; // new
	CPropertyList*     m_pProperties;

	CGarbage m_Garbage;

// Resources
private:
	FXGIFIcon* m_iconFileNew;
	FXGIFIcon* m_iconFileOpen;
	FXGIFIcon* m_iconFileSave;
	FXGIFIcon* m_iconFileSaveAs;

	FXGIFIcon* m_iconAlignLeft;
	FXGIFIcon* m_iconAlignRight;
	FXGIFIcon* m_iconAlignTop;
	FXGIFIcon* m_iconAlignBottom;

	FXGIFIcon* m_iconSizeWidth;
	FXGIFIcon* m_iconSizeHeight;
	FXGIFIcon* m_iconSizeBoth;

	FXGIFIcon* m_iconCentreHorizontal;
	FXGIFIcon* m_iconCentreVertical;
};

#endif // __CMainFrame_H__
