/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>
#include "CDispenserComboBox.h"

#include <CPropertyBackColor.h>
#include <CPropertyTop.h>
#include <CPropertyLeft.h>
#include <CPropertyLayout.h>
#include <CPropertyFrame.h>
#include <CPropertyWidth.h>
#include <CPropertyHeight.h>
#include <CPropertyTipText.h>
#include <CPropertyColumns.h>
#include <CPropertyVisible.h>
#include <CPropertyPadBottom.h>
#include <CPropertyPadLeft.h>
#include <CPropertyPadRight.h>
#include <CPropertyPadTop.h>

#include <CVariant.h>

///////////////////////////////////////////////////////////////////////////////
//

template <class T>
class CPropertyComboBoxStyle : public CProperty
{
public:
	CPropertyComboBoxStyle(IDispenser* pDispenser) : CProperty(pDispenser) {}
	
	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	OPTION* option(void);
	virtual void getOptions(FXWindow* pWindow, FXString& strOptions);

private:
	static OPTION Style[];
};

// From FXComboBox.h
template <class T>
OPTION CPropertyComboBoxStyle<T>::Style[] = 
{
	{"COMBOBOX_REPLACE"      , COMBOBOX_REPLACE      ,"Replace current item with typed text"},
	{"COMBOBOX_INSERT_BEFORE", COMBOBOX_INSERT_BEFORE,"Typed text inserted before current"},
	{"COMBOBOX_INSERT_AFTER" , COMBOBOX_INSERT_AFTER ,"Typed text inserted after current"},
	{"COMBOBOX_INSERT_FIRST" , COMBOBOX_INSERT_FIRST ,"Typed text inserted at begin of list"},
	{"COMBOBOX_INSERT_LAST"  , COMBOBOX_INSERT_LAST  ,"Typed text inserted at end of list"},
	{"COMBOBOX_STATIC"       , COMBOBOX_STATIC       ,"Unchangable text box"},
	{0, 0, 0}
};

template <class T>
const char* CPropertyComboBoxStyle<T>::name(void)
{
	return "Style";
}

template <class T>
ePropertyType CPropertyComboBoxStyle<T>::type(void)
{
	return PROPERTY_COMBO;
}

template <class T>
const char* CPropertyComboBoxStyle<T>::description(void)
{
	return "The combo box style for this object";
}

template <class T>
const char* CPropertyComboBoxStyle<T>::tooltip(void)
{
	return "getComboStyle()\nsetComboStyle()";
}

template <class T>
void CPropertyComboBoxStyle<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setComboStyle( getIntFromString(strValue) );
}

template <class T>
FXString CPropertyComboBoxStyle<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromInt( p->getComboStyle() );
}

template <class T>
OPTION* CPropertyComboBoxStyle<T>::option(void)
{
	return Style;
}

template <class T>
void CPropertyComboBoxStyle<T>::getOptions(FXWindow* pWindow, FXString& strOptions)
{
	T* p = static_cast<T*>(pWindow);
	
	if ( p->getComboStyle() == 0 )
		return;
}

///////////////////////////////////////////////////////////////////////////////
//

const unsigned char ComboBox[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xb3,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x84,0x84,0x84,0x84,0xc6,0xc6,0xc6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x04,0xfe,0x50,0xc8,0x49,0xab,0xbd,0x38,0xeb,
  0xcd,0xbb,0xff,0x60,0x28,0x8e,0x64,0x69,0x9e,0x68,0xaa,0xae,0x6c,0x2b,0x0e,0x70,
  0x2c,0xcf,0x74,0x6d,0xdf,0xb7,0x36,0xb8,0x25,0xb1,0x67,0x3b,0x80,0x70,0x48,0x2c,
  0x1a,0x8f,0xc8,0x24,0x71,0xe0,0xd3,0x09,0x00,0x84,0xa8,0x74,0x4a,0xad,0x5a,0xad,
  0x38,0x18,0x80,0xf9,0xc3,0x04,0xaf,0xe0,0x2a,0x34,0xcc,0x04,0x3f,0xb9,0xce,0x71,
  0x94,0x18,0xa6,0xaa,0xaf,0x5c,0xdb,0xb9,0x09,0x7c,0x4a,0xdf,0xed,0x29,0xbe,0x1a,
  0xaf,0xcd,0xbb,0x17,0x5f,0x04,0x7b,0x83,0x42,0x6b,0x86,0x86,0x85,0x89,0x8a,0x51,
  0x5c,0x47,0x41,0x68,0x75,0x63,0x7b,0x6a,0x50,0x93,0x6b,0x83,0x77,0x99,0x98,0x71,
  0x45,0x30,0x7f,0x69,0x98,0x6e,0x9a,0x97,0x99,0x95,0x9b,0x99,0x7d,0x43,0x31,0xa0,
  0x92,0xa2,0x7a,0xa4,0xa2,0x96,0xb1,0x9c,0x3e,0x31,0x5b,0xac,0x5b,0x74,0x5e,0x76,
  0xaf,0xaf,0xb3,0x9b,0xc0,0xa6,0x9a,0x7d,0x34,0xad,0xbc,0x6f,0x6c,0x8c,0xc1,0x98,
  0xca,0xcb,0xc5,0x33,0xc7,0x81,0xbd,0x79,0xd5,0x70,0x1b,0xba,0x80,0x16,0x82,0xd6,
  0xdd,0x53,0x03,0x4a,0x42,0x91,0xc8,0x59,0xe5,0xe6,0xe6,0xbb,0xd3,0xde,0xeb,0xec,
  0xda,0x15,0xe7,0xf0,0xf1,0x39,0x19,0xe1,0xf5,0x47,0x01,0xf8,0xf6,0x48,0x1a,0x00,
  0xf8,0xfe,0xff,0x00,0x03,0x0a,0x1c,0xf8,0xaf,0x9c,0x10,0x7e,0xfe,0xa4,0x04,0x20,
  0x40,0xb0,0x21,0xc3,0x86,0x65,0xae,0x3c,0x01,0x80,0x30,0x21,0xbe,0x87,0x0d,0x03,
  0x46,0xc9,0x08,0x4d,0xc6,0xc4,0x8a,0xfe,0x17,0x2f,0x6e,0x5c,0x38,0x52,0xe1,0xc8,
  0x90,0x10,0x6d,0xf9,0x39,0x48,0xef,0xdf,0x43,0x8c,0x30,0x17,0xa2,0x8c,0x99,0x52,
  0x06,0xae,0x4f,0x2c,0x31,0xf4,0xb3,0x28,0x53,0xa4,0xc9,0x99,0x3c,0x09,0xaa,0x12,
  0x87,0x93,0x62,0x4b,0x9e,0x34,0x91,0xa2,0xec,0x29,0xd4,0x96,0xa7,0x20,0x46,0x31,
  0x00,0x7c,0x19,0x94,0x21,0x55,0xa6,0x18,0x07,0x3a,0x32,0x02,0x55,0x83,0xcb,0x92,
  0x2f,0x4b,0x02,0x25,0x99,0x35,0x60,0xc7,0x5c,0x51,0x2f,0xec,0xcc,0xc8,0xb6,0x6d,
  0x80,0xb3,0x45,0xf9,0xb1,0x9b,0xdb,0x08,0x5b,0x5a,0x0b,0x95,0x10,0xb1,0x1b,0x12,
  0x46,0x9f,0x5c,0x4d,0xa8,0xac,0x9d,0x82,0xe3,0xa3,0x30,0x17,0xc3,0x77,0x2b,0x24,
  0x3b,0x34,0x29,0x11,0xdf,0x45,0x80,0x09,0x1f,0x9e,0x3c,0xe8,0xaf,0xac,0xc8,0xc0,
  0x4a,0xf9,0xe2,0x63,0xb8,0xb3,0x23,0xcb,0xcc,0x18,0xeb,0x0d,0x1d,0x58,0xb2,0xe7,
  0xc2,0x89,0x29,0x2c,0x46,0x85,0xa7,0x94,0x66,0x30,0x94,0x63,0xa7,0x9e,0xb0,0x7a,
  0xf0,0x30,0x66,0x84,0xb0,0x9c,0x3e,0x3c,0x5b,0x42,0xde,0xc6,0x8c,0x2f,0x9b,0x72,
  0xc6,0x39,0x36,0x62,0xd0,0x74,0xbd,0x19,0xe7,0x8d,0x3c,0x79,0x35,0x83,0xbd,0xa9,
  0x3d,0xde,0x0b,0x99,0xb3,0x99,0x9c,0x6a,0x61,0x95,0xce,0x63,0xbb,0xb8,0x1c,0xec,
  0x78,0xb5,0xf3,0x5d,0xf6,0x98,0x12,0x6c,0x1f,0x8f,0x3e,0xd2,0x13,0x8f,0x39,0x74,
  0x5e,0x32,0x4e,0x97,0x74,0x5d,0x0f,0xf8,0xf7,0xe8,0xd7,0xdb,0xa5,0x0c,0xbd,0xa9,
  0x5e,0x27,0xfb,0xcd,0xae,0x8d,0x3b,0x22,0xd9,0x2d,0x1e,0x81,0xa7,0xd8,0x7f,0xac,
  0xb9,0x97,0xe0,0x80,0x2b,0x45,0xf7,0xdb,0x21,0xc1,0x29,0xb8,0x0c,0x83,0xc6,0x18,
  0xa8,0x9a,0x73,0xeb,0xb8,0xa3,0x98,0x83,0x18,0x7a,0xe3,0x17,0x3d,0xfa,0x84,0x28,
  0xe2,0x3e,0x3c,0x94,0x68,0xe2,0x89,0x28,0xa6,0xa8,0xe2,0x8a,0x2c,0xb6,0x68,0x62,
  0x04,0x00,0x00,0x3b
  };

CDispenserComboBox::CDispenserComboBox()
{
	m_Properties.append(new CPropertyLayout<FXComboBox>(this));
	m_Properties.append(new CPropertyComboBoxStyle<FXComboBox>(this));
	m_Properties.append(new CPropertyBackColor<FXComboBox, FXRGB(192, 192, 192)>(this));
	m_Properties.append(new CPropertyTipText<FXComboBox>(this));
	m_Properties.append(new CPropertyColumns<FXComboBox, 50>(this));
	m_Properties.append(new CPropertyVisible<FXComboBox, 4>(this));
	m_Properties.append(new CPropertyPadTop<FXComboBox>(this));
	m_Properties.append(new CPropertyPadLeft<FXComboBox>(this));
	m_Properties.append(new CPropertyPadBottom<FXComboBox>(this));
	m_Properties.append(new CPropertyPadRight<FXComboBox>(this));
	m_Properties.append(new CPropertyLeft<FXComboBox>(this));
	m_Properties.append(new CPropertyTop<FXComboBox>(this));
	m_Properties.append(new CPropertyWidth<FXComboBox>(this));
	m_Properties.append(new CPropertyHeight<FXComboBox>(this));
}

const char* CDispenserComboBox::getWidgetName(void)
{
	return "FXComboBox";
}

const void* CDispenserComboBox::getWidgetIcon(void)
{
	return ComboBox;
}

const char* CDispenserComboBox::getWidgetCategory(void)
{
	return "Standard";
}

const int CDispenserComboBox::getWidgetGrabbers(void)
{
	return GS_ALL;
}

FXint CDispenserComboBox::getDefaultWidth(void)
{
	return 80;
}

FXint CDispenserComboBox::getDefaultHeight(void)
{
	return 16;
}

FXint CDispenserComboBox::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserComboBox::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserComboBox::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserComboBox::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserComboBox::getClassInfo(void)
{
	return &FXComboBox::metaClass;
}

FXbool CDispenserComboBox::allowChild(const FXMetaClass*)
{
	return FALSE;
}

FXbool CDispenserComboBox::allowParent(FXWindow*)
{
	return TRUE;
}

FXWindow* CDispenserComboBox::getCreateParent(FXWindow* pParent)
{
	return pParent;
}

/*
	FXComboBox
	  FXTextField
	  FXPopup
	    FXList
	  FXMenuButton
*/
FXbool CDispenserComboBox::IsValidWidget(FXWindow** ppWindow)
{
	if ( (*ppWindow)->isMemberOf(&FXComboBox::metaClass) )
	{
		return TRUE;
	}
	else if ( (*ppWindow)->isMemberOf(&FXTextField::metaClass) )
	{
		*ppWindow = (*ppWindow)->getParent();
		return *ppWindow && (*ppWindow)->isMemberOf(&FXComboBox::metaClass);
	}
	else if ( (*ppWindow)->isMemberOf(&FXList::metaClass) )
	{
		*ppWindow = (*ppWindow)->getParent();
		if ( *ppWindow && (*ppWindow)->isMemberOf(&FXPopup::metaClass) ) 
		{
			*ppWindow = (*ppWindow)->getParent();
			return *ppWindow && (*ppWindow)->isMemberOf(&FXComboBox::metaClass);
		}
	}
	else if ( (*ppWindow)->isMemberOf(&FXMenuButton::metaClass) )
	{
		*ppWindow = (*ppWindow)->getParent();
		return *ppWindow && (*ppWindow)->isMemberOf(&FXComboBox::metaClass);
	}

	return FALSE;
}

IDispenser* CDispenserComboBox::clone(void)
{
	return new CDispenserComboBox;
}

FXWindow* CDispenserComboBox::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	return new FXComboBox((FXComposite*)p, /*DDWidth*/50, /*DDHeight*/4, tgt, sel,
				COMBOBOX_STATIC|LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT);
}

void CDispenserComboBox::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	FXString strOptions;

	getProperty("Frame").getOptions(pWindow, strOptions);
	getProperty("Layout").getOptions(pWindow, strOptions);
	getProperty("Style").getOptions(pWindow, strOptions);

	stream << "\t" << widget << " = new FXComboBox(" << parent << ", 50, 4, 0, 0, ";

	stream << strOptions << ", " << getProperty("Left").get(pWindow) << "," << getProperty("Top").get(pWindow) << ",";
	stream << getProperty("Width").get(pWindow) << "," << getProperty("Height").get(pWindow);

	stream  << ", " << getProperty("PadLeft").get(pWindow) << "," << getProperty("PadRight").get(pWindow) << ",";
	stream << getProperty("PadTop").get(pWindow) << "," << getProperty("PadBottom").get(pWindow) << ");\n";

	getPropertyCode(stream, pWindow, widget);
}
