/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

/*
  BorderStyle 0-None, 1-Fixed single, 2-Sizable
  ControlBox
  MinButton
  MaxButton

  if controlbox=false no others show.

enum {
  DECOR_TITLE    = 0x00020000,        /// Window title
  DECOR_MINIMIZE = 0x00040000,        /// Minimize button
  DECOR_MAXIMIZE = 0x00080000,        /// Maximize button
  DECOR_CLOSE    = 0x00100000,        /// Close button
  DECOR_BORDER   = 0x00200000,        /// Border
  DECOR_RESIZE   = 0x00400000,        /// Resize handles
  DECOR_MENU     = 0x00800000,        /// Window menu
  DECOR_ALL      = (DECOR_TITLE|DECOR_MINIMIZE|DECOR_MAXIMIZE|DECOR_CLOSE|DECOR_BORDER|DECOR_RESIZE|DECOR_MENU)
  };

*/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>
#include "CDispenserDialogFrame.h"
#include "CFrame.h"

#include <CPropertyBackColor.h>
#include <CPropertyTitle.h>
#include <CPropertyTop.h>
#include <CPropertyLeft.h>
#include <CPropertyWidth.h>
#include <CPropertyHeight.h>

#include <CVariant.h>

#include "CAppPane.h"
#include "CDialogPane.h"

///////////////////////////////////////////////////////////////////////////////
//

class CPropertyMinimizeButton : public CProperty
{
public:
	CPropertyMinimizeButton(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void) {return "Minimize Button";}
	virtual ePropertyType type(void) {return PROPERTY_BOOL;}
	virtual const char* description(void) {return "Does this dialog display a minimize\nbutton?";}

	void set(FXWindow* pWindow, FXString strValue)
	{
		CFrame* p = static_cast<CFrame*>(pWindow);

		p->setMinimizeButton(getBoolFromString(strValue));
	}

	FXString get(FXWindow* pWindow)
	{
		CFrame* p = static_cast<CFrame*>(pWindow);

		return getStringFromBool(p->getMinimizeButton());
	}
};

///////////////////////////////////////////////////////////////////////////////
//

class CPropertyMaximizeButton : public CProperty
{
public:
	CPropertyMaximizeButton(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void) {return "Maximize Button";}
	virtual ePropertyType type(void) {return PROPERTY_BOOL;}
	virtual const char* description(void) {return "Does this dialog display a maximize\nbutton?";}

	void set(FXWindow* pWindow, FXString strValue)
	{
		CFrame* p = static_cast<CFrame*>(pWindow);

		p->setMaximizeButton(getBoolFromString(strValue));
	}

	FXString get(FXWindow* pWindow)
	{
		CFrame* p = static_cast<CFrame*>(pWindow);

		return getStringFromBool(p->getMaximizeButton());
	}
};

///////////////////////////////////////////////////////////////////////////////
//

class CPropertyMenuButton : public CProperty
{
public:
	CPropertyMenuButton(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void) {return "Menu Button";}
	virtual ePropertyType type(void) {return PROPERTY_BOOL;}
	virtual const char* description(void) {return "Does this dialog display a menu\nbutton?";}

	void set(FXWindow* pWindow, FXString strValue)
	{
		CFrame* p = static_cast<CFrame*>(pWindow);

		p->setMenuButton(getBoolFromString(strValue));
	}

	FXString get(FXWindow* pWindow)
	{
		CFrame* p = static_cast<CFrame*>(pWindow);

		return getStringFromBool(p->getMenuButton());
	}
};

///////////////////////////////////////////////////////////////////////////////
//

class CPropertyDlgTop : public CProperty
{
public:
	CPropertyDlgTop(IDispenser* pDispenser) : CProperty(pDispenser) {m_nValue = 0;}

	const char* name(void) {return "Top";}
	virtual ePropertyType type(void) {return PROPERTY_TEXT;}
	virtual const char* description(void) {return "The distance between the internal\ntop edge of this object and the top\nedge of its container";}

	void set(FXWindow* pWindow, FXString strValue)
	{
		m_nValue = getIntFromString(strValue);
	}

	FXString get(FXWindow* pWindow)
	{
		return getStringFromInt(m_nValue);
	}

private:
	FXint m_nValue;
};

///////////////////////////////////////////////////////////////////////////////
//

class CPropertyDlgLeft : public CProperty
{
public:
	CPropertyDlgLeft(IDispenser* pDispenser) : CProperty(pDispenser) {m_nValue = 0;}

	const char* name(void) {return "Left";}
	virtual ePropertyType type(void) {return PROPERTY_TEXT;}
	virtual const char* description(void) {return "The distance between the internal\nleft edge of this object and the left\nedge of its container";}

	void set(FXWindow* pWindow, FXString strValue)
	{
		m_nValue = getIntFromString(strValue);
	}

	FXString get(FXWindow* pWindow)
	{
		return getStringFromInt(m_nValue);
	}

private:
	FXint m_nValue;
};

///////////////////////////////////////////////////////////////////////////////
//

CDispenserDialogFrame::CDispenserDialogFrame()
{
	m_Properties.append(new CPropertyTitle<CFrame>(this));
	m_Properties.append(new CPropertyBackColor<CFrame, FXRGB(192, 192, 192)>(this));
	m_Properties.append(new CPropertyDlgLeft(this));
	m_Properties.append(new CPropertyMinimizeButton(this));
	m_Properties.append(new CPropertyMaximizeButton(this));
	m_Properties.append(new CPropertyMenuButton(this));
	m_Properties.append(new CPropertyDlgTop(this));
	m_Properties.append(new CPropertyWidth<CFrame>(this));
	m_Properties.append(new CPropertyHeight<CFrame>(this));
}

const char* CDispenserDialogFrame::getWidgetName(void)
{
	return "FXDialog";
}

const void* CDispenserDialogFrame::getWidgetIcon(void)
{
	return 0;
}

const char* CDispenserDialogFrame::getWidgetCategory(void)
{
	return 0;
}

const int CDispenserDialogFrame::getWidgetGrabbers(void)
{
	return GS_DIALOG;
}

FXint CDispenserDialogFrame::getDefaultWidth(void)
{
	return 80;
}

FXint CDispenserDialogFrame::getDefaultHeight(void)
{
	return 16;
}

FXint CDispenserDialogFrame::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserDialogFrame::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserDialogFrame::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserDialogFrame::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserDialogFrame::getClassInfo(void)
{
	return 0;
}

FXbool CDispenserDialogFrame::allowChild(const FXMetaClass*)
{
	return TRUE;
}

FXbool CDispenserDialogFrame::allowParent(FXWindow*)
{
	return TRUE;
}

FXWindow* CDispenserDialogFrame::getCreateParent(FXWindow* pParent)
{
	return pParent;
}

FXbool CDispenserDialogFrame::IsValidWidget(FXWindow** ppWindow)
{
	return TRUE;
}

IDispenser* CDispenserDialogFrame::clone(void)
{
	return new CDispenserDialogFrame;
}

FXWindow* CDispenserDialogFrame::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	return new CFrame(p, "Dialog", NULL, NULL,
				DECOR_TITLE|DECOR_BORDER, 8, 8, 160, 200);
}

/*
 *  The FXDialog c-tor accepts the x,y,w,h parameters as the size of the
 *  client area of the dialog, not the entire dialog including the frame.
 *
 *  Because of this getImplementationCode cannot use getProperty("Width")
 *  and getProperty("Height") since this class will return the dimensions
 *  of the frame it represents.
 *
 *  Instead, getImplementationCode calls the getChildxxx functions below
 *  to get the dimensions of the client area of the 'dialog' which is
 *  represented by CDialogPane.
 */
FXint getChildWidth(FXWindow* pWnd)
{
	for ( FXWindow* pWindow = pWnd->getFirst(); pWindow; pWindow = pWindow->getNext() )
	{
		if ( pWindow->isMemberOf(&CDialogPane::metaClass) )
			return pWindow->getWidth();
	}

	return 0;
}

FXint getChildHeight(FXWindow* pWnd)
{
	for ( FXWindow* pWindow = pWnd->getFirst(); pWindow; pWindow = pWindow->getNext() )
	{
		if ( pWindow->isMemberOf(&CDialogPane::metaClass) )
			return pWindow->getHeight();
	}

	return 0;
}

void CDispenserDialogFrame::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	stream << "\n";
	stream << "FXIMPLEMENT(" << widget << ", FXDialogBox, NULL, 0)\n";
	stream << "\n";

	stream << "" << widget << "::" << widget << "(FXWindow* owner)\n";
	stream << ": FXDialogBox(owner, " << char(34) << getProperty("Title").get(pWindow) << char(34) << ", ";

	stream << "DECOR_TITLE|DECOR_CLOSE|DECOR_BORDER";

	if ( getProperty("Minimize Button").get(pWindow) == "True" )
		stream << "|DECOR_MINIMIZE";

	if ( getProperty("Maximize Button").get(pWindow) == "True" )
		stream << "|DECOR_MAXIMIZE";

	if ( getProperty("Menu Button").get(pWindow) == "True" )
		stream << "|DECOR_MENU";

	stream << ", ";
	stream << getProperty("Left").get(pWindow) << ",";
	stream << getProperty("Top").get(pWindow) << ",";
	stream << getStringFromInt(getChildWidth(pWindow)) << ",";
	stream << getStringFromInt(getChildHeight(pWindow));
	stream << ")\n";
}

