/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>
#include "CDispenserFrame.h"

#include <CPropertyLeft.h>
#include <CPropertyJustify.h>
#include <CPropertyLayout.h>
#include <CPropertyWidth.h>
#include <CPropertyHeight.h>
#include <CPropertyFrame.h>
#include <CPropertyText.h>
#include <CPropertyTipText.h>
#include <CPropertyTop.h>
#include <CPropertyPadBottom.h>
#include <CPropertyPadLeft.h>
#include <CPropertyPadRight.h>
#include <CPropertyPadTop.h>

#include <CVariant.h>

const unsigned char HorizontalFrame[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xb3,0x00,0x00,0x00,0x00,0x00,
  0x84,0x84,0x84,0xc6,0xc6,0xc6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x04,0xfe,0x30,0xc8,0x49,0xab,0xbd,0x38,0xeb,
  0xcd,0xbb,0xff,0x60,0x28,0x8e,0x64,0x69,0x9e,0x68,0x2a,0xac,0x6c,0xeb,0xbe,0x70,
  0x2c,0xcb,0xda,0x6c,0xdf,0x78,0x0c,0x64,0x79,0xef,0xd3,0xbb,0xcb,0x6f,0x48,0xdc,
  0x05,0x2b,0xc4,0x64,0x2f,0x70,0x44,0xb6,0x06,0xd0,0xa8,0x34,0x0a,0x10,0x4c,0xaf,
  0xd0,0x2a,0xf6,0x5a,0x6d,0x19,0x85,0x4f,0x1b,0xd3,0x2a,0xd6,0x96,0x5d,0x4c,0x8c,
  0x6b,0x70,0x66,0xcf,0xc6,0x6e,0xa0,0xeb,0x6b,0x59,0xb7,0xef,0xef,0x2e,0x2b,0x0d,
  0x66,0xc5,0x63,0x70,0x78,0x72,0x5e,0x7c,0x4e,0x7e,0x82,0x80,0x66,0x79,0x68,0x4d,
  0x14,0x76,0x8b,0x7f,0x30,0x81,0x8b,0x84,0x8d,0x12,0x8f,0x72,0x91,0x2f,0x93,0x83,
  0x7b,0x96,0x97,0x61,0x90,0x88,0x92,0x7a,0x2b,0x74,0x86,0x2b,0x9a,0x8c,0x64,0xa2,
  0x94,0x9e,0x6a,0xa1,0x99,0xa3,0x9b,0xa5,0x02,0xa7,0x8e,0xb1,0x89,0xac,0xb2,0xae,
  0x2b,0x85,0xa8,0xbb,0xba,0xaa,0x2d,0x9c,0xba,0x95,0x7d,0xa9,0xb3,0xab,0xc3,0x9e,
  0x8c,0xb0,0x87,0xad,0xbc,0x9d,0xa6,0xbf,0x13,0x98,0xc2,0xca,0xc4,0xb5,0xd5,0xd6,
  0xb9,0xa4,0xc1,0xdf,0xcc,0xbe,0xb5,0xb7,0xdd,0xd0,0xd2,0xd8,0xbd,0x02,0xdc,0xa0,
  0xe7,0xe9,0xe8,0xdf,0xc7,0x75,0xde,0xb4,0xe0,0xf5,0xe2,0xeb,0xdb,0x9f,0x01,0xd7,
  0xe1,0xd9,0xcd,0xf2,0x80,0xe1,0x2b,0xe6,0x4f,0x1d,0x3b,0x7e,0xf4,0x96,0xfd,0x1b,
  0x37,0x87,0x5d,0xbf,0x7b,0x0b,0xf3,0x39,0x43,0x66,0x4f,0x61,0x34,0x63,0x2c,0xca,
  0xb5,0x4b,0x76,0xb1,0xe0,0xb4,0x7c,0xb7,0xcf,0x38,0xc2,0x83,0xa8,0x4e,0x23,0x42,
  0x77,0x1e,0xdf,0x61,0x1c,0x17,0xb2,0xa2,0x36,0x97,0xcd,0x06,0x92,0x73,0x98,0xf0,
  0xa5,0x4c,0x98,0x0c,0xb5,0xb5,0xbc,0xc9,0xf3,0x4c,0x40,0x5c,0x28,0x49,0x8e,0x5c,
  0xa5,0x93,0x62,0xcf,0x8e,0xf5,0x7e,0x9a,0x13,0xa9,0x32,0x65,0xbc,0x57,0x46,0x23,
  0x12,0x4c,0x9a,0x91,0x66,0x50,0x8b,0x43,0x5f,0x16,0x9d,0x77,0xd5,0xa6,0xd4,0x99,
  0x9f,0x1e,0x62,0x6d,0x4a,0x95,0x65,0x54,0xa4,0x63,0x9f,0x52,0x0b,0x5b,0x33,0xe6,
  0xd7,0x89,0x5c,0x99,0x3a,0x15,0xba,0xd2,0x96,0x55,0xb9,0x74,0xe7,0x12,0x85,0x1a,
  0xd7,0x0a,0x07,0x2d,0x7f,0xfd,0x6e,0x00,0x4b,0x11,0x80,0xe1,0xc3,0x88,0x11,0xdb,
  0x4a,0xcc,0xd8,0xf0,0xe2,0xc6,0x89,0xe1,0x02,0x53,0x42,0xf9,0x85,0xc9,0xca,0x98,
  0x49,0xb5,0xcc,0x8c,0xf9,0x32,0xe7,0xcf,0x07,0x3f,0x7f,0x36,0x99,0xe6,0x8b,0xe9,
  0xd2,0xa8,0x4f,0xab,0x4e,0xcd,0x7a,0xf5,0x3e,0x09,0xae,0x5b,0xcb,0x8e,0x4d,0x5b,
  0x76,0x86,0xda,0xb8,0x67,0xeb,0x5e,0x9d,0xa2,0xb7,0xef,0xdf,0x28,0x22,0x00,0x00,
  0x3b
  };


CDispenserHorizontalFrame::CDispenserHorizontalFrame()
{
	m_Properties.append(new CPropertyLayout<FXHorizontalFrame>(this));
	m_Properties.append(new CPropertyFrame<FXHorizontalFrame>(this));
	m_Properties.append(new CPropertyPadTop<FXHorizontalFrame>(this));
	m_Properties.append(new CPropertyPadLeft<FXHorizontalFrame>(this));
	m_Properties.append(new CPropertyPadBottom<FXHorizontalFrame>(this));
	m_Properties.append(new CPropertyPadRight<FXHorizontalFrame>(this));
	m_Properties.append(new CPropertyLeft<FXHorizontalFrame>(this));
	m_Properties.append(new CPropertyTop<FXHorizontalFrame>(this));
	m_Properties.append(new CPropertyWidth<FXHorizontalFrame>(this));
	m_Properties.append(new CPropertyHeight<FXHorizontalFrame>(this));
}

const char* CDispenserHorizontalFrame::getWidgetName(void)
{
	return "FXHorizontalFrame";
}

const void* CDispenserHorizontalFrame::getWidgetIcon(void)
{
	return HorizontalFrame;
}

const char* CDispenserHorizontalFrame::getWidgetCategory(void)
{
	return "Containers";
}

const int CDispenserHorizontalFrame::getWidgetGrabbers(void)
{
	return GS_NONE;
}

FXint CDispenserHorizontalFrame::getDefaultWidth(void)
{
	return 80;
}

FXint CDispenserHorizontalFrame::getDefaultHeight(void)
{
	return 16;
}

FXint CDispenserHorizontalFrame::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserHorizontalFrame::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserHorizontalFrame::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserHorizontalFrame::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserHorizontalFrame::getClassInfo(void)
{
	return &FXHorizontalFrame::metaClass;
}

FXbool CDispenserHorizontalFrame::allowChild(const FXMetaClass*)
{
	return TRUE;
}

FXbool CDispenserHorizontalFrame::allowParent(FXWindow*)
{
	return TRUE;
}

FXWindow* CDispenserHorizontalFrame::getCreateParent(FXWindow* pParent)
{
	return pParent;
}

FXbool CDispenserHorizontalFrame::IsValidWidget(FXWindow** ppWindow)
{
	return (*ppWindow)->isMemberOf(&FXHorizontalFrame::metaClass);
}

IDispenser* CDispenserHorizontalFrame::clone(void)
{
	return new CDispenserHorizontalFrame;
}

FXWindow* CDispenserHorizontalFrame::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	return new FXHorizontalFrame(p, LAYOUT_FILL_X|LAYOUT_FILL_Y);
}

void CDispenserHorizontalFrame::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	FXString strOptions;

	getProperty("Layout").getOptions(pWindow, strOptions);
	getProperty("Frame").getOptions(pWindow, strOptions);
	getProperty("Justify").getOptions(pWindow, strOptions);
	getProperty("Style").getOptions(pWindow, strOptions);

	stream << "\t" << widget << " = new FXHorizontalFrame(" << parent << ", ";

	stream << strOptions << ", " << getProperty("Left").get(pWindow) << ", " << getProperty("Top").get(pWindow) << ", ";
	stream << getProperty("Width").get(pWindow) << ", " << getProperty("Height").get(pWindow);

	stream << ", " << getProperty("PadLeft").get(pWindow) << "," << getProperty("PadRight").get(pWindow) << ",";
	stream << getProperty("PadTop").get(pWindow) << "," << getProperty("PadBottom").get(pWindow) << ");\n";

	getPropertyCode(stream, pWindow, widget);
}

///////////////////////////////////////////////////////////////////////////////
//

const unsigned char VerticalFrame[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xb3,0x00,0x00,0x00,0x00,0x00,
  0x84,0x84,0x84,0xc6,0xc6,0xc6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x04,0xfe,0x30,0xc8,0x49,0xab,0xbd,0x38,0xeb,
  0xcd,0xbb,0xff,0x60,0x28,0x8e,0x64,0x69,0x9e,0x68,0x2a,0xac,0x6c,0xeb,0xbe,0x70,
  0x2c,0xcb,0xda,0x6c,0xdf,0x78,0x0c,0x64,0x79,0xef,0xd3,0xbb,0xcb,0x6f,0x48,0xdc,
  0x05,0x2b,0xc4,0x64,0x2f,0x70,0x44,0xb6,0x06,0xd0,0xa8,0x74,0x4a,0xad,0x5a,0xab,
  0x00,0x97,0x51,0xf8,0x54,0x26,0x99,0x2e,0x26,0xc6,0x35,0xf0,0x12,0xc1,0xad,0xad,
  0x85,0x6c,0x1e,0xa2,0x59,0x62,0x2e,0xab,0xdc,0xf6,0xbd,0x57,0x6a,0xe7,0xbc,0x6e,
  0xcf,0xb6,0xe2,0x6b,0x5d,0x7c,0x39,0x77,0x02,0x79,0x14,0x6c,0x83,0x38,0x85,0x80,
  0x7a,0x2b,0x74,0x8a,0x36,0x85,0x87,0x13,0x89,0x91,0x33,0x8c,0x4d,0x88,0x82,0x97,
  0x40,0x5a,0x8d,0x95,0x9c,0x9d,0x30,0x99,0x63,0x4f,0x29,0x21,0x7e,0x2c,0x94,0x12,
  0x5a,0x00,0xaf,0xb0,0xb1,0xb2,0xb3,0xb4,0xb5,0xb4,0x61,0x9a,0xa1,0xa3,0x49,0xac,
  0x01,0x64,0x57,0xc0,0xc1,0xc2,0xaa,0x2b,0xa0,0xba,0x8f,0xbb,0x31,0x93,0xc6,0xbe,
  0xa2,0xc9,0x70,0xc4,0x02,0xcc,0xcd,0x7b,0xcf,0xb8,0x9f,0xb9,0xd4,0xc8,0xd6,0x7f,
  0xd1,0xd3,0x96,0xdc,0xd2,0xd1,0xbd,0xe0,0xdc,0xa5,0x72,0xdb,0xe1,0xe2,0xd8,0xe8,
  0x02,0x90,0xe1,0xe7,0x81,0xd5,0xea,0xcb,0xd9,0xe5,0xd6,0xf1,0x8e,0xee,0xea,0xc5,
  0xe3,0xcc,0x64,0xa8,0x40,0x78,0xcb,0xa6,0xcd,0x90,0xad,0x83,0x08,0x13,0xfa,0xb3,
  0xc7,0x6f,0x11,0xc1,0x5f,0xc2,0x22,0x4a,0x84,0xb2,0xb0,0xdd,0x3b,0x73,0x03,0x4d,
  0xcd,0x83,0x57,0x51,0x5e,0x3a,0x8e,0x64,0xb8,0x34,0x7e,0xc4,0xc8,0xce,0xe3,0x3e,
  0x7e,0xf9,0x36,0x6d,0x24,0x99,0xe6,0x9f,0x33,0x7c,0x19,0x2d,0x36,0xac,0x27,0x13,
  0x65,0x4c,0x93,0x17,0x61,0x96,0xd4,0x97,0xf3,0x59,0xca,0x63,0xee,0x02,0x7e,0xe8,
  0xa8,0x4f,0xa1,0xd1,0xa3,0xaf,0x42,0xb6,0x6b,0xa8,0xc3,0x25,0x53,0x4c,0x0f,0x9f,
  0xce,0x20,0x27,0x15,0x88,0xc8,0xaa,0x3b,0x2b,0x6c,0xd9,0x2a,0x86,0xab,0xd7,0xae,
  0x60,0xbf,0x8a,0x05,0x9b,0x61,0xac,0xd9,0xb0,0x68,0xcf,0x8e,0x2d,0x9b,0xb6,0xad,
  0x5a,0xb7,0x70,0x09,0x0a,0x9d,0x4b,0xb7,0x6e,0x85,0x08,0x00,0x00,0x3b
  };

CDispenserVerticalFrame::CDispenserVerticalFrame()
{
	m_Properties.append(new CPropertyLayout<FXVerticalFrame>(this));
	m_Properties.append(new CPropertyFrame<FXVerticalFrame>(this));
	m_Properties.append(new CPropertyPadTop<FXVerticalFrame>(this));
	m_Properties.append(new CPropertyPadLeft<FXVerticalFrame>(this));
	m_Properties.append(new CPropertyPadBottom<FXVerticalFrame>(this));
	m_Properties.append(new CPropertyPadRight<FXVerticalFrame>(this));
	m_Properties.append(new CPropertyLeft<FXVerticalFrame>(this));
	m_Properties.append(new CPropertyTop<FXVerticalFrame>(this));
	m_Properties.append(new CPropertyWidth<FXVerticalFrame>(this));
	m_Properties.append(new CPropertyHeight<FXVerticalFrame>(this));
}

const char* CDispenserVerticalFrame::getWidgetName(void)
{
	return "FXVerticalFrame";
}

const void* CDispenserVerticalFrame::getWidgetIcon(void)
{
	return VerticalFrame;
}

const char* CDispenserVerticalFrame::getWidgetCategory(void)
{
	return "Containers";
}

const int CDispenserVerticalFrame::getWidgetGrabbers(void)
{
	return GS_NONE;
}

FXint CDispenserVerticalFrame::getDefaultWidth(void)
{
	return 80;
}

FXint CDispenserVerticalFrame::getDefaultHeight(void)
{
	return 16;
}

FXint CDispenserVerticalFrame::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserVerticalFrame::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserVerticalFrame::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserVerticalFrame::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserVerticalFrame::getClassInfo(void)
{
	return &FXVerticalFrame::metaClass;
}

FXbool CDispenserVerticalFrame::allowChild(const FXMetaClass*)
{
	return TRUE;
}

FXbool CDispenserVerticalFrame::allowParent(FXWindow*)
{
	return TRUE;
}

FXWindow* CDispenserVerticalFrame::getCreateParent(FXWindow* pParent)
{
	return pParent;
}

FXbool CDispenserVerticalFrame::IsValidWidget(FXWindow** ppWindow)
{
	return (*ppWindow)->isMemberOf(&FXVerticalFrame::metaClass);
}

IDispenser* CDispenserVerticalFrame::clone(void)
{
	return new CDispenserVerticalFrame;
}

FXWindow* CDispenserVerticalFrame::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	return new FXVerticalFrame(p, LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);
}

void CDispenserVerticalFrame::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	FXString strOptions;

	getProperty("Layout").getOptions(pWindow, strOptions);
	getProperty("Frame").getOptions(pWindow, strOptions);
	getProperty("Justify").getOptions(pWindow, strOptions);
	getProperty("Style").getOptions(pWindow, strOptions);

	stream << "\t" << widget << " = new FXVerticalFrame(" << parent << ", ";

	stream << strOptions << ", " << getProperty("Left").get(pWindow) << ", " << getProperty("Top").get(pWindow) << ", ";
	stream << getProperty("Width").get(pWindow) << ", " << getProperty("Height").get(pWindow);

	stream  << ", " << getProperty("PadLeft").get(pWindow) << "," << getProperty("PadRight").get(pWindow) << ",";
	stream << getProperty("PadTop").get(pWindow) << "," << getProperty("PadBottom").get(pWindow) << ");\n";

	getPropertyCode(stream, pWindow, widget);
}

