/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>
#include "CDispenserLabel.h"

#include <CPropertyBackColor.h>
#include <CPropertyTextColor.h>
#include <CPropertyLeft.h>
#include <CPropertyJustify.h>
#include <CPropertyLayout.h>
#include <CPropertyWidth.h>
#include <CPropertyHeight.h>
#include <CPropertyFrame.h>
#include <CPropertyText.h>
#include <CPropertyTipText.h>
#include <CPropertyTop.h>

#include <CVariant.h>

const unsigned char Label[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xb3,0x00,0x00,0x00,0x00,0x00,
  0x84,0x84,0x84,0xc6,0xc6,0xc6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x04,0xfe,0x30,0xc8,0x49,0xab,0xbd,0x38,0xeb,
  0xcd,0xbb,0xff,0x60,0x28,0x8e,0x64,0x69,0x9e,0x68,0xaa,0xae,0x6c,0xeb,0xbe,0x70,
  0x2c,0xcf,0x74,0x6d,0xdf,0x78,0x28,0xec,0x7c,0xef,0xff,0xc0,0xa0,0x50,0xa8,0x19,
  0x1a,0x8f,0xc8,0x60,0x31,0xc9,0x6c,0x2a,0x33,0xce,0x28,0xd1,0xc2,0x5b,0x4a,0xaf,
  0x55,0x80,0x76,0x5b,0x85,0x62,0xbf,0x01,0xc0,0x60,0x3c,0xee,0x62,0x78,0x5a,0xf4,
  0x17,0x19,0x26,0x97,0x77,0x56,0xae,0x00,0xb0,0x3e,0xb6,0xdd,0xe6,0x0b,0x9a,0x4b,
  0x9f,0xcb,0xb7,0x69,0x72,0x7e,0x69,0x43,0x77,0x64,0x79,0x54,0x3b,0x74,0x80,0x3e,
  0x81,0x7d,0x84,0x8e,0x8a,0x7d,0x4a,0x62,0x78,0x70,0x5e,0x73,0x92,0x8f,0x8c,0x90,
  0x9b,0x83,0x82,0x40,0x86,0x6f,0x02,0x71,0x7b,0x7e,0x92,0xa7,0xa9,0x91,0x6c,0x95,
  0x87,0x97,0x67,0x8a,0x3d,0x8b,0x9e,0x9d,0xa8,0x84,0x76,0xad,0xa3,0xa5,0x6a,0x9f,
  0xb3,0xa8,0xaa,0x8c,0x44,0xb9,0x03,0x88,0x15,0x75,0x57,0x01,0x53,0xb0,0xc7,0x4e,
  0x6e,0xae,0xa4,0x98,0xcc,0xc7,0x56,0xd2,0x75,0xd4,0xd5,0x60,0xd1,0xd8,0x58,0xd7,
  0xdb,0x52,0xdd,0xde,0xa1,0x89,0xd0,0xcb,0xe1,0xc2,0x80,0x93,0xe0,0xa6,0xde,0xa2,
  0xc4,0xaf,0x7a,0x9a,0xb5,0xec,0xc3,0xc5,0x14,0xeb,0xab,0xdb,0xed,0xf5,0x13,0xf7,
  0xb3,0x82,0xc1,0xe8,0xf6,0xdc,0xf2,0xa1,0xef,0xdd,0xb8,0x5a,0x08,0x69,0x29,0xfc,
  0x45,0x90,0x9e,0x41,0x63,0xf1,0x3c,0x9d,0x4a,0x68,0x2b,0xe0,0x8f,0x82,0xe4,0xe0,
  0xa5,0xea,0x07,0xac,0xe3,0xc0,0x50,0x43,0x0e,0x33,0x1e,0x9c,0xf4,0x6b,0x95,0x49,
  0x5f,0x85,0x42,0xc6,0x21,0xd9,0xf1,0x53,0xc9,0x4d,0xa0,0xb2,0x38,0xdb,0x27,0xc1,
  0x9c,0x91,0x64,0x4f,0xca,0xd9,0xfc,0xe1,0x4c,0x97,0xb6,0x9d,0x49,0xd4,0x01,0x2d,
  0xf4,0x73,0xe8,0xcd,0x1c,0x48,0x93,0x2a,0x5d,0xca,0xb4,0xa9,0xd3,0xa7,0x50,0xa3,
  0x4a,0x9d,0x4a,0xb5,0xaa,0xd5,0xab,0x58,0x33,0x44,0x00,0x00,0x3b
  };


CDispenserLabel::CDispenserLabel()
{
	m_Properties.append(new CPropertyLayout<FXLabel>(this));
	m_Properties.append(new CPropertyFrame<FXLabel>(this));
	m_Properties.append(new CPropertyJustify<FXLabel>(this));
	m_Properties.append(new CPropertyBackColor<FXLabel, FXRGB(192, 192, 192)>(this));
	m_Properties.append(new CPropertyTextColor<FXLabel, FXRGB(0, 0, 0)>(this));
	m_Properties.append(new CPropertyText<FXLabel>(this));
	m_Properties.append(new CPropertyTipText<FXLabel>(this));
	m_Properties.append(new CPropertyLeft<FXLabel>(this));
	m_Properties.append(new CPropertyTop<FXLabel>(this));
	m_Properties.append(new CPropertyWidth<FXLabel>(this));
	m_Properties.append(new CPropertyHeight<FXLabel>(this));
}

const char* CDispenserLabel::getWidgetName(void)
{
	return "FXLabel";
}

const void* CDispenserLabel::getWidgetIcon(void)
{
	return Label;
}

const char* CDispenserLabel::getWidgetCategory(void)
{
	return "Standard";
}

const int   CDispenserLabel::getWidgetGrabbers(void)
{
	return GS_ALL;
}

FXint CDispenserLabel::getDefaultWidth(void)
{
	return 80;
}

FXint CDispenserLabel::getDefaultHeight(void)
{
	return 16;
}

FXint CDispenserLabel::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserLabel::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserLabel::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserLabel::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserLabel::getClassInfo(void)
{
	return &FXLabel::metaClass;
}

FXbool CDispenserLabel::allowChild(const FXMetaClass*)
{
	return FALSE;
}

FXbool CDispenserLabel::allowParent(FXWindow*)
{
	return TRUE;
}

FXWindow* CDispenserLabel::getCreateParent(FXWindow* pParent)
{
	return pParent;
}

FXbool CDispenserLabel::IsValidWidget(FXWindow** ppWindow)
{
	return (*ppWindow)->isMemberOf(&FXLabel::metaClass);
}

IDispenser* CDispenserLabel::clone(void)
{
	return new CDispenserLabel;
}

FXWindow* CDispenserLabel::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	return new FXLabel(p, "", 0, 
				LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT);
}

void CDispenserLabel::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	FXString strOptions;

	getProperty("Frame").getOptions(pWindow, strOptions);
	getProperty("Layout").getOptions(pWindow, strOptions);
	getProperty("Justify").getOptions(pWindow, strOptions);

	stream << "\t" << widget << " = new FXLabel(" << parent << ", \"Caption\", 0, ";
	stream << strOptions << ", " << pWindow->getX() << ", " << pWindow->getY() << ", ";
	stream << pWindow->getWidth() << ", " << pWindow->getHeight() << ");\n";

	getPropertyCode(stream, pWindow, widget);
}

