/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>
#include "CDispenserScrollbar.h"

#include <CPropertyBackColor.h>
#include <CPropertyTop.h>
#include <CPropertyLeft.h>
#include <CPropertyText.h>
#include <CPropertyJustify.h>
#include <CPropertyLayout.h>
#include <CPropertyWidth.h>
#include <CPropertyHeight.h>
#include <CPropertyTipText.h>
#include <CPropertyRange.h>

#include <CVariant.h>

///////////////////////////////////////////////////////////////////////////////
//

template <class T>
class CPropertyHiliteColor : public CProperty
{
public:
	CPropertyHiliteColor(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	virtual FXString getImplementationCode(FXWindow* pWindow, FXString& widget);
};

template <class T>
const char* CPropertyHiliteColor<T>::name(void)
{
	return "HiliteColor";
}

template <class T>
ePropertyType CPropertyHiliteColor<T>::type(void)
{
	return PROPERTY_COLOR;
}

template <class T>
const char* CPropertyHiliteColor<T>::description(void)
{
	return "The hilite color of this object";
}

template <class T>
const char* CPropertyHiliteColor<T>::tooltip(void)
{
	return "getHiliteColor()\nsetHiliteColor()";
}

template <class T>
void CPropertyHiliteColor<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setHiliteColor( getColorFromString(strValue) );
}

template <class T>
FXString CPropertyHiliteColor<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromColor( p->getHiliteColor() );
}

template <class T>
FXString CPropertyHiliteColor<T>::getImplementationCode(FXWindow* pWindow, FXString& widget)
{
	T* p = static_cast<T*>(pWindow);
	FXString str = getFXRGBString(p->getHiliteColor());

	if ( str != "FXRGB(255, 255, 255)" )
	{
		return "\t" + widget + "->setHiliteColor(" + str + ");\n";
	}

	return "";
}

///////////////////////////////////////////////////////////////////////////////
//

template <class T>
class CPropertyShadowColor : public CProperty
{
public:
	CPropertyShadowColor(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	virtual FXString getImplementationCode(FXWindow* pWindow, FXString& widget);
};

template <class T>
const char* CPropertyShadowColor<T>::name(void)
{
	return "ShadowColor";
}

template <class T>
ePropertyType CPropertyShadowColor<T>::type(void)
{
	return PROPERTY_COLOR;
}

template <class T>
const char* CPropertyShadowColor<T>::description(void)
{
	return "The shadow color of this object";
}

template <class T>
const char* CPropertyShadowColor<T>::tooltip(void)
{
	return "getShadowColor()\nsetShadowColor()";
}

template <class T>
void CPropertyShadowColor<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setShadowColor( getColorFromString(strValue) );
}

template <class T>
FXString CPropertyShadowColor<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromColor( p->getShadowColor() );
}

template <class T>
FXString CPropertyShadowColor<T>::getImplementationCode(FXWindow* pWindow, FXString& widget)
{
	T* p = static_cast<T*>(pWindow);
	FXString str = getFXRGBString(p->getShadowColor());

	if ( str != "FXRGB(128, 128, 128)" )
	{
		return "\t" + widget + "->setShadowColor(" + str + ");\n";
	}

	return "";
}

///////////////////////////////////////////////////////////////////////////////
//

template <class T>
class CPropertyBorderColor : public CProperty
{
public:
	CPropertyBorderColor(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	virtual FXString getImplementationCode(FXWindow* pWindow, FXString& widget);
};

template <class T>
const char* CPropertyBorderColor<T>::name(void)
{
	return "BorderColor";
}

template <class T>
ePropertyType CPropertyBorderColor<T>::type(void)
{
	return PROPERTY_COLOR;
}

template <class T>
const char* CPropertyBorderColor<T>::description(void)
{
	return "The border color of this object";
}

template <class T>
const char* CPropertyBorderColor<T>::tooltip(void)
{
	return "getBorderColor()\nsetBorderColor()";
}

template <class T>
void CPropertyBorderColor<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setBorderColor( getColorFromString(strValue) );
}

template <class T>
FXString CPropertyBorderColor<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromColor( p->getBorderColor() );
}

template <class T>
FXString CPropertyBorderColor<T>::getImplementationCode(FXWindow* pWindow, FXString& widget)
{
	T* p = static_cast<T*>(pWindow);
	FXString str = getFXRGBString(p->getBorderColor());

	if ( str != "FXRGB(0, 0, 0)" )
	{
		return "\t" + widget + "->setBorderColor(" + str + ");\n";
	}

	return "";
}

///////////////////////////////////////////////////////////////////////////////
//

const unsigned char VScroll[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xb3,0x00,0x00,0x00,0x00,0x00,
  0x84,0x84,0x84,0xc6,0xc6,0xc6,0xce,0xce,0xce,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x04,0xfe,0x30,0xc8,0x49,0xab,0xbd,0x38,0xeb,
  0xcd,0xbb,0xff,0x60,0x28,0x8e,0x64,0x69,0x9e,0x68,0xaa,0xae,0x6c,0x2b,0x02,0x03,
  0x2c,0xc7,0xf4,0x6c,0xd7,0xf8,0xad,0xe7,0xb8,0x16,0x07,0x00,0xe0,0x2f,0x48,0x1c,
  0x02,0x85,0xc4,0xa3,0x4c,0x69,0x2c,0x26,0x9f,0x1a,0x22,0x60,0x4a,0xad,0x5a,0xaf,
  0xd8,0xac,0x96,0x4a,0xf3,0x51,0x09,0x00,0xb0,0x38,0x4c,0x1e,0x9b,0xcb,0xe8,0xf1,
  0x60,0xcd,0x6e,0xaf,0xa7,0xe0,0x28,0xf0,0x4c,0x4f,0xdb,0xeb,0x84,0x01,0x61,0xcf,
  0xef,0x03,0x05,0x30,0x5e,0x70,0x77,0x84,0x78,0x76,0x7a,0x6e,0x6d,0x40,0x64,0x72,
  0x85,0x8e,0x86,0x66,0x88,0x89,0x6b,0x4a,0x02,0x82,0x8f,0x98,0x86,0x92,0x6f,0x8a,
  0x53,0x47,0x19,0x44,0x90,0x99,0x85,0x9b,0x5c,0x94,0x49,0x97,0xa2,0xaa,0x84,0x92,
  0x56,0xa7,0xa8,0x18,0xa1,0xa3,0xab,0x91,0x79,0x58,0x03,0x4a,0x1b,0x31,0x83,0xb4,
  0xb3,0x30,0x79,0x93,0xb8,0x9e,0x41,0xa0,0x73,0xbe,0xbd,0x79,0xc0,0x93,0xb9,0xa9,
  0xc7,0xa3,0xb8,0x51,0xb0,0x17,0xb2,0xc8,0xbe,0x5b,0xc3,0xba,0x5f,0xce,0xab,0xc1,
  0x6d,0xd8,0xc5,0xdb,0xb3,0x7a,0x7d,0xe4,0xcc,0x19,0xbb,0xe1,0xdc,0xca,0x89,0x8b,
  0x9f,0xb1,0xc6,0xd5,0xb4,0x9b,0xec,0xd2,0x17,0xe8,0xf1,0xcf,0xeb,0x6e,0xed,0xc4,
  0xef,0xe9,0x99,0xf3,0xf6,0xd5,0xb3,0x70,0xef,0x9f,0x26,0x7d,0x9d,0x06,0x52,0xa0,
  0x66,0xf0,0x0e,0x34,0x70,0xd9,0x78,0x35,0x3c,0x73,0x8d,0x8a,0xbb,0x69,0xf0,0x26,
  0xa6,0x41,0x94,0x8c,0x23,0xa2,0x8b,0xc7,0xf6,0xb4,0xe1,0x53,0xe5,0xb1,0x63,0x32,
  0x85,0x13,0x18,0x8e,0x24,0x65,0xb2,0xa4,0xb0,0x88,0x1a,0xf1,0xb8,0x34,0x89,0x52,
  0x82,0xca,0x98,0x61,0x66,0x7a,0x04,0x49,0x50,0x24,0x4e,0x31,0x3a,0x4f,0xf2,0x5c,
  0x98,0x71,0xa5,0xcc,0x96,0x2d,0x87,0x52,0x28,0x68,0x94,0x65,0xd0,0x9a,0x4a,0x9a,
  0x42,0x0a,0xfa,0xb1,0x1f,0x06,0xa6,0x3f,0x73,0x22,0xe5,0x08,0xf5,0xa6,0xd4,0x32,
  0x54,0xf3,0x28,0x9d,0x80,0xf5,0x6b,0xad,0xa7,0x63,0xa3,0x66,0x05,0xba,0xb5,0x63,
  0xda,0x00,0x65,0x71,0x76,0x63,0xf3,0xcd,0x9f,0x59,0x87,0xe4,0xca,0x41,0x85,0xeb,
  0xf3,0x2e,0x98,0x80,0x09,0xd3,0x7a,0x8d,0x89,0xe8,0x96,0xb9,0xab,0x7d,0xe5,0x02,
  0x73,0x25,0xac,0x6b,0xd1,0x9f,0xa5,0xa6,0xb0,0x39,0x1c,0x52,0xa2,0xdf,0x5f,0xde,
  0x02,0x5b,0xb5,0x30,0xd8,0x2c,0xe0,0xc9,0x7b,0xe3,0xde,0xfd,0xfc,0x4a,0xf0,0x63,
  0xbf,0x0f,0xfd,0xc1,0xbc,0x3c,0xa6,0x62,0x5d,0x8c,0x3b,0x62,0xf3,0x90,0xbd,0x04,
  0x8a,0xed,0x6c,0xac,0xed,0xe4,0xba,0xbd,0xb9,0x02,0xed,0xd9,0xc0,0x7f,0xf3,0xa6,
  0xec,0xa2,0xb8,0xf1,0xe3,0xc8,0x93,0x2b,0x5f,0xce,0xbc,0xb9,0xf3,0x08,0x00,0x00,
  0x3b
  };


CDispenserVScrollbar::CDispenserVScrollbar()
{
	m_Properties.append(new CPropertyLayout<FXScrollbar>(this));
	m_Properties.append(new CPropertyBackColor<FXScrollbar, FXRGB(192, 192, 192)>(this));
	m_Properties.append(new CPropertyHiliteColor<FXScrollbar>(this));
	m_Properties.append(new CPropertyShadowColor<FXScrollbar>(this));
	m_Properties.append(new CPropertyBorderColor<FXScrollbar>(this));
	m_Properties.append(new CPropertyRange<FXScrollbar, 100>(this));
	m_Properties.append(new CPropertyLeft<FXScrollbar>(this));
	m_Properties.append(new CPropertyTop<FXScrollbar>(this));
	m_Properties.append(new CPropertyWidth<FXScrollbar>(this));
	m_Properties.append(new CPropertyHeight<FXScrollbar>(this));
}

const char* CDispenserVScrollbar::getWidgetName(void)
{
	return "FXScrollbar";
}

const void* CDispenserVScrollbar::getWidgetIcon(void)
{
	return VScroll;
}

const char* CDispenserVScrollbar::getWidgetCategory(void)
{
	return "Standard";
}

const int CDispenserVScrollbar::getWidgetGrabbers(void)
{
	return GS_TOP_CENTRE|GS_BOTTOM_CENTRE;
}

FXint CDispenserVScrollbar::getDefaultWidth(void)
{
	return 16;
}

FXint CDispenserVScrollbar::getDefaultHeight(void)
{
	return 40;
}

FXint CDispenserVScrollbar::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserVScrollbar::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserVScrollbar::getMaximumWidth(void)
{
	return 16;
}

FXint CDispenserVScrollbar::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserVScrollbar::getClassInfo(void)
{
	return &FXScrollbar::metaClass;
}

FXbool CDispenserVScrollbar::allowChild(const FXMetaClass*)
{
	return FALSE;
}

FXbool CDispenserVScrollbar::allowParent(FXWindow*)
{
	return TRUE;
}

FXWindow* CDispenserVScrollbar::getCreateParent(FXWindow* pParent)
{
	return pParent;
}

FXbool CDispenserVScrollbar::IsValidWidget(FXWindow** ppWindow)
{
	return (*ppWindow)->isMemberOf(&FXScrollbar::metaClass);
}

IDispenser* CDispenserVScrollbar::clone(void)
{
	return new CDispenserVScrollbar;
}

FXWindow* CDispenserVScrollbar::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	return new FXScrollbar(p, tgt, sel, 
				SCROLLBAR_VERTICAL|LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT);
}

void CDispenserVScrollbar::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	FXString strOptions;

	getProperty("Layout").getOptions(pWindow, strOptions);

	stream << "\t" << widget << " = new FXScrollbar(" << parent << ", 0, 0, ";
	stream << "SCROLLBAR_VERTICAL|" << strOptions << ", " << pWindow->getX() << ", " << pWindow->getY() << ", ";
	stream << pWindow->getWidth() << ", " << pWindow->getHeight() << ");\n";

	getPropertyCode(stream, pWindow, widget);
}

///////////////////////////////////////////////////////////////////////////////
//

const unsigned char HScroll[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xb3,0x00,0x00,0x00,0x00,0x00,
  0x84,0x84,0x84,0xc6,0xc6,0xc6,0xce,0xce,0xce,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x04,0xfe,0x30,0xc8,0x49,0xab,0xbd,0x38,0xeb,
  0xcd,0xbb,0xff,0x60,0x28,0x8e,0x64,0x69,0x9e,0x68,0xaa,0xae,0x6c,0x2b,0x02,0x03,
  0x2c,0xc7,0xf4,0x6c,0xd7,0xf8,0xad,0xe7,0xb8,0x16,0x07,0x00,0xa0,0x30,0x48,0x1c,
  0x1a,0x8b,0xc8,0xa3,0x32,0x09,0x20,0x04,0x33,0x44,0x80,0x74,0x4a,0xad,0x5a,0xaf,
  0xd8,0xec,0x94,0xe6,0x9b,0x3a,0xbf,0xcd,0x30,0x78,0x2c,0x2e,0x93,0xcf,0xe6,0xb0,
  0x86,0x88,0x6e,0xa7,0xdd,0xf0,0x30,0x37,0x13,0x93,0xc6,0xef,0xef,0x3c,0x7a,0x0d,
  0xc4,0xfb,0xf5,0x79,0x73,0x18,0x75,0x80,0x7f,0x86,0x69,0x7c,0x85,0x8a,0x87,0x64,
  0x82,0x17,0x84,0x8c,0x8b,0x92,0x4e,0x89,0x91,0x96,0x69,0x8e,0x16,0x90,0x93,0x9c,
  0x7f,0x95,0x9d,0xa0,0x6e,0x99,0x15,0x9b,0x97,0xa6,0x6a,0x50,0x7d,0xa1,0xab,0x8d,
  0x30,0x5d,0x76,0xac,0xb1,0x60,0x9f,0xa7,0x97,0xa3,0x14,0xa5,0xb2,0xb1,0xb4,0xba,
  0x91,0xb7,0x13,0xb9,0xb5,0xab,0xbc,0xc2,0x87,0xbf,0x12,0xc1,0xbd,0xa0,0xc4,0xca,
  0x7f,0xc7,0x01,0xc9,0xc5,0x9c,0xcc,0xd2,0x78,0xcf,0xd1,0xcd,0x92,0xd4,0xd9,0x71,
  0xd7,0x5e,0xdc,0xcb,0xa9,0xe0,0xc6,0xae,0x74,0xdf,0xd5,0xe1,0x18,0x6c,0xe3,0x92,
  0xde,0xb0,0xe8,0x87,0xdb,0xf0,0x81,0xe5,0x83,0xe7,0xec,0x78,0xf2,0xf8,0x65,0xee,
  0x02,0xfe,0xff,0x00,0x05,0x00,0x08,0xf8,0x6f,0x20,0x01,0x01,0x07,0x13,0x22,0x5c,
  0xa8,0x90,0xa0,0xbf,0x3a,0x94,0xc4,0x21,0x24,0x40,0xb1,0x22,0x45,0x20,0x13,0x2d,
  0xf6,0x61,0xc8,0x51,0x61,0x42,0x8b,0xbd,0x1a,0x89,0x0c,0x78,0xb5,0xd0,0x21,0xc6,
  0x83,0x00,0x4f,0x76,0x5c,0x49,0xd1,0xa1,0x00,0x20,0x45,0x24,0xa2,0xf4,0x37,0xd0,
  0x9f,0xca,0x94,0x4d,0x58,0xea,0xa4,0x89,0x53,0x08,0xc9,0x99,0x52,0xfe,0xdd,0x14,
  0x6a,0x50,0xa7,0x47,0xa0,0x35,0x5f,0x4a,0xf1,0xa9,0xee,0xe6,0x14,0xa2,0x25,0x89,
  0x7a,0x34,0x1a,0xf5,0xa9,0x52,0xa6,0xf6,0x8a,0x52,0x81,0x3a,0xd3,0x66,0xd1,0xa9,
  0x60,0x91,0x06,0x85,0x89,0xd5,0x02,0x91,0x92,0x56,0x95,0x46,0xf5,0x0a,0x96,0x2a,
  0xcd,0xa0,0x57,0x9f,0x98,0x2b,0xfa,0x96,0x6b,0x40,0x95,0x6d,0x8f,0x16,0x4c,0x4a,
  0x56,0xee,0x85,0xb3,0x5d,0xf9,0xd2,0x95,0x4a,0x75,0x2a,0x4f,0xa2,0x65,0x35,0x05,
  0xed,0x8a,0x73,0xad,0xda,0xbc,0x1d,0x19,0x13,0x8d,0xd9,0x74,0xe0,0x9a,0x97,0x66,
  0x21,0x4f,0x4d,0xb4,0x81,0x90,0x16,0x2d,0x85,0x19,0x7e,0xa6,0xfc,0x17,0xc8,0x0f,
  0x26,0x4b,0x52,0xa3,0x5e,0xad,0xa4,0x0b,0x25,0xd6,0xb0,0x55,0xcb,0x4e,0xb2,0x41,
  0xc7,0xec,0xdb,0xb1,0x51,0x27,0x76,0xc1,0xbb,0xb7,0xef,0xdf,0xc0,0x83,0x0b,0x1f,
  0x4e,0x7c,0x78,0x04,0x00,0x00,0x3b
  };

CDispenserHScrollbar::CDispenserHScrollbar()
{
	m_Properties.append(new CPropertyLayout<FXScrollbar>(this));
	m_Properties.append(new CPropertyBackColor<FXScrollbar, FXRGB(192, 192, 192)>(this));
	m_Properties.append(new CPropertyHiliteColor<FXScrollbar>(this));
	m_Properties.append(new CPropertyShadowColor<FXScrollbar>(this));
	m_Properties.append(new CPropertyBorderColor<FXScrollbar>(this));
	m_Properties.append(new CPropertyRange<FXScrollbar, 100>(this));
	m_Properties.append(new CPropertyLeft<FXScrollbar>(this));
	m_Properties.append(new CPropertyTop<FXScrollbar>(this));
	m_Properties.append(new CPropertyWidth<FXScrollbar>(this));
	m_Properties.append(new CPropertyHeight<FXScrollbar>(this));
}

const char* CDispenserHScrollbar::getWidgetName(void)
{
	return "FXScrollbar";
}

const void* CDispenserHScrollbar::getWidgetIcon(void)
{
	return HScroll;
}

const char* CDispenserHScrollbar::getWidgetCategory(void)
{
	return "Standard";
}

const int CDispenserHScrollbar::getWidgetGrabbers(void)
{
	return GS_CENTRE_LEFT|GS_CENTRE_RIGHT;
}

FXint CDispenserHScrollbar::getDefaultWidth(void)
{
	return 80;
}

FXint CDispenserHScrollbar::getDefaultHeight(void)
{
	return 16;
}

FXint CDispenserHScrollbar::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserHScrollbar::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserHScrollbar::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserHScrollbar::getMaximumHeight(void)
{
	return 16;
}

const FXMetaClass* CDispenserHScrollbar::getClassInfo(void)
{
	return &FXScrollbar::metaClass;
}

FXbool CDispenserHScrollbar::allowChild(const FXMetaClass*)
{
	return FALSE;
}

FXbool CDispenserHScrollbar::allowParent(FXWindow*)
{
	return TRUE;
}

FXWindow* CDispenserHScrollbar::getCreateParent(FXWindow* pParent)
{
	return pParent;
}

FXbool CDispenserHScrollbar::IsValidWidget(FXWindow** ppWindow)
{
	return (*ppWindow)->isMemberOf(&FXScrollbar::metaClass);
}

IDispenser* CDispenserHScrollbar::clone(void)
{
	return new CDispenserHScrollbar;
}

FXWindow* CDispenserHScrollbar::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	return new FXScrollbar(p, tgt, sel, 
				SCROLLBAR_HORIZONTAL|LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT);
}

void CDispenserHScrollbar::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	FXString strOptions;

	getProperty("Layout").getOptions(pWindow, strOptions);

	stream << "\t" << widget << " = new FXScrollbar(" << parent << ", 0, 0, ";
	stream << "SCROLLBAR_HORIZONTAL|" << strOptions << ", " << pWindow->getX() << ", " << pWindow->getY() << ", ";
	stream << pWindow->getWidth() << ", " << pWindow->getHeight() << ");\n";

	getPropertyCode(stream, pWindow, widget);
}

