/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>
#include "CDispenserSpinner.h"

#include <CPropertyBackColor.h>
#include <CPropertyTop.h>
#include <CPropertyLeft.h>
#include <CPropertyFrame.h>
#include <CPropertyLayout.h>
#include <CPropertyWidth.h>
#include <CPropertyHeight.h>
#include <CPropertyTipText.h>
#include <CPropertyPadBottom.h>
#include <CPropertyPadLeft.h>
#include <CPropertyPadRight.h>
#include <CPropertyPadTop.h>

#include <CVariant.h>

///////////////////////////////////////////////////////////////////////////////
//

template <class T>
class CPropertySpinnerStyle : public CProperty
{
public:
	CPropertySpinnerStyle(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	OPTION* option(void);
	virtual void getOptions(FXWindow* pWindow, FXString& strOptions);

private:
	static OPTION Style[];
};

///////////////////////////////////////////////////////////////////////////////
//

// From FXSpinner.h
template <class T>
OPTION CPropertySpinnerStyle<T>::Style[] = 
{
	{"SPIN_NORMAL", SPIN_NORMAL, "Normal, non-cyclic"},
	{"SPIN_CYCLIC", SPIN_CYCLIC, "Cyclic spinner"},
	{"SPIN_NOTEXT", SPIN_NOTEXT, "No text visible"},
	{"SPIN_NOMAX",  SPIN_NOMAX,  "Spin all the way up to infinity"},
	{"SPIN_NOMIN",  SPIN_NOMIN,  "Spin all the way down to -infinity"},
	{0, 0, 0}
};

template <class T>
const char* CPropertySpinnerStyle<T>::name(void)
{
	return "Style";
}

template <class T>
ePropertyType CPropertySpinnerStyle<T>::type(void)
{
	return PROPERTY_COMBO;
}

template <class T>
const char* CPropertySpinnerStyle<T>::description(void)
{
	return "The spinner style of this object";
}

template <class T>
const char* CPropertySpinnerStyle<T>::tooltip(void)
{
	return "getSpinnerStyle()\nsetSpinnerStyle()";
}

template <class T>
void CPropertySpinnerStyle<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setSpinnerStyle( getIntFromString(strValue) );
}

template <class T>
FXString CPropertySpinnerStyle<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromInt( p->getSpinnerStyle() );
}

template <class T>
OPTION* CPropertySpinnerStyle<T>::option(void)
{
	return Style;
}

template <class T>
void CPropertySpinnerStyle<T>::getOptions(FXWindow* pWindow, FXString& strOptions)
{
	T* p = static_cast<T*>(pWindow);
	
	if ( p->getSpinnerStyle() == 0 )
		return;

	CProperty::getOptions(pWindow, strOptions);
}

///////////////////////////////////////////////////////////////////////////////
//

const unsigned char Spinner[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xf7,0x00,0x00,0x00,0x00,0x00,
  0x80,0x00,0x00,0x00,0x80,0x00,0x80,0x80,0x00,0x00,0x00,0x80,0x80,0x00,0x80,0x00,
  0x80,0x80,0xc0,0xc0,0xc0,0xc0,0xdc,0xc0,0xa6,0xca,0xf0,0x00,0x00,0x00,0x00,0x00,
  0x2a,0x00,0x00,0x55,0x00,0x00,0x7f,0x00,0x00,0xaa,0x00,0x00,0xd4,0x00,0x2a,0x00,
  0x00,0x2a,0x2a,0x00,0x2a,0x55,0x00,0x2a,0x7f,0x00,0x2a,0xaa,0x00,0x2a,0xd4,0x00,
  0x55,0x00,0x00,0x55,0x2a,0x00,0x55,0x55,0x00,0x55,0x7f,0x00,0x55,0xaa,0x00,0x55,
  0xd4,0x00,0x7f,0x00,0x00,0x7f,0x2a,0x00,0x7f,0x55,0x00,0x7f,0x7f,0x00,0x7f,0xaa,
  0x00,0x7f,0xd4,0x00,0xaa,0x00,0x00,0xaa,0x2a,0x00,0xaa,0x55,0x00,0xaa,0x7f,0x00,
  0xaa,0xaa,0x00,0xaa,0xd4,0x00,0xd4,0x00,0x00,0xd4,0x2a,0x00,0xd4,0x55,0x00,0xd4,
  0x7f,0x00,0xd4,0xaa,0x00,0xd4,0xd4,0x2a,0x00,0x00,0x2a,0x00,0x2a,0x2a,0x00,0x55,
  0x2a,0x00,0x7f,0x2a,0x00,0xaa,0x2a,0x00,0xd4,0x2a,0x2a,0x00,0x2a,0x2a,0x2a,0x2a,
  0x2a,0x55,0x2a,0x2a,0x7f,0x2a,0x2a,0xaa,0x2a,0x2a,0xd4,0x2a,0x55,0x00,0x2a,0x55,
  0x2a,0x2a,0x55,0x55,0x2a,0x55,0x7f,0x2a,0x55,0xaa,0x2a,0x55,0xd4,0x2a,0x7f,0x00,
  0x2a,0x7f,0x2a,0x2a,0x7f,0x55,0x2a,0x7f,0x7f,0x2a,0x7f,0xaa,0x2a,0x7f,0xd4,0x2a,
  0xaa,0x00,0x2a,0xaa,0x2a,0x2a,0xaa,0x55,0x2a,0xaa,0x7f,0x2a,0xaa,0xaa,0x2a,0xaa,
  0xd4,0x2a,0xd4,0x00,0x2a,0xd4,0x2a,0x2a,0xd4,0x55,0x2a,0xd4,0x7f,0x2a,0xd4,0xaa,
  0x2a,0xd4,0xd4,0x55,0x00,0x00,0x55,0x00,0x2a,0x55,0x00,0x55,0x55,0x00,0x7f,0x55,
  0x00,0xaa,0x55,0x00,0xd4,0x55,0x2a,0x00,0x55,0x2a,0x2a,0x55,0x2a,0x55,0x55,0x2a,
  0x7f,0x55,0x2a,0xaa,0x55,0x2a,0xd4,0x55,0x55,0x00,0x55,0x55,0x2a,0x55,0x55,0x55,
  0x55,0x55,0x7f,0x55,0x55,0xaa,0x55,0x55,0xd4,0x55,0x7f,0x00,0x55,0x7f,0x2a,0x55,
  0x7f,0x55,0x55,0x7f,0x7f,0x55,0x7f,0xaa,0x55,0x7f,0xd4,0x55,0xaa,0x00,0x55,0xaa,
  0x2a,0x55,0xaa,0x55,0x55,0xaa,0x7f,0x55,0xaa,0xaa,0x55,0xaa,0xd4,0x55,0xd4,0x00,
  0x55,0xd4,0x2a,0x55,0xd4,0x55,0x55,0xd4,0x7f,0x55,0xd4,0xaa,0x55,0xd4,0xd4,0x7f,
  0x00,0x00,0x7f,0x00,0x2a,0x7f,0x00,0x55,0x7f,0x00,0x7f,0x7f,0x00,0xaa,0x7f,0x00,
  0xd4,0x7f,0x2a,0x00,0x7f,0x2a,0x2a,0x7f,0x2a,0x55,0x7f,0x2a,0x7f,0x7f,0x2a,0xaa,
  0x7f,0x2a,0xd4,0x7f,0x55,0x00,0x7f,0x55,0x2a,0x7f,0x55,0x55,0x7f,0x55,0x7f,0x7f,
  0x55,0xaa,0x7f,0x55,0xd4,0x7f,0x7f,0x00,0x7f,0x7f,0x2a,0x7f,0x7f,0x55,0x7f,0x7f,
  0x7f,0x7f,0x7f,0xaa,0x7f,0x7f,0xd4,0x7f,0xaa,0x00,0x7f,0xaa,0x2a,0x7f,0xaa,0x55,
  0x7f,0xaa,0x7f,0x7f,0xaa,0xaa,0x7f,0xaa,0xd4,0x7f,0xd4,0x00,0x7f,0xd4,0x2a,0x7f,
  0xd4,0x55,0x7f,0xd4,0x7f,0x7f,0xd4,0xaa,0x7f,0xd4,0xd4,0xaa,0x00,0x00,0xaa,0x00,
  0x2a,0xaa,0x00,0x55,0xaa,0x00,0x7f,0xaa,0x00,0xaa,0xaa,0x00,0xd4,0xaa,0x2a,0x00,
  0xaa,0x2a,0x2a,0xaa,0x2a,0x55,0xaa,0x2a,0x7f,0xaa,0x2a,0xaa,0xaa,0x2a,0xd4,0xaa,
  0x55,0x00,0xaa,0x55,0x2a,0xaa,0x55,0x55,0xaa,0x55,0x7f,0xaa,0x55,0xaa,0xaa,0x55,
  0xd4,0xaa,0x7f,0x00,0xaa,0x7f,0x2a,0xaa,0x7f,0x55,0xaa,0x7f,0x7f,0xaa,0x7f,0xaa,
  0xaa,0x7f,0xd4,0xaa,0xaa,0x00,0xaa,0xaa,0x2a,0xaa,0xaa,0x55,0xaa,0xaa,0x7f,0xaa,
  0xaa,0xaa,0xaa,0xaa,0xd4,0xaa,0xd4,0x00,0xaa,0xd4,0x2a,0xaa,0xd4,0x55,0xaa,0xd4,
  0x7f,0xaa,0xd4,0xaa,0xaa,0xd4,0xd4,0xd4,0x00,0x00,0xd4,0x00,0x2a,0xd4,0x00,0x55,
  0xd4,0x00,0x7f,0xd4,0x00,0xaa,0xd4,0x00,0xd4,0xd4,0x2a,0x00,0xd4,0x2a,0x2a,0xd4,
  0x2a,0x55,0xd4,0x2a,0x7f,0xd4,0x2a,0xaa,0xd4,0x2a,0xd4,0xd4,0x55,0x00,0xd4,0x55,
  0x2a,0xd4,0x55,0x55,0xd4,0x55,0x7f,0xd4,0x55,0xaa,0xd4,0x55,0xd4,0xd4,0x7f,0x00,
  0xd4,0x7f,0x2a,0xd4,0x7f,0x55,0xd4,0x7f,0x7f,0xd4,0x7f,0xaa,0xd4,0x7f,0xd4,0xd4,
  0xaa,0x00,0xd4,0xaa,0x2a,0xd4,0xaa,0x55,0xd4,0xaa,0x7f,0xd4,0xaa,0xaa,0xd4,0xaa,
  0xd4,0xd4,0xd4,0x00,0xd4,0xd4,0x2a,0xd4,0xd4,0x55,0xd4,0xd4,0x7f,0xd4,0xd4,0xaa,
  0xd4,0xd4,0xd4,0x00,0x00,0x00,0x0c,0x0c,0x0c,0x19,0x19,0x19,0x26,0x26,0x26,0x33,
  0x33,0x33,0x3f,0x3f,0x3f,0x4c,0x4c,0x4c,0x59,0x59,0x59,0x66,0x66,0x66,0x72,0x72,
  0x72,0x7f,0x7f,0x7f,0x8c,0x8c,0x8c,0x99,0x99,0x99,0xa5,0xa5,0xa5,0xb2,0xb2,0xb2,
  0xbf,0xbf,0xbf,0xcc,0xcc,0xcc,0xd8,0xd8,0xd8,0xe5,0xe5,0xe5,0xf2,0xf2,0xf2,0xff,
  0xfb,0xf0,0xa0,0xa0,0xa4,0x80,0x80,0x80,0xff,0x00,0x00,0x00,0xff,0x00,0xff,0xff,
  0x00,0x00,0x00,0xff,0xff,0x00,0xff,0x00,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x08,0xfe,0x00,0xf1,0x09,0x1c,0x48,0xb0,0xa0,
  0xc1,0x83,0x08,0x13,0x2a,0x5c,0xc8,0xb0,0xa1,0xc3,0x87,0x10,0x23,0x4a,0x9c,0x48,
  0xb1,0xa2,0xc5,0x8b,0x18,0x33,0x6a,0xdc,0xc8,0xb1,0xa3,0xc7,0x8f,0x20,0x43,0x8a,
  0x1c,0x49,0xb2,0xa4,0xc9,0x93,0x28,0x53,0xaa,0x5c,0xc9,0xb2,0xa5,0xcb,0x97,0x30,
  0x5b,0x1e,0x98,0x49,0xb3,0xa6,0xcd,0x9b,0x38,0x73,0xea,0xd4,0xa9,0x70,0xa7,0xcf,
  0x9f,0x40,0x73,0xf6,0x3c,0x10,0xf3,0xdf,0xcc,0xa1,0xf8,0x00,0x28,0x5d,0xca,0xb4,
  0xa9,0xd3,0xa7,0x50,0xa3,0x2a,0x0d,0x67,0x94,0x68,0xc2,0xa3,0x00,0xfe,0x69,0xdd,
  0xca,0xb5,0xab,0xd7,0xaf,0x60,0xc3,0x02,0xa0,0x7a,0xf4,0x2a,0xd1,0xac,0x61,0xd3,
  0xaa,0xd5,0x1a,0xae,0xad,0xdb,0xb7,0xe1,0x92,0x92,0xb5,0x8a,0x10,0xeb,0xda,0xbb,
  0x60,0xe1,0x8e,0x7d,0x2b,0xb7,0x2a,0x52,0xb4,0x78,0x03,0xb3,0x75,0xbb,0x94,0xef,
  0x58,0xbf,0x66,0x93,0x0a,0x5e,0x4c,0x98,0xa9,0xdb,0xbe,0x65,0xeb,0x9e,0x5d,0x2c,
  0xb8,0xed,0xd3,0xb6,0x90,0xe9,0x1e,0xb4,0x4b,0xf9,0x2b,0xe0,0xad,0x70,0x43,0x67,
  0xfe,0xdb,0xd9,0xab,0x52,0xaf,0x0c,0x0f,0x47,0xde,0x3c,0xb9,0x74,0xd7,0xcf,0xff,
  0xa4,0x3a,0x46,0x2c,0x59,0xb1,0x6b,0xae,0xb0,0x43,0xeb,0x6e,0x4b,0x9b,0xb5,0xed,
  0xdb,0x5a,0x61,0xdf,0x55,0xad,0xd9,0x20,0x67,0xe0,0xb1,0xbd,0xee,0x7e,0x4c,0x9c,
  0x34,0xf2,0xe4,0x5d,0x2d,0x3b,0xc5,0xdc,0x3c,0xb1,0xf0,0xce,0x4b,0xa3,0x4b,0x2f,
  0x4c,0x7d,0xae,0xf3,0xe7,0xca,0x1b,0x53,0xef,0xed,0xde,0xdb,0x78,0x6b,0xf0,0xda,
  0x09,0xc3,0x1d,0x6d,0x1d,0x7d,0xf8,0xe5,0xec,0x6b,0x5f,0x07,0x9f,0xda,0x7b,0x7b,
  0xf7,0x5b,0x65,0x17,0x2e,0x5f,0xf0,0xe8,0xf2,0xff,0x00,0x06,0x18,0x1a,0x7f,0x04,
  0x1d,0x85,0x5f,0x69,0xab,0x99,0x17,0xd4,0x82,0x0c,0xe2,0x34,0x54,0x83,0x10,0x36,
  0x18,0xd3,0x84,0x14,0x56,0x68,0xe1,0x85,0x18,0x66,0xa8,0xe1,0x86,0x1c,0x76,0xe8,
  0xe1,0x87,0x20,0x86,0x28,0xe2,0x88,0x24,0x62,0x18,0x10,0x00,0x3b
  };

CDispenserSpinner::CDispenserSpinner()
{
	m_Properties.append(new CPropertyLayout<FXSpinner>(this));
	m_Properties.append(new CPropertyFrame<FXSpinner>(this));
	m_Properties.append(new CPropertySpinnerStyle<FXSpinner>(this));
	m_Properties.append(new CPropertyBackColor<FXSpinner, FXRGB(192, 192, 192)>(this));
	m_Properties.append(new CPropertyTipText<FXSpinner>(this));
	m_Properties.append(new CPropertyPadTop<FXSpinner>(this));
	m_Properties.append(new CPropertyPadLeft<FXSpinner>(this));
	m_Properties.append(new CPropertyPadBottom<FXSpinner>(this));
	m_Properties.append(new CPropertyPadRight<FXSpinner>(this));
	m_Properties.append(new CPropertyLeft<FXSpinner>(this));
	m_Properties.append(new CPropertyTop<FXSpinner>(this));
	m_Properties.append(new CPropertyWidth<FXSpinner>(this));
	m_Properties.append(new CPropertyHeight<FXSpinner>(this));
}

const char* CDispenserSpinner::getWidgetName(void)
{
	return "FXSpinner";
}

const void* CDispenserSpinner::getWidgetIcon(void)
{
	return Spinner;
}

const char* CDispenserSpinner::getWidgetCategory(void)
{
	return "Extended";
}

const int CDispenserSpinner::getWidgetGrabbers(void)
{
	return GS_ALL;
}

FXint CDispenserSpinner::getDefaultWidth(void)
{
	return 80;
}

FXint CDispenserSpinner::getDefaultHeight(void)
{
	return 40;
}

FXint CDispenserSpinner::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserSpinner::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserSpinner::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserSpinner::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserSpinner::getClassInfo(void)
{
	return &FXSpinner::metaClass;
}

FXbool CDispenserSpinner::allowChild(const FXMetaClass*)
{
	return FALSE;
}

FXbool CDispenserSpinner::allowParent(FXWindow*)
{
	return TRUE;
}

FXWindow* CDispenserSpinner::getCreateParent(FXWindow* pParent)
{
	return pParent;
}

/*
	FXSpinner
	  FXTextField
	  FXArrowButton
	  FXArrowButton
*/
FXbool CDispenserSpinner::IsValidWidget(FXWindow** ppWindow)
{
	if ( (*ppWindow)->isMemberOf(&FXSpinner::metaClass) )
	{
		return TRUE;
	}
	else if ( (*ppWindow)->isMemberOf(&FXTextField::metaClass) )
	{
		*ppWindow = (*ppWindow)->getParent();
		return *ppWindow && (*ppWindow)->isMemberOf(&FXSpinner::metaClass);
	}
	else if ( (*ppWindow)->isMemberOf(&FXArrowButton::metaClass) )
	{
		*ppWindow = (*ppWindow)->getParent();
		return *ppWindow && (*ppWindow)->isMemberOf(&FXSpinner::metaClass);
	}

	return FALSE;
}

IDispenser* CDispenserSpinner::clone(void)
{
	return new CDispenserSpinner;
}

FXWindow* CDispenserSpinner::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	return new FXSpinner(p, 20, tgt, sel,
				FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT);
}

void CDispenserSpinner::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	FXString strOptions;

	getProperty("Frame").getOptions(pWindow, strOptions);
	getProperty("Layout").getOptions(pWindow, strOptions);
	getProperty("Style").getOptions(pWindow, strOptions);

	stream << "\t" << widget << " = new FXSpinner(" << parent << ", 20, 0, 0, ";

	stream << strOptions << ", " << getProperty("Left").get(pWindow) << "," << getProperty("Top").get(pWindow) << ",";
	stream << getProperty("Width").get(pWindow) << "," << getProperty("Height").get(pWindow);

	stream << ", " << getProperty("PadLeft").get(pWindow) << "," << getProperty("PadRight").get(pWindow) << ",";
	stream << getProperty("PadTop").get(pWindow) << "," << getProperty("PadBottom").get(pWindow) << ");\n";

	getPropertyCode(stream, pWindow, widget);
}
