/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>
#include "CDispenserStatusbar.h"

#include <CPropertyBackColor.h>
#include <CPropertyTop.h>
#include <CPropertyLeft.h>
#include <CPropertyFrame.h>
#include <CPropertyLayout.h>
#include <CPropertyWidth.h>
#include <CPropertyHeight.h>
#include <CPropertyPadBottom.h>
#include <CPropertyPadLeft.h>
#include <CPropertyPadRight.h>
#include <CPropertyPadTop.h>

#include <CVariant.h>

///////////////////////////////////////////////////////////////////////////////
//

const unsigned char Statusbar[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xf7,0x00,0x00,0x00,0x00,0x00,
  0x80,0x00,0x00,0x00,0x80,0x00,0x80,0x80,0x00,0x00,0x00,0x80,0x80,0x00,0x80,0x00,
  0x80,0x80,0xc0,0xc0,0xc0,0xc0,0xdc,0xc0,0xa6,0xca,0xf0,0x00,0x00,0x00,0x00,0x00,
  0x2a,0x00,0x00,0x55,0x00,0x00,0x7f,0x00,0x00,0xaa,0x00,0x00,0xd4,0x00,0x2a,0x00,
  0x00,0x2a,0x2a,0x00,0x2a,0x55,0x00,0x2a,0x7f,0x00,0x2a,0xaa,0x00,0x2a,0xd4,0x00,
  0x55,0x00,0x00,0x55,0x2a,0x00,0x55,0x55,0x00,0x55,0x7f,0x00,0x55,0xaa,0x00,0x55,
  0xd4,0x00,0x7f,0x00,0x00,0x7f,0x2a,0x00,0x7f,0x55,0x00,0x7f,0x7f,0x00,0x7f,0xaa,
  0x00,0x7f,0xd4,0x00,0xaa,0x00,0x00,0xaa,0x2a,0x00,0xaa,0x55,0x00,0xaa,0x7f,0x00,
  0xaa,0xaa,0x00,0xaa,0xd4,0x00,0xd4,0x00,0x00,0xd4,0x2a,0x00,0xd4,0x55,0x00,0xd4,
  0x7f,0x00,0xd4,0xaa,0x00,0xd4,0xd4,0x2a,0x00,0x00,0x2a,0x00,0x2a,0x2a,0x00,0x55,
  0x2a,0x00,0x7f,0x2a,0x00,0xaa,0x2a,0x00,0xd4,0x2a,0x2a,0x00,0x2a,0x2a,0x2a,0x2a,
  0x2a,0x55,0x2a,0x2a,0x7f,0x2a,0x2a,0xaa,0x2a,0x2a,0xd4,0x2a,0x55,0x00,0x2a,0x55,
  0x2a,0x2a,0x55,0x55,0x2a,0x55,0x7f,0x2a,0x55,0xaa,0x2a,0x55,0xd4,0x2a,0x7f,0x00,
  0x2a,0x7f,0x2a,0x2a,0x7f,0x55,0x2a,0x7f,0x7f,0x2a,0x7f,0xaa,0x2a,0x7f,0xd4,0x2a,
  0xaa,0x00,0x2a,0xaa,0x2a,0x2a,0xaa,0x55,0x2a,0xaa,0x7f,0x2a,0xaa,0xaa,0x2a,0xaa,
  0xd4,0x2a,0xd4,0x00,0x2a,0xd4,0x2a,0x2a,0xd4,0x55,0x2a,0xd4,0x7f,0x2a,0xd4,0xaa,
  0x2a,0xd4,0xd4,0x55,0x00,0x00,0x55,0x00,0x2a,0x55,0x00,0x55,0x55,0x00,0x7f,0x55,
  0x00,0xaa,0x55,0x00,0xd4,0x55,0x2a,0x00,0x55,0x2a,0x2a,0x55,0x2a,0x55,0x55,0x2a,
  0x7f,0x55,0x2a,0xaa,0x55,0x2a,0xd4,0x55,0x55,0x00,0x55,0x55,0x2a,0x55,0x55,0x55,
  0x55,0x55,0x7f,0x55,0x55,0xaa,0x55,0x55,0xd4,0x55,0x7f,0x00,0x55,0x7f,0x2a,0x55,
  0x7f,0x55,0x55,0x7f,0x7f,0x55,0x7f,0xaa,0x55,0x7f,0xd4,0x55,0xaa,0x00,0x55,0xaa,
  0x2a,0x55,0xaa,0x55,0x55,0xaa,0x7f,0x55,0xaa,0xaa,0x55,0xaa,0xd4,0x55,0xd4,0x00,
  0x55,0xd4,0x2a,0x55,0xd4,0x55,0x55,0xd4,0x7f,0x55,0xd4,0xaa,0x55,0xd4,0xd4,0x7f,
  0x00,0x00,0x7f,0x00,0x2a,0x7f,0x00,0x55,0x7f,0x00,0x7f,0x7f,0x00,0xaa,0x7f,0x00,
  0xd4,0x7f,0x2a,0x00,0x7f,0x2a,0x2a,0x7f,0x2a,0x55,0x7f,0x2a,0x7f,0x7f,0x2a,0xaa,
  0x7f,0x2a,0xd4,0x7f,0x55,0x00,0x7f,0x55,0x2a,0x7f,0x55,0x55,0x7f,0x55,0x7f,0x7f,
  0x55,0xaa,0x7f,0x55,0xd4,0x7f,0x7f,0x00,0x7f,0x7f,0x2a,0x7f,0x7f,0x55,0x7f,0x7f,
  0x7f,0x7f,0x7f,0xaa,0x7f,0x7f,0xd4,0x7f,0xaa,0x00,0x7f,0xaa,0x2a,0x7f,0xaa,0x55,
  0x7f,0xaa,0x7f,0x7f,0xaa,0xaa,0x7f,0xaa,0xd4,0x7f,0xd4,0x00,0x7f,0xd4,0x2a,0x7f,
  0xd4,0x55,0x7f,0xd4,0x7f,0x7f,0xd4,0xaa,0x7f,0xd4,0xd4,0xaa,0x00,0x00,0xaa,0x00,
  0x2a,0xaa,0x00,0x55,0xaa,0x00,0x7f,0xaa,0x00,0xaa,0xaa,0x00,0xd4,0xaa,0x2a,0x00,
  0xaa,0x2a,0x2a,0xaa,0x2a,0x55,0xaa,0x2a,0x7f,0xaa,0x2a,0xaa,0xaa,0x2a,0xd4,0xaa,
  0x55,0x00,0xaa,0x55,0x2a,0xaa,0x55,0x55,0xaa,0x55,0x7f,0xaa,0x55,0xaa,0xaa,0x55,
  0xd4,0xaa,0x7f,0x00,0xaa,0x7f,0x2a,0xaa,0x7f,0x55,0xaa,0x7f,0x7f,0xaa,0x7f,0xaa,
  0xaa,0x7f,0xd4,0xaa,0xaa,0x00,0xaa,0xaa,0x2a,0xaa,0xaa,0x55,0xaa,0xaa,0x7f,0xaa,
  0xaa,0xaa,0xaa,0xaa,0xd4,0xaa,0xd4,0x00,0xaa,0xd4,0x2a,0xaa,0xd4,0x55,0xaa,0xd4,
  0x7f,0xaa,0xd4,0xaa,0xaa,0xd4,0xd4,0xd4,0x00,0x00,0xd4,0x00,0x2a,0xd4,0x00,0x55,
  0xd4,0x00,0x7f,0xd4,0x00,0xaa,0xd4,0x00,0xd4,0xd4,0x2a,0x00,0xd4,0x2a,0x2a,0xd4,
  0x2a,0x55,0xd4,0x2a,0x7f,0xd4,0x2a,0xaa,0xd4,0x2a,0xd4,0xd4,0x55,0x00,0xd4,0x55,
  0x2a,0xd4,0x55,0x55,0xd4,0x55,0x7f,0xd4,0x55,0xaa,0xd4,0x55,0xd4,0xd4,0x7f,0x00,
  0xd4,0x7f,0x2a,0xd4,0x7f,0x55,0xd4,0x7f,0x7f,0xd4,0x7f,0xaa,0xd4,0x7f,0xd4,0xd4,
  0xaa,0x00,0xd4,0xaa,0x2a,0xd4,0xaa,0x55,0xd4,0xaa,0x7f,0xd4,0xaa,0xaa,0xd4,0xaa,
  0xd4,0xd4,0xd4,0x00,0xd4,0xd4,0x2a,0xd4,0xd4,0x55,0xd4,0xd4,0x7f,0xd4,0xd4,0xaa,
  0xd4,0xd4,0xd4,0x00,0x00,0x00,0x0c,0x0c,0x0c,0x19,0x19,0x19,0x26,0x26,0x26,0x33,
  0x33,0x33,0x3f,0x3f,0x3f,0x4c,0x4c,0x4c,0x59,0x59,0x59,0x66,0x66,0x66,0x72,0x72,
  0x72,0x7f,0x7f,0x7f,0x8c,0x8c,0x8c,0x99,0x99,0x99,0xa5,0xa5,0xa5,0xb2,0xb2,0xb2,
  0xbf,0xbf,0xbf,0xcc,0xcc,0xcc,0xd8,0xd8,0xd8,0xe5,0xe5,0xe5,0xf2,0xf2,0xf2,0xff,
  0xfb,0xf0,0xa0,0xa0,0xa4,0x80,0x80,0x80,0xff,0x00,0x00,0x00,0xff,0x00,0xff,0xff,
  0x00,0x00,0x00,0xff,0xff,0x00,0xff,0x00,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x08,0xfe,0x00,0xf1,0x09,0x1c,0x48,0xb0,0xa0,
  0xc1,0x83,0x08,0x13,0x2a,0x5c,0xc8,0xb0,0xa1,0xc3,0x87,0x10,0x23,0x4a,0x9c,0x48,
  0xb1,0xa2,0xc5,0x8b,0x18,0x33,0x1e,0xd8,0xc8,0xb1,0xa3,0xc7,0x8f,0x20,0x43,0x8a,
  0x14,0xa9,0x70,0xa4,0xc9,0x93,0x28,0x43,0x96,0x4c,0xc9,0xb2,0xa5,0xca,0x84,0x2e,
  0x63,0xca,0x5c,0x29,0xb3,0x26,0x4a,0x9a,0x36,0x73,0xbe,0x44,0xa8,0xb3,0x27,0x48,
  0x9c,0x3e,0x7d,0x02,0x0d,0xaa,0x73,0x28,0x51,0x9b,0x46,0x8f,0xce,0x84,0xa9,0xb4,
  0x67,0xd2,0xa6,0x2d,0x9f,0x42,0x4d,0x29,0x75,0xea,0xc9,0xaa,0x56,0x47,0x62,0xcd,
  0xba,0xf3,0x20,0xd7,0xa5,0x3c,0xbf,0xba,0xdc,0x2a,0xb6,0x23,0xd9,0xb2,0x1b,0xcf,
  0xa2,0x55,0x5b,0x96,0xad,0x58,0xb7,0x5f,0xe1,0x72,0x95,0x9b,0x95,0xae,0x55,0xbb,
  0x53,0xf1,0x42,0xd5,0xdb,0x94,0xaf,0x52,0xbf,0x47,0x01,0x13,0x2d,0x99,0xb1,0xf0,
  0xbf,0x03,0x84,0xd1,0x9a,0x3c,0x9c,0x58,0xb1,0x48,0xc6,0x30,0xf1,0x39,0x7e,0x8c,
  0x38,0xf2,0xe4,0x90,0x90,0x79,0x4a,0xf6,0x09,0x40,0x67,0x66,0xaf,0x9b,0x0f,0x00,
  0x18,0xdd,0x39,0x66,0x69,0x9b,0x9f,0x0d,0x22,0xe6,0x78,0xfa,0x34,0x4b,0xd7,0x32,
  0x53,0x17,0x5c,0xbd,0xb1,0x35,0xeb,0xd1,0xb7,0x5b,0x93,0x16,0xcd,0xda,0xa5,0x6c,
  0x82,0xb4,0x79,0xd7,0x1e,0x4e,0xbc,0xb7,0x6d,0xdb,0x2d,0x7f,0x0f,0x0c,0x4e,0x5a,
  0xf7,0x6e,0xd1,0xbb,0x91,0x77,0x86,0x8d,0x52,0xb9,0x40,0xe6,0xc5,0x61,0x1f,0xef,
  0xcd,0x9b,0xfa,0x49,0xeb,0x92,0x43,0x48,0x4b,0x37,0x4e,0x3e,0x7b,0x47,0xef,0x20,
  0xc1,0x63,0xe7,0xfe,0x1c,0x3a,0x6e,0xf7,0xc8,0x8b,0x2f,0xae,0xac,0x59,0x29,0xfa,
  0xef,0xf4,0x41,0xdb,0xaf,0xa9,0x3e,0xf4,0xe5,0x8e,0xfd,0xfd,0xf7,0x91,0x7a,0xff,
  0x14,0x68,0xe0,0x81,0x08,0x26,0xa8,0xe0,0x82,0x0c,0x2a,0x98,0x9f,0x6a,0x02,0x76,
  0x55,0xd8,0x84,0x14,0x56,0x68,0xe1,0x85,0x18,0x66,0xa8,0xe1,0x86,0x10,0x05,0x04,
  0x00,0x3b
  };

CDispenserStatusbar::CDispenserStatusbar()
{
	m_Properties.append(new CPropertyLayout<FXStatusbar>(this));
	m_Properties.append(new CPropertyFrame<FXStatusbar>(this));
	m_Properties.append(new CPropertyBackColor<FXStatusbar, FXRGB(192, 192, 192)>(this));
	m_Properties.append(new CPropertyPadTop<FXStatusbar>(this));
	m_Properties.append(new CPropertyPadLeft<FXStatusbar>(this));
	m_Properties.append(new CPropertyPadBottom<FXStatusbar>(this));
	m_Properties.append(new CPropertyPadRight<FXStatusbar>(this));
	m_Properties.append(new CPropertyLeft<FXStatusbar>(this));
	m_Properties.append(new CPropertyTop<FXStatusbar>(this));
	m_Properties.append(new CPropertyWidth<FXStatusbar>(this));
	m_Properties.append(new CPropertyHeight<FXStatusbar>(this));
}

const char* CDispenserStatusbar::getWidgetName(void)
{
	return "FXStatusbar";
}

const void* CDispenserStatusbar::getWidgetIcon(void)
{
	return Statusbar;
}

const char* CDispenserStatusbar::getWidgetCategory(void)
{
	return "Extended";
}

const int CDispenserStatusbar::getWidgetGrabbers(void)
{
	return GS_ALL;
}

FXint CDispenserStatusbar::getDefaultWidth(void)
{
	return 80;
}

FXint CDispenserStatusbar::getDefaultHeight(void)
{
	return 40;
}

FXint CDispenserStatusbar::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserStatusbar::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserStatusbar::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserStatusbar::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserStatusbar::getClassInfo(void)
{
	return &FXStatusbar::metaClass;
}

FXbool CDispenserStatusbar::allowChild(const FXMetaClass*)
{
	return FALSE;
}

FXbool CDispenserStatusbar::allowParent(FXWindow*)
{
	return TRUE;
}

FXWindow* CDispenserStatusbar::getCreateParent(FXWindow* pParent)
{
	return pParent;
}

/*
	FXStatusbar
	  FXDragCorner
	  FXStatusline
*/
FXbool CDispenserStatusbar::IsValidWidget(FXWindow** ppWindow)
{
	if ( (*ppWindow)->isMemberOf(&FXStatusbar::metaClass) )
	{
		return TRUE;
	}
	else if ( (*ppWindow)->isMemberOf(&FXDragCorner::metaClass) )
	{
		*ppWindow = (*ppWindow)->getParent();
		return *ppWindow && (*ppWindow)->isMemberOf(&FXStatusbar::metaClass);
	}
	else if ( (*ppWindow)->isMemberOf(&FXStatusline::metaClass) )
	{
		*ppWindow = (*ppWindow)->getParent();
		return *ppWindow && (*ppWindow)->isMemberOf(&FXStatusbar::metaClass);
	}

	return FALSE;
}

IDispenser* CDispenserStatusbar::clone(void)
{
	return new CDispenserStatusbar;
}

FXWindow* CDispenserStatusbar::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	return new FXStatusbar(p, LAYOUT_SIDE_BOTTOM|LAYOUT_BOTTOM|LAYOUT_FILL_X);
}

void CDispenserStatusbar::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	FXString strOptions;

	getProperty("Frame").getOptions(pWindow, strOptions);
	getProperty("Layout").getOptions(pWindow, strOptions);

	stream << "\t" << widget << " = new FXStatusbar(" << parent << ", ";

	stream << strOptions << ", " << getProperty("Left").get(pWindow) << "," << getProperty("Top").get(pWindow) << ",";
	stream << getProperty("Width").get(pWindow) << "," << getProperty("Height").get(pWindow);

	stream << ", " << getProperty("PadLeft").get(pWindow) << "," << getProperty("PadRight").get(pWindow) << ",";
	stream << getProperty("PadTop").get(pWindow) << "," << getProperty("PadBottom").get(pWindow) << ");\n";

	getPropertyCode(stream, pWindow, widget);
}
