/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#ifndef __CPropertyBackColor_H__
#define __CPropertyBackColor_H__

template <class T, FXColor C>
class CPropertyBackColor : public CProperty
{
public:
	CPropertyBackColor(IDispenser* pDispenser) : CProperty(pDispenser) {};

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	virtual FXString getImplementationCode(FXWindow* pWindow, FXString& widget);
};

template <class T, FXColor C>
const char* CPropertyBackColor<T, C>::name(void)
{
	return "BackColor";
}

template <class T, FXColor C>
ePropertyType CPropertyBackColor<T, C>::type(void)
{
	return PROPERTY_COLOR;
}

template <class T, FXColor C>
const char* CPropertyBackColor<T, C>::description(void)
{
	return "The background color of this\nobject";
}

template <class T, FXColor C>
const char* CPropertyBackColor<T, C>::tooltip(void)
{
	return "getBackColor()\nsetBackColor()";
}

template <class T, FXColor C>
void CPropertyBackColor<T, C>::set(FXWindow* pWindow, FXString strValue)
{
	pWindow->setBackColor( getColorFromString(strValue) );
}

template <class T, FXColor C>
FXString CPropertyBackColor<T, C>::get(FXWindow* pWindow)
{
	return getStringFromColor( pWindow->getBackColor() );
}

template <class T, FXColor C>
FXString CPropertyBackColor<T, C>::getImplementationCode(FXWindow* pWindow, FXString& widget)
{
	T* p = static_cast<T*>(pWindow);

	if ( p->getBackColor() != C )
	{
		FXString str = getFXRGBString(p->getBackColor());
		return "\t" + widget + "->setBackColor(" + str + ");\n";
	}

	return "";
}

#endif // __CPropertyBackColor_H__
