/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#ifndef __CPropertyEditable_H__
#define __CPropertyEditable_H__

template <class T>
class CPropertyEditable : public CProperty
{
public:
	CPropertyEditable(IDispenser* pDispenser) : CProperty(pDispenser) {};

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);
};

template <class T>
const char* CPropertyEditable<T>::name(void)
{
	return "Editable";
}

template <class T>
ePropertyType CPropertyEditable<T>::type(void)
{
	return PROPERTY_BOOL;
}

template <class T>
const char* CPropertyEditable<T>::description(void)
{
	return "Is this object editable?";
}

template <class T>
const char* CPropertyEditable<T>::tooltip(void)
{
	return "isEditable()\nsetEditable(FXbool edit=TRUE)";
}

template <class T>
void CPropertyEditable<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setEditable( getBoolFromString(strValue) );
}

template <class T>
FXString CPropertyEditable<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromBool( p->isEditable() );
}

#endif // __CPropertyEditable_H__
