/**************************************************************************  
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#ifndef __CPropertyFrame_H__
#define __CPropertyFrame_H__

template <class T>
class CPropertyFrame : public CProperty
{
public:
	CPropertyFrame(IDispenser* pDispenser) : CProperty(pDispenser) {};

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	OPTION* option(void);
	virtual void getOptions(FXWindow* pWindow, FXString& strOptions);

private:
	static OPTION Style[];
};

// From FXWindow.h
template <class T>
OPTION CPropertyFrame<T>::Style[] = 
{
	{"FRAME_SUNKEN", FRAME_SUNKEN, ""},
	{"FRAME_RAISED", FRAME_RAISED, ""},
	{"FRAME_THICK" , FRAME_THICK , ""},
	{"FRAME_GROOVE", FRAME_GROOVE, "Same as FRAME_THICK"},
	{"FRAME_RIDGE" , FRAME_RIDGE , "Same as FRAME_THICK|FRAME_RAISED|FRAME_SUNKEN"},
	{"FRAME_LINE"  , FRAME_LINE  , "Same as FRAME_RAISED|FRAME_SUNKEN"},
	{"FRAME_NORMAL", FRAME_NORMAL, "Same as FRAME_SUNKEN|FRAME_THICK"},
	{0, 0, 0}
};

template <class T>
const char* CPropertyFrame<T>::name(void)
{
	return "Frame";
}

template <class T>
ePropertyType CPropertyFrame<T>::type(void)
{
	return PROPERTY_COMBO;
}

template <class T>
const char* CPropertyFrame<T>::description(void)
{
	return "The style of the border or\nboundary of this object";
}

template <class T>
const char* CPropertyFrame<T>::tooltip(void)
{
	return "getFrameStyle()\nsetFrameStyle()";
}

template <class T>
void CPropertyFrame<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setFrameStyle( getIntFromString(strValue) );
}

template <class T>
FXString CPropertyFrame<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromInt( p->getFrameStyle() );
}

template <class T>
OPTION* CPropertyFrame<T>::option(void)
{
	return Style;
}

template <class T>
void CPropertyFrame<T>::getOptions(FXWindow* pWindow, FXString& strOptions)
{
	T* p = static_cast<T*>(pWindow);
	FXuint option = p->getFrameStyle();

	for (OPTION* pOption = Style; pOption->pszName; pOption++)
	{
		if ( (option & pOption->value) == pOption->value )
		{
			if ( strOptions.length() > 0 )
			{
				strOptions += "|";
			}
			strOptions += pOption->pszName;
		}
	}
}

#endif // __CPropertyFrame_H__
