/**************************************************************************  
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#ifndef __CPropertyIconPosition_H__
#define __CPropertyIconPosition_H__

template <class T>
class CPropertyIconPosition : public CProperty
{
public:
	CPropertyIconPosition(IDispenser* pDispenser) : CProperty(pDispenser) {};

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);
	virtual void validate(IConnector* pConnector, FXString& strNewValue);
	virtual void validate(IConnector* pConnector) {}

	OPTION* option(void);
	virtual void getOptions(FXWindow* pWindow, FXString& strOptions);

private:
	static OPTION Style[];
};

// From FXLabel.h
template <class T>
OPTION CPropertyIconPosition<T>::Style[] = 
{
	{"ICON_AFTER_TEXT",  ICON_AFTER_TEXT,  "Icon appears after text (to its right)"},
	{"ICON_BEFORE_TEXT", ICON_BEFORE_TEXT, "Icon appears before text (to its left)"},
	{"ICON_ABOVE_TEXT",  ICON_ABOVE_TEXT,  "Icon appears above text"},
	{"ICON_BELOW_TEXT",  ICON_BELOW_TEXT,  "Icon appears below text"},
	{"TEXT_OVER_ICON",   TEXT_OVER_ICON,   "Same as ICON_UNDER_TEXT"},
	{"TEXT_AFTER_ICON",  TEXT_AFTER_ICON,  "Same as ICON_BEFORE_TEXT"},
	{"TEXT_BEFORE_ICON", TEXT_BEFORE_ICON, "Same as ICON_AFTER_TEXT"},
	{"TEXT_ABOVE_ICON",  TEXT_ABOVE_ICON,  "Same as ICON_BELOW_TEXT"},
	{"TEXT_BELOW_ICON",  TEXT_BELOW_ICON,  "Same as ICON_ABOVE_TEXT"},
	{0, 0, 0}
};

template <class T>
const char* CPropertyIconPosition<T>::name(void)
{
	return "IconPosition";
}

template <class T>
ePropertyType CPropertyIconPosition<T>::type(void)
{
	return PROPERTY_COMBO;
}

template <class T>
const char* CPropertyIconPosition<T>::description(void)
{
	return "The position of any icon relative\nto this object";
}

template <class T>
const char* CPropertyIconPosition<T>::tooltip(void)
{
	return "getIconPosition()\nsetIconPosition()";
}

template <class T>
void CPropertyIconPosition<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setIconPosition( getIntFromString(strValue) );
}

template <class T>
FXString CPropertyIconPosition<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	FXint nValue = p->getIconPosition();
	return getStringFromInt(nValue);
}

template <class T>
void CPropertyIconPosition<T>::validate(IConnector* pConnector, FXString& strNewValue)
{
	T* p = static_cast<T*>(pConnector->getWidget());
	FXint nOldValue = p->getIconPosition();
	FXint nNewValue = getIntFromString(strNewValue);

	toggleAttribute(nOldValue, nNewValue, ICON_BEFORE_TEXT, ICON_AFTER_TEXT);

	strNewValue = getStringFromInt(nNewValue);
}

template <class T>
OPTION* CPropertyIconPosition<T>::option(void)
{
	return Style;
}

template <class T>
void CPropertyIconPosition<T>::getOptions(FXWindow* pWindow, FXString& strOptions)
{
	T* p = static_cast<T*>(pWindow);
	FXuint option = p->getIconPosition();

	for (OPTION* pOption = Style; pOption->pszName; pOption++)
	{
		if ( (option & pOption->value) == pOption->value )
		{
			if ( strOptions.length() > 0 )
			{
				strOptions += "|";
			}
			strOptions += pOption->pszName;
		}
	}
}

#endif // __CPropertyIconPosition_H__
