/**************************************************************************  
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#ifndef __CPropertyPack_H__
#define __CPropertyPack_H__

// From FXLabel.h
extern OPTION PackStyle[];

template <class T>
class CPropertyPack : public CProperty
{
public:
	CPropertyPack(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	OPTION* option(void);
	virtual void getOptions(FXWindow* pWindow, FXString& strOptions);
};

template <class T>
const char* CPropertyPack<T>::name(void)
{
	return "Justify";
}

template <class T>
ePropertyType CPropertyPack<T>::type(void)
{
	return PROPERTY_COMBO;
}

template <class T>
const char* CPropertyPack<T>::description(void)
{
	return "The position of this object's contents\nrelative to the boundary of this object";
}

template <class T>
void CPropertyPack<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setJustify(getIntFromString(strValue));
}

template <class T>
FXString CPropertyPack<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromInt(p->getJustify());
}

template <class T>
OPTION* CPropertyPack<T>::option(void)
{
	return JustifyStyle;
}

template <class T>
void CPropertyPack<T>::getOptions(FXWindow* pWindow, FXString& strOptions)
{
	T* p = static_cast<T*>(pWindow);
	FXuint option = p->getJustify();

	for (OPTION* pOption = JustifyStyle; pOption->pszName; pOption++)
	{
		if ( (option & pOption->value) == pOption->value )
		{
			if ( strOptions.length() > 0 )
			{
				strOptions += "|";
			}
			strOptions += pOption->pszName;
		}
	}
}

#endif // __CPropertyPack_H__
