/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#ifndef __CPropertyPadTop_H__
#define __CPropertyPadTop_H__

template <class T>
class CPropertyPadTop : public CProperty
{
public:
	CPropertyPadTop(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);
	virtual FXint style(void) {return PROPERTY_MULTIPLE;}

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);
};

template <class T>
const char* CPropertyPadTop<T>::name(void)
{
	return "PadTop";
}

template <class T>
ePropertyType CPropertyPadTop<T>::type(void)
{
	return PROPERTY_TEXT;
}

template <class T>
const char* CPropertyPadTop<T>::description(void)
{
	return "The distance between the internal\ntop edge of this object and the top\nedge of its content";
}

template <class T>
const char* CPropertyPadTop<T>::tooltip(void)
{
	return "getPadTop()\nsetPadTop()";
}

template <class T>
void CPropertyPadTop<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setPadTop( getIntFromString(strValue) );
}

template <class T>
FXString CPropertyPadTop<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromInt( p->getPadTop() );
}

#endif // __CPropertyPadTop_H__
