/**************************************************************************  
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#ifndef __CPropertyRange_H__
#define __CPropertyRange_H__

template <class T, FXint I>
class CPropertyRange : public CProperty
{
public:
	CPropertyRange(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);
	virtual FXint style(void) {return 0;}

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	virtual FXString getImplementationCode(FXWindow* pWindow, FXString& widget);
};

template <class T, FXint I>
const char* CPropertyRange<T, I>::name(void)
{
	return "Range";
}

template <class T, FXint I>
ePropertyType CPropertyRange<T, I>::type(void)
{
	return PROPERTY_TEXT;
}

template <class T, FXint I>
const char* CPropertyRange<T, I>::description(void)
{
	return "Sets or returns the range\nof this object";
}

template <class T, FXint I>
const char* CPropertyRange<T, I>::tooltip(void)
{
	return "getRange()\nsetRange()";
}

template <class T, FXint I>
void CPropertyRange<T, I>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = (T*)pWindow;
	p->setRange( getIntFromString(strValue) );
}

template <class T, FXint I>
FXString CPropertyRange<T, I>::get(FXWindow* pWindow)
{
	T* p = (T*)pWindow;
	return getStringFromInt( p->getRange() );
}

template <class T, FXint I>
FXString CPropertyRange<T, I>::getImplementationCode(FXWindow* pWindow, FXString& widget)
{
	T* p = static_cast<T*>(pWindow);
	
	if ( p && p->getRange() != I )
	{
		return "\t" + widget + "->setRange(" + get(pWindow) + ");\n";
	}

	return "";
}

#endif // __CPropertyRange_H__
