/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#ifndef __CPropertyTextColor_H__
#define __CPropertyTextColor_H__

template <class T, FXColor C>
class CPropertyTextColor : public CProperty
{
public:
	CPropertyTextColor(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	virtual FXString getImplementationCode(FXWindow* pWindow, FXString& widget);
};

template <class T, FXColor C>
const char* CPropertyTextColor<T, C>::name(void)
{
	return "TextColor";
}

template <class T, FXColor C>
ePropertyType CPropertyTextColor<T, C>::type(void)
{
	return PROPERTY_COLOR;
}

template <class T, FXColor C>
const char* CPropertyTextColor<T, C>::description(void)
{
	return "The text color of this object";
}

template <class T, FXColor C>
const char* CPropertyTextColor<T, C>::tooltip(void)
{
	return "getTextColor()\nsetTextColor()";
}

template <class T, FXColor C>
void CPropertyTextColor<T, C>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setTextColor( getColorFromString(strValue) );
}

template <class T, FXColor C>
FXString CPropertyTextColor<T, C>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromColor( p->getTextColor() );
}

template <class T, FXColor C>
FXString CPropertyTextColor<T, C>::getImplementationCode(FXWindow* pWindow, FXString& widget)
{
	T* p = static_cast<T*>(pWindow);

	if ( p->getTextColor() != C )
	{
		FXString str = getFXRGBString(p->getTextColor());
		return "\t" + widget + "->setTextColor(" + str + ");\n";
	}

	return "";
}

#endif // __CPropertyTextColor_H__
