/**************************************************************************  
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#ifndef __CPropertyVisible_H__
#define __CPropertyVisible_H__

template <class T, FXint D>
class CPropertyVisible : public CProperty
{
public:
	CPropertyVisible(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	virtual FXString getImplementationCode(FXWindow* pWindow, FXString& widget);
};

template <class T, FXint D>
const char* CPropertyVisible<T, D>::name(void)
{
	return "Visible";
}

template <class T, FXint D>
ePropertyType CPropertyVisible<T, D>::type(void)
{
	return PROPERTY_TEXT;
}

template <class T, FXint D>
const char* CPropertyVisible<T, D>::description(void)
{
	return "Sets or returns the number of\nvisible items in this object";
}

template <class T, FXint D>
const char* CPropertyVisible<T, D>::tooltip(void)
{
	return "getNumVisible()\nseNumtVisible()";
}

template <class T, FXint D>
void CPropertyVisible<T, D>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = (T*)pWindow;
	p->setNumVisible( getIntFromString(strValue) );
}

template <class T, FXint D>
FXString CPropertyVisible<T, D>::get(FXWindow* pWindow)
{
	T* p = (T*)pWindow;
	return getStringFromInt( p->getNumVisible() );
}

template <class T, FXint D>
FXString CPropertyVisible<T, D>::getImplementationCode(FXWindow* pWindow, FXString& widget)
{
	T* p = static_cast<T*>(pWindow);
	
	if ( p && p->getNumVisible() != D )
	{
		return "\t" + widget + "->setNumVisible(" + get(pWindow) + ");\n";
	}

	return "";
}

#endif // __CPropertyVisible_H__
