/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#ifndef __IDispenser_H__
#define __IDispenser_H__

enum eGrabber
{
	TOP_LEFT,
	TOP_CENTRE,
	TOP_RIGHT,
	CENTRE_LEFT,
	CENTRE_RIGHT,
	BOTTOM_LEFT,
	BOTTOM_CENTRE,
	BOTTOM_RIGHT,
	ALL,
	NONE
};

#define GS_NONE          0
#define GS_TOP_LEFT      1
#define GS_TOP_CENTRE    2
#define GS_TOP_RIGHT     4
#define GS_CENTRE_LEFT   8 
#define GS_CENTRE_RIGHT  16
#define GS_BOTTOM_LEFT   32
#define GS_BOTTOM_CENTRE 64
#define GS_BOTTOM_RIGHT  128
#define GS_ALL GS_TOP_LEFT | GS_TOP_CENTRE | GS_TOP_RIGHT | GS_CENTRE_LEFT | GS_CENTRE_RIGHT | GS_BOTTOM_LEFT | GS_BOTTOM_CENTRE | GS_BOTTOM_RIGHT
#define GS_DIALOG GS_CENTRE_RIGHT | GS_BOTTOM_CENTRE | GS_BOTTOM_RIGHT

struct IDispenser
{
	virtual void addProperty(IProperty* property) = 0;

	// Adding a widget to toolbox
	virtual const char* getWidgetName(void) = 0;
	virtual const void* getWidgetIcon(void) = 0;
	virtual const char* getWidgetCategory(void) = 0;
	virtual const int getWidgetGrabbers(void) = 0;

	// Creating a widget (user has clicked on the toolbar)
	virtual FXWindow* createWidget(FXComposite* p, FXObject* tgt=NULL, FXSelector sel=0) = 0;

	virtual FXint getDefaultWidth(void)  = 0;
	virtual FXint getDefaultHeight(void) = 0;
	virtual FXint getMinimumWidth(void)  = 0;
	virtual FXint getMinimumHeight(void) = 0;
	virtual FXint getMaximumWidth(void)  = 0;
	virtual FXint getMaximumHeight(void) = 0;

	// widget properties
	virtual FXint getPropertyCount(void) = 0;
	virtual IProperty* getProperty(FXint index) = 0;
	virtual IProperty& getProperty(const FXchar* pszName) = 0;

	//	Validate Parent/Child relationships
	virtual const FXMetaClass* getClassInfo(void) = 0;
	virtual FXbool canHaveChildren(void) = 0;
	virtual FXbool allowChild(const FXMetaClass* pChildClass) = 0;
	virtual FXbool allowParent(FXWindow* pParent) = 0;
	virtual FXWindow* getCreateParent(FXWindow* pParent) = 0;
	virtual FXbool IsValidWidget(FXWindow** ppWindow) = 0;
	virtual FXbool widgetDeletesChildren(void) = 0;

	// Another instance of this component is required
	virtual IDispenser* clone(void) = 0;

	// Return the code snippet necessary to create this component
	virtual FXString getChildSuffix(void) = 0;
	virtual void getDefinitionCode(ITextStream& stream, FXWindow* pWidget, FXString& widget, FXString& parent) = 0;
	virtual void getImplementationCode(ITextStream& stream, FXWindow* pWidget, FXString& widget, FXString& parent) = 0;
	virtual void getPropertyCode(ITextStream& stream, FXWindow* pWindow, FXString& widget) = 0;
};

#endif // __IDispenser_H__
