/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CConnector.h>

#include <CVariant.h>

#include <CComboBox.h>
#include <CCheckComboBox.h>

///////////////////////////////////////////////////////////////////////////////
//

FXDEFMAP(CCheckButton) CCheckButtonMap[]=
{
	FXMAPFUNC(SEL_PAINT,0,CCheckButton::onPaint),
	FXMAPFUNC(SEL_LEFTBUTTONPRESS,0,CCheckButton::onLeftBtnPress),
};

// Object implementation
FXIMPLEMENT(CCheckButton,FXCheckButton,CCheckButtonMap,ARRAYNUMBER(CCheckButtonMap))

CCheckButton::CCheckButton()
{
	m_pOption = 0;
	m_pParent = 0;
	m_pConnector = 0;
}

CCheckButton::CCheckButton(CCheckComboBox* pParent, CConnector* pConnector, FXComposite* p,OPTION* o,FXObject* tgt,FXSelector sel, const FXString& text)
	: FXCheckButton(p, text,tgt,sel, LAYOUT_SIDE_RIGHT|LAYOUT_FILL_X|LAYOUT_FILL_Y|JUSTIFY_HZ_APART|ICON_AFTER_TEXT,0,0,0,0, 1,1,0,1)
{
	m_pOption    = o;
	m_pParent    = pParent;
	m_pConnector = pConnector;

	setFont( new FXFont(getApp(),"helvetica",8,FONTWEIGHT_NORMAL) );

	updateCheckBox();
}

void CCheckButton::updateCheckBox(void)
{
	if ( ( getIntFromString(m_pConnector->get()) & m_pOption->value ) == m_pOption->value )
	{
		setCheck();
	}
	else
	{
		setCheck(FALSE);
	}

	m_pConnector->setDirty();
}

long CCheckButton::onLeftBtnPress(FXObject* o,FXSelector s,void* p)
{ 
	long l = FXCheckButton::onLeftBtnPress(o,s,p);

	FXint value = getIntFromString(m_pConnector->get());

	if ( check )
		m_pConnector->set(getStringFromInt(value | m_pOption->value));
	else
		m_pConnector->set(getStringFromInt(value &~ m_pOption->value));

	m_pParent->updateCheckBoxes();

	return l;
}

long CCheckButton::onPaint(FXObject*,FXSelector,void* ptr)
{
  FXEvent *ev=(FXEvent*)ptr;
  FXint tw=0,th=0,tx,ty,ix,iy;
  FXDCWindow dc(this,ev);
  
  dc.setForeground(backColor);
  dc.fillRectangle(ev->rect.x,ev->rect.y,ev->rect.w,ev->rect.h);
  drawFrame(dc,0,0,width,height);
  
  if(!label.empty()){
    tw=labelWidth(label);
    th=labelHeight(label);
    }

  just_x(tx,ix,tw,13);
  just_y(ty,iy,th,13);

//// 3d
////  dc.setForeground(borderColor);
////  dc.drawLine(ix+1,iy+1,ix+10,iy+1);
////  dc.drawLine(ix+1,iy+1,ix+1,iy+10);
////
////  dc.setForeground(shadowColor/*hiliteColor*/);
////  dc.drawLine(ix,iy+12,ix+12,iy+12);
////  dc.drawLine(ix+12,iy+12,ix+12,iy);
//// end 3d
//
//  dc.setForeground(shadowColor);
//  dc.drawLine(ix,iy,ix+12,iy);
//  dc.drawLine(ix,iy,ix,iy+12);
//
////  dc.setForeground(borderColor);
////  dc.drawLine(ix+1,iy+1,ix+10,iy+1);
////  dc.drawLine(ix+1,iy+1,ix+1,iy+10);
//
//  dc.setForeground(shadowColor/*hiliteColor*/);
//  dc.drawLine(ix,iy+12,ix+12,iy+12);
//  dc.drawLine(ix+12,iy+12,ix+12,iy);
//
////  dc.setForeground(shadowColor/*backColor*/);
////  dc.drawLine(ix+2,iy+11,ix+11,iy+11);
////  dc.drawLine(ix+11,iy+2,ix+11,iy+11);

  dc.setForeground(shadowColor);
  dc.drawLine(ix,iy,ix+11,iy);
  dc.drawLine(ix,iy,ix,iy+11);

  dc.setForeground(borderColor);
  dc.drawLine(ix+1,iy+1,ix+10,iy+1);
  dc.drawLine(ix+1,iy+1,ix+1,iy+10);

  dc.setForeground(hiliteColor);
  dc.drawLine(ix,iy+12,ix+12,iy+12);
  dc.drawLine(ix+12,iy+12,ix+12,iy);

  dc.setForeground(backColor);
  dc.drawLine(ix+2,iy+11,ix+11,iy+11);
  dc.drawLine(ix+11,iy+2,ix+11,iy+11);

  if(check==MAYBE)
    dc.setForeground(baseColor);
  else
    dc.setForeground(boxColor);
  dc.fillRectangle(ix+2,iy+2,9,9);

  if(check!=FALSE){
    FXSegment seg[6];
    seg[0].x1=3+ix; seg[0].y1=5+iy; seg[0].x2=5+ix; seg[0].y2=7+iy;
    seg[1].x1=3+ix; seg[1].y1=6+iy; seg[1].x2=5+ix; seg[1].y2=8+iy;
    seg[2].x1=3+ix; seg[2].y1=7+iy; seg[2].x2=5+ix; seg[2].y2=9+iy;
    seg[3].x1=5+ix; seg[3].y1=7+iy; seg[3].x2=9+ix; seg[3].y2=3+iy;
    seg[4].x1=5+ix; seg[4].y1=8+iy; seg[4].x2=9+ix; seg[4].y2=4+iy;
    seg[5].x1=5+ix; seg[5].y1=9+iy; seg[5].x2=9+ix; seg[5].y2=5+iy;
    if(isEnabled()){
      if(check==MAYBE)
        dc.setForeground(shadowColor);
      else
        dc.setForeground(textColor);
      }
    else{
      dc.setForeground(shadowColor);
      }
    dc.drawLineSegments(seg,6);
    }

  if(!label.empty()){
    dc.setTextFont(font);
    if(isEnabled()){
      dc.setForeground(textColor);
      drawLabel(dc,label,hotoff,tx,ty,tw,th);
      if(hasFocus()){
//        drawFocusRectangle(dc,tx-1,ty-1,tw+2,th+2);
        }
      }
    else{
      dc.setForeground(hiliteColor);
      drawLabel(dc,label,hotoff,tx+1,ty+1,tw,th);
      dc.setForeground(shadowColor);
      drawLabel(dc,label,hotoff,tx,ty,tw,th);
      } 
    }
  return 1;
  }

///////////////////////////////////////////////////////////////////////////////
//

FXIMPLEMENT(CCheckComboBox,FXPacker,0,0)

CCheckComboBox::CCheckComboBox(CConnector* pConnector, FXComposite *p,FXint cols,FXint nvis,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb):
  CComboBox(p,cols,nvis,tgt,sel,opts,x,y,w,h, pl,pr,pt,pb)
{
	// CMW Note the 2,0,2,0 padding -- this is necessary to align the "(Select)" with the parent CPropertyItem "(Select)".
	text=new FXTextField(this,cols,this,0,0, 0,0,0,0, 2,0,2,0/*pl,pr,pt,pb*/);
	text->setText("(Select)");
	text->setEditable(FALSE);

	matrix = new FXMatrix(pane,FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y,0,0,0,0);

	CCheckButton* pButton = 0;
	FXString s;
	for (OPTION* pOption = pConnector->pProperty->option(); pOption->pszName; pOption++)
	{
		s = pOption->pszName;
		if ( pOption->pszTip && strcmp(pOption->pszTip, "") )
		{
			s += "\t";
			s += pOption->pszTip;
		}
		pButton = new CCheckButton(this, pConnector, matrix, pOption, 0, 0, s);
	}
}

CCheckComboBox::~CCheckComboBox()
{
	text=(FXTextField*)-1;
	matrix=(FXMatrix*)-1;
}

FXint CCheckComboBox::getDefaultWidth()
{
	FXint ww,pw;
	ww=text->getDefaultWidth()+button->getDefaultWidth()+(border<<1);
	pw=pane->getDefaultWidth();
	return FXMAX(ww,pw);
}

FXint CCheckComboBox::getDefaultHeight()
{
	FXint th,bh;
	th=text->getDefaultHeight();
	bh=button->getDefaultHeight();
	return FXMAX(th,bh)+(border<<1);
}

void CCheckComboBox::updateCheckBoxes(void)
{
	FXint nCount = matrix->getNumRows();
	for ( FXint i = 0; i < nCount; i++ )
	{
		CCheckButton* pButton = (CCheckButton*)matrix->childAtIndex(i); 
		if ( pButton )
		{
			pButton->updateCheckBox();
		}
	}
}

// Recalculate layout
void CCheckComboBox::layout()
{
	FXint buttonWidth,textWidth,itemHeight;
	itemHeight=height-(border<<1);
	buttonWidth=button->getDefaultWidth();
	textWidth=width-buttonWidth-(border<<1);
	text->position(border,border,textWidth,itemHeight);
	button->position(border+textWidth,border,buttonWidth,itemHeight);

	pane->setX(pane->getX()-width);
	pane->resize(width*2,pane->getDefaultHeight());
  
	flags&=~FLAG_DIRTY;
}

///////////////////////////////////////////////////////////////////////////////
//

FXDEFMAP(CListComboBox) CListComboBoxMap[]=
{
	FXMAPFUNC(SEL_KEYPRESS,0,CListComboBox::onKeyPress),
	FXMAPFUNC(SEL_KEYRELEASE,0,CListComboBox::onKeyRelease),
	FXMAPFUNC(SEL_COMMAND,FXWindow::ID_UNPOST,CListComboBox::onCmdUnpost)
};

FXIMPLEMENT(CListComboBox,FXPacker,CListComboBoxMap,ARRAYNUMBER(CListComboBoxMap))

//FXIMPLEMENT(CListComboBox,FXPacker,0,0)

CListComboBox::CListComboBox(CConnector* pConnector, FXComposite *p,FXint cols,FXint nvis,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb):
   CComboBox(p,cols,nvis,tgt,sel,opts,x,y,w,h, pl,pr,pt,pb)
{
	// CMW Note the 2,0,2,0 padding -- this is necessary to align the "(Select)" with the parent CPropertyItem "(Select)".
	text=new FXTextField(this,cols,this,0,0, 0,0,0,0, 2,0,2,0/*pl,pr,pt,pb*/);
	text->setText("(Select)");
	text->setEditable(FALSE);

	list=new FXText(pane);
}

CListComboBox::~CListComboBox()
{
	text=(FXTextField*)-1;
	list=(FXText*)-1;
}

// Recalculate layout
void CListComboBox::layout()
{
	FXint buttonWidth,textWidth,itemHeight;
	itemHeight=height-(border<<1);
	buttonWidth=button->getDefaultWidth();
	textWidth=width-buttonWidth-(border<<1);
	text->position(border,border,textWidth,itemHeight);
	button->position(border+textWidth,border,buttonWidth,itemHeight);

	pane->setX(pane->getX()-width);
	pane->resize(width*2+1,itemHeight*6);
  
//	list->position(1, 1, pane->getWidth()-2/*-list->getBorderWidth()*2*/, pane->getHeight()-2/*-list->getBorderWidth()*2*/);

	flags&=~FLAG_DIRTY;
}

long CListComboBox::onKeyPress(FXObject*,FXSelector,void* ptr)
{
//	if ( m_pParent ) m_pParent->handle(this,MKUINT(message,SEL_KEYPRESS),ptr);
	list->handle(this,MKUINT(message,SEL_KEYPRESS),ptr);
	return 1;
}

long CListComboBox::onKeyRelease(FXObject*,FXSelector,void* ptr)
{
//	if ( m_pParent ) m_pParent->handle(this,MKUINT(message,SEL_KEYRELEASE),ptr);
	list->handle(this,MKUINT(message,SEL_KEYRELEASE),ptr);
	return 1;
}

long CListComboBox::onCmdUnpost(FXObject*,FXSelector,void* ptr)
{
	return 1;
}