/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CConnector.h>

#include <CComboBox.h>
#include <CNameComboBox.h>

///////////////////////////////////////////////////////////////////////////////
//

FXDEFMAP(CNameTextField) CNameTextFieldMap[]=
{
	FXMAPFUNC(SEL_PAINT,0,CNameTextField::onPaint),
	FXMAPFUNC(SEL_FOCUSIN,0,CNameTextField::onFocusIn),
	FXMAPFUNC(SEL_FOCUSOUT,0,CNameTextField::onFocusOut),
};

FXIMPLEMENT(CNameTextField,FXTextField,CNameTextFieldMap,ARRAYNUMBER(CNameTextFieldMap))

CNameTextField::CNameTextField()
:FXTextField()
{
}

CNameTextField::CNameTextField(FXComposite* p,FXint ncols,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb)
: FXTextField(p,ncols,tgt,sel,opts,x,y,w,h,pl,pr,pt, pb)
{
}

long CNameTextField::onFocusIn(FXObject* sender,FXSelector sel,void* ptr)
{
	return 1;
}

long CNameTextField::onFocusOut(FXObject* sender,FXSelector sel,void* ptr)
{
	return 1;
}

long CNameTextField::onPaint(FXObject*,FXSelector,void* ptr)
{
	FXEvent *ev=(FXEvent*)ptr;
	FXDCWindow dc(this,ev);

	FXint x = getX();
	FXint y = getY();
	FXint w = getWidth();
	FXint h = getHeight();
  
	FXString  s = contents;
	FXString  s2 = "And Poo";

	drawFrame(dc,0,0,width,height);

  FXint th = 0;
  if(s.text()) th=getFont()->getFontHeight();
    dc.setForeground(getBackColor());
    dc.fillRectangle(x,y,w,h);

  FXFont* f = getFont();
  FXFont bold(getApp(),f->getName(),f->getSize()/10,FONTWEIGHT_BOLD,f->getSlant(),
	  f->getEncoding(), f->getSetWidth(), f->getHints());
  bold.create();

  if(contents.text()){
    dc.setTextFont(&bold);
    if(!isEnabled())
      dc.setForeground(makeShadowColor(getBackColor()));
    else
      dc.setForeground(getTextColor());
    dc.drawText(x,y+(h-th)/2+getFont()->getFontAscent(),contents.text(),contents.length());
    }

  if(m_strClass.text()){
    dc.setTextFont(getFont());
    if(!isEnabled())
      dc.setForeground(makeShadowColor(getBackColor()));
    else
      dc.setForeground(getTextColor());
    dc.drawText(x+bold.getTextWidth(contents.text(),contents.length())+5,y+(h-th)/2+getFont()->getFontAscent(),m_strClass.text(),m_strClass.length());
    }

	return 1;
}

///////////////////////////////////////////////////////////////////////////////
//

FXIMPLEMENT(CNameListItem,FXListItem,NULL,0)

CNameListItem::CNameListItem(CConnector connector, FXString strName, FXString strClass)
{
	label = strName;
	m_strClass = strClass;
	m_Connector = connector;
}

CNameListItem::~CNameListItem()
{
}

void CNameListItem::draw(const FXList* list,FXDC& dc,FXint x,FXint y,FXint w,FXint h)
{
  FXint ih=0,th=0;

  FXFont* f = list->getFont();
  FXFont bold(list->getApp(),f->getName(),f->getSize()/10,FONTWEIGHT_BOLD,f->getSlant(),
	  f->getEncoding(), f->getSetWidth(), f->getHints());
  bold.create();


  if(icon) ih=icon->getHeight();
  if(!label.empty()) th=list->getFont()->getFontHeight();
  if(isSelected())
    dc.setForeground(list->getSelBackColor());
  else
    dc.setForeground(list->getBackColor());
  dc.fillRectangle(x,y,w,h);
  if(hasFocus()){
    drawFocus(list,dc,x,y,w,h);
    }
//  x+=SIDE_SPACING/2;
  if(icon){
    dc.drawIcon(icon,x,y+(h-ih)/2);
    x+=/*ICON_SPACING+*/icon->getWidth();
    }

  if(!label.empty()){
    dc.setTextFont(&bold);
    if(!isEnabled())
      dc.setForeground(makeShadowColor(list->getBackColor()));
    else if(state&SELECTED)
      dc.setForeground(list->getSelTextColor());
    else
      dc.setForeground(list->getTextColor());
    dc.drawText(x,y+(h-th)/2+list->getFont()->getFontAscent(),label.text(),label.length());
    }

  if(!m_strClass.empty()){
    dc.setTextFont(list->getFont());
    if(!isEnabled())
      dc.setForeground(makeShadowColor(list->getBackColor()));
    else if(state&SELECTED)
      dc.setForeground(list->getSelTextColor());
    else
      dc.setForeground(list->getTextColor());
    dc.drawText(x+bold.getTextWidth(label.text(),label.length())+5,y+(h-th)/2+list->getFont()->getFontAscent(),m_strClass.text(),m_strClass.length());
    }
  }

///////////////////////////////////////////////////////////////////////////////
//

FXDEFMAP(CNameComboBox) CNameComboBoxMap[]=
{
	FXMAPFUNC(SEL_CLICKED,CNameComboBox::ID_LIST,CNameComboBox::onListClicked)
};

FXIMPLEMENT(CNameComboBox,CComboBox,CNameComboBoxMap,ARRAYNUMBER(CNameComboBoxMap))

CNameComboBox::CNameComboBox(CConnector& connector, FXComposite *p,FXint cols,FXint nvis,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb):
  CComboBox(p,cols,nvis,tgt,sel,opts,x,y,w,h, pl,pr,pt,pb)
{
	m_Connector = connector;

	text=new CNameTextField(this,cols,this,0/*CCheckComboBox::ID_TEXT*/,0, 0,0,0,0, 2,0,2,0/*pl,pr,pt,pb*/);
	text->setEditable(FALSE);

	list=new FXList(pane,nvis,this,CNameComboBox::ID_LIST,LIST_BROWSESELECT|LIST_AUTOSELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y|SCROLLERS_TRACK|HSCROLLER_NEVER);
}

CNameComboBox::~CNameComboBox()
{
	text=(CNameTextField*)-1;
}

FXint CNameComboBox::getDefaultWidth()
{
	FXint ww,pw;
	ww=text->getDefaultWidth()+button->getDefaultWidth()+(border<<1);
	pw=pane->getDefaultWidth();
	return FXMAX(ww,pw);
}

FXint CNameComboBox::getDefaultHeight()
{
	FXint th,bh;
	th=text->getDefaultHeight();
	bh=button->getDefaultHeight();
	return FXMAX(th,bh)+(border<<1);
}

void CNameComboBox::layout()
{
	FXint buttonWidth,textWidth,itemHeight;
	itemHeight=height-(border<<1);
	buttonWidth=button->getDefaultWidth();
	textWidth=width-buttonWidth-(border<<1);
	text->position(border,border,textWidth,itemHeight);
	button->position(border+textWidth,border,buttonWidth,itemHeight);
	pane->resize(width,pane->getDefaultHeight());
  
	flags&=~FLAG_DIRTY;
}

void CNameComboBox::create()
{
	CComboBox::create();
	text->create();
	list->create();
}

FXint CNameComboBox::appendItem(CNameListItem* item)
{
	FXint r = list->appendItem(item);
	recalc();
	return r;
}

void CNameComboBox::setCurrentItem(FXint index)
{
	list->setCurrentItem(index);

	if ( 0 <= index )
	{
		CNameListItem* pCurrent = (CNameListItem*)list->retrieveItem(index);
		setText(list->getItemText(index));
		setClass(pCurrent->getClass());
    }
	else
	{
		setText("");
	}
}

void CNameComboBox::setCurrentItem(CConnector& connector, FXbool bMultiple)
{
	if ( bMultiple )
	{
		setText(FXString(""));
		setClass(FXString(""));
		disable();
	}
	else
	{
		enable();

		FXint nitems = list->getNumItems();

		FXint index = 0;

		for( index = 0; index < nitems; index++ )
		{
			CNameListItem* pCurrent = (CNameListItem*)list->retrieveItem(index);
			
			FXWindow* p1 = connector.pWidget;
			FXWindow* p2 = pCurrent->getConnector()->pWidget;
			
			if ( p1 == p2 )
			{
				setCurrentItem(index);
				return;
			}
		}
	}
}

void CNameComboBox::removeItem(CConnector& connector)
{
	FXint nitems = list->getNumItems();

	for( FXint index = 0; index < nitems; index++ )
	{
		CNameListItem* pCurrent = (CNameListItem*)list->retrieveItem(index);
		
		FXWindow* p1 = connector.pWidget;
		FXWindow* p2 = pCurrent->getConnector()->pWidget;
		if ( p1 == p2 )
		{
			removeItem(index);
			return;
		}
	}
}

void CNameComboBox::removeItem(FXint index)
{ 
	FXint current;
	list->removeItem(index);
	current=list->getCurrentItem();
	
	if( 0 <= current )
	{
		setText(list->getItemText(current));
	}
	else
	{
		setText("");
	}
	recalc();
}

long CNameComboBox::onListClicked(FXObject*,FXSelector,void* ptr)
{
	button->handle(this,MKUINT(ID_UNPOST,SEL_COMMAND),NULL);    // Unpost the list

	if ( 0 <= ((FXint)(long)ptr) )
	{ 
		FXint index = (FXint)(long)ptr;

		CNameListItem* pCurrent = (CNameListItem*)list->retrieveItem(index);

		FXString strName = pCurrent->getText();
	    text->setText(strName);

		FXString strClass = pCurrent->getClass();
		text->setClass(strClass);

		handle(this,MKUINT(0,SEL_COMMAND),(void*)strName.text());

		pCurrent->getConnector()->select();
//		target = NULL;
	}

  return 1;
  }


