/**************************************************************************  
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        *
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#ifndef __Property_H__
#define __Property_H__

///////////////////////////////////////////////////////////////////////////////
//

class FXAPI CTextField : public FXTextField
{
	FXDECLARE(CTextField)

// Services
public:
	CTextField() : FXTextField() {}
	CTextField(FXComposite* p, CConnector connector, FXString strProperty);
	virtual ~CTextField();

// Operations
public:
	virtual void setFocus();
	void updateWidget(FXbool bIsWindowRunning = TRUE);

public:
	long onKeyPress(FXObject* o, FXSelector s,void* ptr);
	long onClose(FXObject* o, FXSelector s,void* ptr);

// Attributes
private:
	CConnector  m_Connector;
	FXbool      m_bHadFocus;
};

///////////////////////////////////////////////////////////////////////////////
//

/*
	CPropertyItem provides additional functionality to split
	a standard list item into two: The property label and its
	value. 
	
	CPropertyItem also manages an associated child widget. This
	behaviour is very different from a FXListItem: CPropertyItem
	retains information about the position of the child widget.
*/

#define ICON_SPACING             4    // Spacing between icon and label
#define SIDE_SPACING             6    // Left or right spacing between items
#define LINE_SPACING             4    // Line spacing between items

class FXAPI CPropertyItem : public FXListItem
{
	FXDECLARE(CPropertyItem)
	friend class FXList;

// Services
public:
	CPropertyItem();
	CPropertyItem(CConnector connector);
	virtual ~CPropertyItem();

// Operations
public:
	FXObject* invokeChildWidget(void);
	void closeChildWidget(void);
	void moveChildWidget(FXint x, FXint w);

	void setChildWidget(FXint y);
	FXWindow* getChildWidget(void) {return m_pChildWidget;}

	void close() {closeChildWidget();}

	FXString getTipText(void) {return tip;}

	void refreshProperty(void);
	void refreshDescription(void);

protected:
	virtual void draw(const FXList* list,FXDC& dc,FXint x,FXint y,FXint w,FXint h);
	void drawColumn(FXchar column, FXString& s,const FXList* list,FXDC& dc,FXint x,FXint y,FXint w,FXint h, FXbool bHighlight);

// Events
public:

// Attributes
private:
	FXString m_strProperty;
	FXColor m_Color;

	ePropertyType m_type;

	CConnector m_Connector;

	FXint m_x;
	FXint m_y;
	FXint m_w;
	FXint m_h;
	FXList* m_pList;
	FXWindow* m_pChildWidget;

	FXString tip;
};

///////////////////////////////////////////////////////////////////////////////
//

class FXAPI CPropertyList : public FXList
{
	FXDECLARE(CPropertyList)

// Services
public:
	CPropertyList(FXComposite* p,FXuint opts=0); 

	CPropertyList() : FXList() {}
	CPropertyList(const CPropertyList&) : FXList() {}
	virtual ~CPropertyList();

	virtual FXint getDefaultWidth() {return 150;}

// Operations
public:
	virtual void layout();
	FXbool selectItem(FXint index);

	void refreshProperty(void);
	void clearItems();
	void setup(CConnector& connector, FXbool bMultiple = FALSE);

	void close(void);

// Events
public:
	long onChanged(FXObject* o,FXSelector s,void* ptr);
	long onVScrollerChanged(FXObject*,FXSelector,void* ptr);
	long onVScrollerDragged(FXObject* o,FXSelector s,void* ptr);
	long onLeftBtnPress(FXObject* o,FXSelector s,void* ptr);
	long onLeftBtnRelease(FXObject* o,FXSelector s,void* ptr);
	long onMotion(FXObject* o,FXSelector s,void* ptr);
	long onKeyPress(FXObject*,FXSelector,void* ptr);
	long onKeyRelease(FXObject*,FXSelector,void* ptr);
	long onFocusIn(FXObject* sender,FXSelector sel,void* ptr);
	long onFocusOut(FXObject* sender,FXSelector sel,void* ptr);
	long onEnter(FXObject* sender,FXSelector sel,void* ptr);
	long onLeave(FXObject* sender,FXSelector sel,void* ptr);
	long onQueryTip(FXObject* sender,FXSelector,void*);

// Attributes
private:
	CPropertyItem* m_pCurrentItem;
};

#endif // __Property_H__
