/**************************************************************************  
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include "fxkeys.h" 

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CConnector.h>

#include "CTracker.h"
#include "CTrackManager.h"
#include <CApp.h>
#include "CAppDialog.h"

FXDEFMAP(CTrackManager) CTrackManagerMap[]=
{
	FXMAPFUNC(SEL_COMMAND,CTrackManager::ID_ADDWIDGET,CTrackManager::onAddWidget)
};

FXIMPLEMENT(CTrackManager, FXObject, CTrackManagerMap, ARRAYNUMBER(CTrackManagerMap)) 

CTrackManager::CTrackManager()
{
	m_bOutline = FALSE;
	m_bSnapToGrid = TRUE;
	m_nSnap = 8;

	m_pAppFrame = 0;
	m_pFrame = 0;
	m_pPane = 0;
	
	m_pTrackingWindow = 0;
	m_pCreateWindow = 0;

	m_bMouseDown = FALSE;
	m_bRubberBand = FALSE;

	m_nPrimaryTracker = 0xFFFF;

	m_pAction = 0;
	m_pCurrent = 0;
}

CTrackManager::~CTrackManager()
{
	clearTrackers();
}

void CTrackManager::clearTrackers(void)
{
	for ( FXint i = 0; i < m_Trackers.no(); i++ )
	{
		CTracker* pTracker = m_Trackers[i];
		delete pTracker;
	}
	m_Trackers.clear();
}

void CTrackManager::clearGrabbers(void)
{
	if ( m_pAppFrame )
	{

		FXuint width = m_pAppFrame->getWidth(); 
		FXuint height = m_pAppFrame->getHeight(); 

		if ( width > 0 && height > 0 )
		{
			m_pAppFrame->update(0, 0, width, height); 
		}
	}
	if ( m_pPane )
	{
		m_pPane->update();

		FXuint width = m_pPane->getWidth();
		FXuint height = m_pPane->getHeight(); 

		if ( width > 0 && height > 0 )
		{
			m_pPane->update(0, 0, width, height); 
		}

//		for ( FXint i = 3; i < m_Trackers.no(); i++ )
		for ( FXint i = 0; i < m_Trackers.no(); i++ )
		{
			FXWindow* pWnd = 0;

			if ( !m_Trackers[i]->window() )
				continue;

			pWnd = m_Trackers[i]->window();
			do
			{
				for( FXWindow* p = pWnd->getFirst(); p; p=p->getNext() ) 
				{
					width = p->getWidth(); 
					height = p->getHeight(); 

					if ( width > 0 && height > 0 )
					{
						p->update(0, 0, width, height); 
					}
				} 

				width = pWnd->getWidth(); 
				height = pWnd->getHeight(); 

				if ( width > 0 && height > 0 )
				{
					pWnd->update(0, 0, width, height); 
				}

				pWnd = pWnd->getParent();
			}
			while (pWnd);
		}
/*
		for(FXWindow* pWnd=m_pPane->getFirst(); pWnd; pWnd=pWnd->getNext()) 
		{
			width = pWnd->getWidth(); 
			height = pWnd->getHeight(); 

			if ( width > 0 && height > 0 )
			{
				pWnd->update(); 
			}
		} 
*/

		m_pPane->hide();
		application->forceRefresh();
		m_pPane->show();

	}

}

void CTrackManager::setInactive(void)
{
	for ( FXint i = 0; i < m_Trackers.no(); i++ )
	{
		m_Trackers[i]->setInactive();
	}
}

FXbool CTrackManager::isActive(FXint& nIndex, FXshort& x, FXshort& y)
{
	nIndex = 0xFFFF; 
	for ( FXint i = 0; i < m_Trackers.no(); i++ )
	{
		if ( m_Trackers[i]->isActive() && m_Trackers[i]->isGrabberAtPoint(m_pTrackingWindow, m_nCursorX, m_nCursorY) )
		{
			nIndex = i;
			return TRUE;
		}
	}
	return FALSE;
}

FXbool CTrackManager::isActive(FXint& nIndex)
{
	nIndex = 0xFFFF; 
	for ( FXint i = 0; i < m_Trackers.no(); i++ )
	{
		if ( m_Trackers[i]->isActive() )
		{
			nIndex = i;
			return TRUE;
		}
	}
	return FALSE;
}

void CTrackManager::drawBoundary(FXDCWindow& dc)
{
	for ( FXint i = 0; i < m_Trackers.no(); i++ )
	{
		m_Trackers[i]->drawBoundary(dc);
	}
}

void CTrackManager::snap(FXshort nSnap)
{
	for ( FXint i = 0; i < m_Trackers.no(); i++ )
	{
		m_Trackers[i]->snap(nSnap);
	}
}

void CTrackManager::position(void)
{
	for ( FXint i = 0; i < m_Trackers.no(); i++ )
	{
		m_Trackers[i]->position(m_Trackers.no() == 1 ? FALSE : TRUE);
 
#ifndef WIN32  
	if ( m_Trackers[i]->window() == m_pFrame )
	{ 
		m_Trackers[i]->window()->show(); 
	}
#endif 
	}
}

void CTrackManager::release(void)
{
	for ( FXint i = 0; i < m_Trackers.no(); i++ )
	{
		m_Trackers[i]->release();
	}
}

FXbool CTrackManager::isWindowInList(FXint& nIndex, FXWindow* pWnd)
{
	nIndex = 0xFFFF; 
	for ( FXint i = 0; i < m_Trackers.no(); i++ )
	{
		if ( m_Trackers[i]->window() == pWnd )
		{
			nIndex = i;
			return TRUE;
		}
	}

	return FALSE;
}

void CTrackManager::track(FXint& nIndex)
{
	FXbool bOneTrackerActive = FALSE;
	for ( FXint i = 0; i < m_Trackers.no(); i++ )
	{
		if ( i == nIndex )
		{
			bOneTrackerActive = TRUE;
			m_Trackers[i]->track(TRUE);
		}
		else
		{
			m_Trackers[i]->track(FALSE);
		}
	}

	if ( !bOneTrackerActive && m_Trackers.no() > 0 )
	{
		m_Trackers[0]->track(TRUE);
	}
}

void CTrackManager::track(void)
{
	for ( FXint t = 0; t < m_Trackers.no(); t++ )
	{
		FXWindow* w = m_Trackers[t]->window();
		if ( !w )
		{
			continue;
		}

		m_Trackers[t]->track(m_pTrackingWindow, w, GS_ALL);
	}
}

FXbool CTrackManager::trackDialog(void)
{
	// OnLButtonDown will have already done this but
	// trackDialog can be called from CApp's File|xxx
	// handlers.
	application->isValidWidget(m_pFrame, &m_pCurrent);

	// Clear existing trackers and start tracking the dialog instead.

	FXuint nIndex;

	for ( FXint i = 0; i < m_Trackers.no(); i++ )
	{
		CTracker* pTracker = m_Trackers[i];
		delete pTracker;
	}
	m_Trackers.clear();

	CTracker* pTracker = new CTracker;
	m_Trackers.append(pTracker);
	nIndex = m_Trackers.no()-1;
	m_Trackers[nIndex]->track(m_pAppFrame, m_pFrame, GS_DIALOG);

	m_pTrackingWindow = m_pAppFrame;
	CTracker::getPosition(m_pTrackingWindow, m_pCurrent->pWidget, m_nCursorX, m_nCursorY);
	CTracker::getPosition(m_pTrackingWindow, m_pCurrent->pWidget, m_nOriginalLeft, m_nOriginalTop);

	FXDCWindow dc(m_pTrackingWindow);
	m_Trackers[nIndex]->drawBoundary(dc);

	m_nPrimaryTracker = 0;

	FXWindow* temp = m_pCurrent->pWidget;
	m_pCurrent->pWidget = m_pFrame;
	m_pCurrent->select();
	m_pCurrent->pWidget = temp;

	return TRUE;
}

FXbool CTrackManager::isGrabberAtPoint2(FXWindow* window, FXshort& x, FXshort& y, FXint& nIndex, eGrabber& grabber)
{
	FXshort tx = x;
	FXshort ty = y;

	CTracker::getAbsolutePosition(window, tx, ty);

	for ( FXint t = 0; t < m_Trackers.no(); t++ )
	{
		FXWindow* w = m_Trackers[t]->window();
		if ( !w )
		{
			continue;
		}

		FXshort tlx = /*w->getX()*/-7; //left
		FXshort tly = /*w->getY()*/-7; //top
		CTracker::getAbsolutePosition(w, tlx, tly);

		FXshort brx = /*w->getX() +*/ w->getWidth() + 14;
		FXshort bry = /*w->getY() +*/ w->getHeight() + 14;
		CTracker::getAbsolutePosition(w, brx, bry);

		if	(
			tx >= tlx && tx <= brx &&
			ty >= tly && ty <= bry
			)
		{
			// Now find specific grabber
			for ( FXint g = 0; g < 8; g++ )
			{
				switch (g)
				{
				case TOP_LEFT:
					tlx = /*w->getX()*/ - 7;//left
					tly = /*w->getY()*/ - 7;
					break;
				case TOP_CENTRE:
					tlx = /*w->getX() +*/ w->getWidth()/2;
					tly = /*w->getY()*/ - 7;
					break;
				case TOP_RIGHT:
					tlx = /*w->getX() +*/ w->getWidth();
					tly = /*w->getY()*/ - 7;
					break;
				case CENTRE_LEFT:
					tlx = /*w->getX()*/ - 7;//left
					tly = /*w->getY() +*/ w->getHeight()/2-4;
					break;
				case CENTRE_RIGHT:
					tlx = /*w->getX() +*/ w->getWidth();
					tly = /*w->getY() +*/ w->getHeight()/2-4;
					break;
				case BOTTOM_LEFT:
					tlx = /*w->getX()*/ - 7;//left
					tly = /*w->getY() +*/ w->getHeight();
					break;
				case BOTTOM_CENTRE:
					tlx = /*w->getX() +*/ w->getWidth()/2;
					tly = /*w->getY() +*/ w->getHeight();
					break;
				case BOTTOM_RIGHT:
					tlx = /*w->getX() +*/ w->getWidth();
					tly = /*w->getY() +*/ w->getHeight();
					break;
				};

				CTracker::getAbsolutePosition(w, tlx, tly);

				if	(
					m_Trackers[t]->getGrabber((eGrabber)g).bActive &&
					tx >= tlx && tx <= (tlx + 7) &&
					ty >= tly && ty <= (tly + 7)
					)
				{
					nIndex  = t;
					grabber = (eGrabber)g;
					return TRUE;
				}
			}
		}
	}

	return FALSE;
}

FXbool CTrackManager::onLButtonDown(FXWindow* window, FXint x, FXint y, FXbool bCtrl)
{
	if ( !application )
	{
		return FALSE;
	}

	m_pTrackingWindow = m_pPane;
/*
	if ( m_pCurrent )
	{
		CConnector* pParentTest = 0;
		application->isValidWidget(m_pCurrent->pWidget->getParent(), &pParentTest);
		if ( pParentTest )
		{
			m_pTrackingWindow = m_pCurrent->pWidget->getParent();
		}
	}
*/
/*
	// Safety check -- user may not have selected anything yet
	if ( m_pCurrent == 0 )
		application->isValidWidget(m_pFrame, &m_pCurrent);

	m_pTrackingWindow = m_pCurrent->pWidget;
*/


	if ( m_pCurrent )
	{
		CConnector* pDialog = 0;
		application->isValidWidget(m_pFrame, &pDialog);
		if ( m_pCurrent != pDialog )
		{
			m_pCurrent->select(m_Trackers.no() == 1 ? FALSE : TRUE);
			m_pCurrent = 0;
		}
	}

	if ( !application->isValidWidget(window, &m_pCurrent) )
	{
		/* CMW 11.8.99	Since the changes to isValidWidget this entire
						else clause is never executed */
		if ( application->isValidWidget(window->getParent(), &m_pCurrent) )
		{
			m_pCurrent->pWidget = window->getParent();
			if ( m_pCurrent->pWidget == m_pFrame )
			{
				// Win32 - clicked on close\min\max\restore buttons of 'dialog'
				m_bMouseDown = TRUE;
				return trackDialog();
			}
		}
		else
		{
			return FALSE;
		}
	}

	//m_pTrackingWindow = m_pCurrent->pWidget;

	if ( window != m_pCurrent->pWidget )
	{
		// CMW 08.03.00 
		CConnector* pconnParent = 0;
		application->isValidWidget(m_pCurrent->pWidget->getParent(), &pconnParent);
		if ( pconnParent )
		{
			// CMW 7.7.00
			m_pTrackingWindow = pconnParent->pWidget;
	//		m_pTrackingWindow = m_pCurrent->pDispenser->getCreateParent(m_pCurrent->pWidget);
		}
		else
		// CMW 24.11.99 If m_pTrackingWindow hasn't been set its because the user
		//				hasn't selected a widget yet. Default to the dialog itself.
	//	if ( !m_pTrackingWindow )
		{
			m_pTrackingWindow = m_pAppFrame;
		}
	}


	/* CMW 11.8.99	This bit from the above else clause is still needed */
	if ( m_pCurrent->pWidget == m_pFrame )
	{
		// Win32 - clicked on close\min\max\restore buttons of 'dialog'
		m_bMouseDown = TRUE;
		return trackDialog();
	}

	m_nCursorX = x;
	m_nCursorY = y;

	/* CMW 11.8.99	window MUST be used in this call to getPosition
					here, not pValid because it must match a similar
					call to getPosition in onMouseMove. Failing to
					do so causes widgets to jump as you traverse child
					widgets. The key is they both use m_nCursorX&Y.
	*/
	CTracker::getPosition(m_pTrackingWindow, window, m_nCursorX, m_nCursorY);

	clearGrabbers();

	eGrabber grabber;
	if ( isGrabberAtPoint2(window, (FXshort&)x, (FXshort&)y, m_nPrimaryTracker, grabber) )
	{
////		m_pTrackingWindow = m_Trackers[m_nPrimaryTracker]->frame();

		//
		// 7.7.00 This is crucial. Before the call to getConnector m_pCurrent points
		// to the window underneath the grabber which is NOT the current widget!
		//
		m_pCurrent = getConnector(m_nPrimaryTracker);

		m_pCurrent->select(m_Trackers.no() == 1 ? FALSE : TRUE);

//		if ( m_pCurrent->pWidget == m_pFrame )
//			m_pTrackingWindow = m_pAppFrame;

	// Safety check -- user may not have selected anything yet
//	if ( m_pCurrent == 0 )
//		application->isValidWidget(m_pFrame, &m_pCurrent);

	// CMW 08.03.00 
/*	CConnector* pconnParent = 0;
	application->isValidWidget(m_pCurrent->pWidget->getParent(), &pconnParent);
	if ( pconnParent )
	{
		// CMW 7.7.00
		m_pTrackingWindow = pconnParent->pWidget;
//		m_pTrackingWindow = m_pCurrent->pDispenser->getCreateParent(m_pCurrent->pWidget);
	}
	else
	// CMW 24.11.99 If m_pTrackingWindow hasn't been set its because the user
	//				hasn't selected a widget yet. Default to the dialog itself.
//	if ( !m_pTrackingWindow )
	{
		m_pTrackingWindow = m_pAppFrame;
	}
*/

		m_bMouseDown = TRUE;

		m_nCursorX = x; 
		m_nCursorY = y; 
		CTracker::getPosition(m_pTrackingWindow, window, m_nCursorX, m_nCursorY); 

		/* CMW 22.10.99 Added next line so that clicking on a selected widget
						will still recalc the widgets position as this may
						now have changed with the introduction of the Layout property.
		*/
//		m_pPane->recalc();
//		m_Trackers[m_nPrimaryTracker]->track(m_pTrackingWindow, window, GS_ALL);

		FXDCWindow dc(m_pTrackingWindow);
		m_Trackers[m_nPrimaryTracker]->track(m_pTrackingWindow, m_Trackers[m_nPrimaryTracker]->window(), GS_ALL);
		m_Trackers[m_nPrimaryTracker]->drawBoundary(dc);
		m_Trackers[m_nPrimaryTracker]->setGrabber(grabber);
		return TRUE;
	}

	if ( window == /*m_pTrackingWindow && m_pTrackingWindow ==*/ m_pPane )
	{
		// User clicked on the form itself so clear existing
		// trackers, enable rubber band and start tracking
		// the dialog instead.
		m_bRubberBand = TRUE;

		m_bMouseDown = TRUE;
		trackDialog(); // CMW 30.9. was m_pFrame

		CTracker* pTracker = new CTracker;
		m_Trackers.append(pTracker);

		m_nCursorX = x; 
		m_nCursorY = y;
		
		// CMW 02.12.99 Required to make rubber band on form work correctly
		if ( m_pCurrent->pWidget != m_pFrame )
		{
			CTracker::getPosition(m_pPane, m_pCurrent->pWidget, m_nCursorX, m_nCursorY); 
		}

		if ( m_bSnapToGrid && m_pAction )
		{
			CTracker::snap(m_nCursorX, m_nSnap);
			CTracker::snap(m_nCursorY, m_nSnap);
		}

		m_nOriginalLeft = m_nCursorX;
		m_nOriginalTop = m_nCursorY;

		// Magic Number 1 is used as trackDialog has cleared tracker 
		// and added m_pFrame
		m_nRubberBandIndex = 1;
		m_Trackers[m_nRubberBandIndex]->track(m_nCursorY, m_nCursorX);

		// CMW 27.9.99 Needed for FXGroupBox
		m_pCreateWindow   = m_pPane;

		return TRUE;
	}
	else
	{
		if ( window == m_pFrame )
		{
			// Linux\Win32 - I've clicked on the title bar of the dialog box
			m_bMouseDown = TRUE;
			return trackDialog();
		}
		else if ( window == m_pAppFrame )
		{
			m_pTrackingWindow = m_pAppFrame;
		}

		m_bMouseDown = TRUE;

		// CMW 13.10.99 Added && !m_pAction test. If you had selected a GroupBox
		//				and then tried to draw a new widget on it the program
		//				selected the GroupBox again. This change fixes that.
		if ( isWindowInList(m_nPrimaryTracker, m_pCurrent->pWidget) && !m_pAction )
		{
			if ( bCtrl )
			{
				CTracker* pTracker = m_Trackers[m_nPrimaryTracker];
				delete pTracker;

				m_Trackers.remove(m_nPrimaryTracker);
			}

			// CMW 28.10.99 User has deselected all the trackers
			//				so track the dialog instead. pValid, PDispenser
			//				etc are for the widget thats just been deselected
			//				so we need to reestablish them for the dialog
			if ( m_Trackers.no() == 0 )
			{
				application->isValidWidget(m_pFrame, &m_pCurrent);
				m_bMouseDown = TRUE;
				return trackDialog();
			}

			// CMW 08.03.00 
//			CConnector* pconnParent = 0;
//			application->isValidWidget(m_pCurrent->pWidget->getParent(), &pconnParent);
//			if ( pconnParent )
////			if ( m_pCurrent->pWidget->getParent()->isMemberOf(&FXGroupBox::metaClass) )
//			{
//				m_pTrackingWindow = m_pCurrent->pWidget->getParent();
//			}
////			else
////			{
////				m_pTrackingWindow = m_pPane;
////			}
			/* CMW 22.10.99 Added next line so that clicking on a selected widget
							will still recalc the widgets position as this may
							now have changed with the introduction of the Layout property.
			*/

			if ( window == m_pTrackingWindow )
			{
				CConnector* pconnParent = 0;
				application->isValidWidget(m_pCurrent->pWidget->getParent(), &pconnParent);
				if ( pconnParent )
				{
					m_pTrackingWindow = m_pCurrent->pWidget->getParent();
				}
			}

			// Check array bounds
			if ( m_nPrimaryTracker == m_Trackers.no() )
				m_nPrimaryTracker--;

			// We may have just deselected the current widget from
			// a group of selected widgets with the CTRL key
			application->isValidWidget(m_Trackers[m_nPrimaryTracker]->window(), &m_pCurrent);

			m_Trackers[m_nPrimaryTracker]->track(m_pTrackingWindow, m_pCurrent->pWidget, GS_ALL);
			track(m_nPrimaryTracker);
			m_nCursorX = x; 
			m_nCursorY = y; 
			CTracker::getPosition(m_pTrackingWindow, window, m_nCursorX, m_nCursorY); 

			//setupProperties(m_pCurrent->pWidget, m_pCurrent->pDispenser, m_pCurrent->pProperties);
			//m_pCurrent->pProperties->setup(*m_pCurrent);
			m_pCurrent->select(m_Trackers.no() == 1 ? FALSE : TRUE);
		}
		else
		{
//			if ( window == m_pPane )
//			{
//				// Linux\Win32 - I've clicked on the title bar of the dialog box
//				m_bMouseDown = TRUE;
//				return trackDialog();
//			}
/////				/* CMW 11.8.99	window MUST be used in this call to getPosition
/////								here, not pValid because it must match a similar
/////								call to getPosition in onMouseMove. Failing to
/////								do so causes widgets to jump as you traverse child
/////								widgets. The key is they both use m_nCursorX&Y.
/////				*/
/////			// CMW 08.03.00 
/////			CConnector* pconnParent = 0;
/////			application->isValidWidget(m_pCurrent->pWidget->getParent(), &pconnParent);
/////			if ( pconnParent )
///////			if ( m_pCurrent->pWidget->getParent()->isMemberOf(&FXGroupBox::metaClass) )
/////			{
/////				m_pTrackingWindow = m_pCurrent->pWidget->getParent();
/////			}

	// Safety check -- user may not have selected anything yet
	if ( m_pCurrent == 0 )
		application->isValidWidget(m_pFrame, &m_pCurrent);

	// CMW 08.03.00 
	CConnector* pconnParent = 0;
	application->isValidWidget(m_pCurrent->pWidget->getParent(), &pconnParent);
	if ( pconnParent )
	{
		// CMW 7.7.00
		m_pTrackingWindow = pconnParent->pWidget;
//		m_pTrackingWindow = m_pCurrent->pDispenser->getCreateParent(m_pCurrent->pWidget);
	}
	else
	// CMW 24.11.99 If m_pTrackingWindow hasn't been set its because the user
	//				hasn't selected a widget yet. Default to the dialog itself.
//	if ( !m_pTrackingWindow )
	{
		m_pTrackingWindow = m_pAppFrame;
	}


			m_nCursorX = x; 
			m_nCursorY = y; 
			CTracker::getPosition(m_pTrackingWindow, window, m_nCursorX, m_nCursorY); 

//BEGIN		if	(
//				m_nPrimaryTracker == 0xFFFF ||
//				!isGrabberAtPoint(m_nPrimaryTracker, m_nCursorX, m_nCursorY)
//				)
//			{
/*INSTEAD*/		m_nPrimaryTracker = 0xFFFF;
				/*CMW 11.8.99 [1] Added this if due to changes in isValidWidget*/
				if ( window != m_pAppFrame )
				{
					if ( !bCtrl || m_pAction )
					{
						for ( FXint i = 0; i < m_Trackers.no(); i++ )
						{
							CTracker* pTracker = m_Trackers[i];
							delete pTracker;
						}
						m_Trackers.clear();
					}

					if ( m_pAction )
					{
						// We've clicked inside a container widget?
						m_bRubberBand = TRUE;

						CTracker* pTracker = new CTracker;
						m_Trackers.append(pTracker);

						m_pTrackingWindow = window;
						// CMW 27.9.99 Needed for FXGroupBox
						m_pCreateWindow   = window;

						m_nCursorX = x; 
						m_nCursorY = y;

						if ( m_bSnapToGrid && m_pAction )
						{
							CTracker::snap(m_nCursorX, m_nSnap);
							CTracker::snap(m_nCursorY, m_nSnap);
						}

						m_nOriginalLeft = m_nCursorX;
						m_nOriginalTop = m_nCursorY;

						m_nRubberBandIndex = 0;
						m_Trackers[m_nRubberBandIndex]->track(m_nCursorY, m_nCursorX);

					}
					else
					{
						// CMW 22.11.99 Check that new tracker has same parent as current trackers
						if	(
							m_Trackers.no() == 0 ||
							(m_Trackers.no() > 0 && m_pCurrent->pWidget->getParent() == m_Trackers[0]->window()->getParent())
							)
						{
							// CMW 27.9.99 m_pPane replaced with pValid->getParent()
							//             for FXGroupBox
							CTracker* pTracker = new CTracker;
							m_Trackers.append(pTracker);
							m_nPrimaryTracker = m_Trackers.no()-1;

							/* CMW 29.9 changed window to pValid */
							m_Trackers[m_nPrimaryTracker]->track(m_pTrackingWindow, m_pCurrent->pWidget, GS_ALL);
							track(m_nPrimaryTracker);

							m_pCurrent->select(m_Trackers.no() == 1 ? FALSE : TRUE);
						}
					}
				}
//END		}
		}

		if ( window == m_pAppFrame ) 
		{ 
			m_pTrackingWindow = m_pAppFrame; 
		}

//		::getPosition(m_pTrackingWindow, window, m_nOriginalLeft, m_nOriginalTop);

		/* CMW 11.8.99 Added this test on nIndex due to check -- see [1] above */ 
		if ( m_nPrimaryTracker != 0xFFFF )
		{
			FXDCWindow dc(m_pTrackingWindow);
			m_Trackers[m_nPrimaryTracker]->drawBoundary(dc);
		}
		else 
		{
			m_bMouseDown = FALSE;
		}
	}

	return TRUE;
}

void CTrackManager::select(CConnector& connector, FXbool bMultiple)
{
	if ( !bMultiple )
	{
		clearGrabbers();

		for ( FXint i = 0; i < m_Trackers.no(); i++ )
		{
			CTracker* pTracker = m_Trackers[i];
			delete pTracker;
		}
		m_Trackers.clear();
	}

	m_pCurrent = &connector;

	// Safety check -- user may not have selected anything yet
	if ( m_pCurrent == 0 )
		application->isValidWidget(m_pFrame, &m_pCurrent);

	// CMW 08.03.00 
	CConnector* pconnParent = 0;
	application->isValidWidget(m_pCurrent->pWidget->getParent(), &pconnParent);
	if ( pconnParent )
	{
		// CMW 7.7.00
		m_pTrackingWindow = pconnParent->pWidget;
//		m_pTrackingWindow = m_pCurrent->pDispenser->getCreateParent(m_pCurrent->pWidget);
		m_pTrackingWindow = pconnParent->pDispenser->getCreateParent(pconnParent->pWidget);
	}
	else
	// CMW 24.11.99 If m_pTrackingWindow hasn't been set its because the user
	//				hasn't selected a widget yet. Default to the dialog itself.
//	if ( !m_pTrackingWindow )
	{
		m_pTrackingWindow = m_pAppFrame;
	}

	if ( !bMultiple )
	{
		CTracker* pTracker = new CTracker;
		m_Trackers.append(pTracker);
		m_nPrimaryTracker = m_Trackers.no()-1;

		m_Trackers[m_nPrimaryTracker]->track(m_pTrackingWindow, m_pCurrent->pWidget, m_pCurrent->pDispenser->getWidgetGrabbers());
	}
}

FXbool CTrackManager::onMouseMove(FXWindow* window, FXint x, FXint y)
{
	static FXint oldx = 0;
	static FXint oldy = 0;

	// CMW 24.05.00 Why do we get WM_MOUSEMOVE when I haven't moved the mouse ??
	if ( x == oldx && y == oldy )
	{
		return TRUE;
	}
	else
	{
		oldx = x;
		oldy = y;
	}
/*
	// Safety check -- user may not have selected anything yet
	if ( m_pCurrent == 0 )
		application->isValidWidget(m_pFrame, &m_pCurrent);

	// CMW 08.03.00 
	CConnector* pconnParent = 0;
	application->isValidWidget(m_pCurrent->pWidget->getParent(), &pconnParent);
	if ( pconnParent )
	{
		// CMW 7.7.00
		m_pTrackingWindow = pconnParent->pWidget;
//		m_pTrackingWindow = m_pCurrent->pDispenser->getCreateParent(m_pCurrent->pWidget);
	}
	else
	// CMW 24.11.99 If m_pTrackingWindow hasn't been set its because the user
	//				hasn't selected a widget yet. Default to the dialog itself.
//	if ( !m_pTrackingWindow )
	{
		m_pTrackingWindow = m_pAppFrame;
	}
*/

	
	if ( m_bRubberBand )
	{
		// CMW 13.10.99 Added this test. Without it m_pPane would work for everything
		//				except a rubber band on the dialog itself. m_TR would work
		//				for dialog but all other track rectangles would be wrong
		FXWindow* p = m_pTrackingWindow;
		if ( p == m_pAppFrame )
		{
			p = m_pPane;
		}
		// CMW 27.09.99 Changed for FXGroupBox
		// CMW 12.10.99 m_pTrackingWindow -> m_pPane: The rubber band can only ever 
		//				be activated on the pane.					
		FXDCWindow dc(p);

		m_Trackers[m_nRubberBandIndex]->drawBoundary(dc); 

		FXshort nCol = x;
		FXshort nRow  = y;

		// CMW 27.09.99 m_pPane -> m_pTrackingWindow for FXGroupBox
		// CMW 12.10.99 m_pTrackingWindow -> m_pPane: The rubber band can only ever 
		//				be activated on the pane.					
		CTracker::getPosition(p, window, nCol, nRow);

		if ( m_bSnapToGrid && m_pAction )
		{
			CTracker::snap(nCol, m_nSnap);
			CTracker::snap(nRow, m_nSnap);
			CTracker::snap(m_nCursorX, m_nSnap);
			CTracker::snap(m_nCursorY, m_nSnap);
		}

		m_Trackers[m_nRubberBandIndex]->move((nCol - m_nCursorX), (nRow - m_nCursorY), false);
		m_Trackers[m_nRubberBandIndex]->drawBoundary(dc);

		m_nCursorX = nCol;
		m_nCursorY = nRow;

		return TRUE;
	}

	if ( m_bMouseDown )
	{
//		if (!m_pTrackingWindow->id())
//		{
//			return TRUE;
//		}

		FXDCWindow dc(m_pTrackingWindow);
		if ( m_bOutline )
		{
			drawBoundary(dc);
		}
		else
		{
//			clearGrabbers();
		}

		// Calculate new position
		FXshort nCol = x;
		FXshort nRow  = y;

		CTracker::getPosition(m_pTrackingWindow, window, nCol, nRow);

		if ( m_bSnapToGrid && m_pTrackingWindow != m_pAppFrame )
		{
			CTracker::snap(nCol, m_nSnap);
			CTracker::snap(nRow, m_nSnap);
			CTracker::snap(m_nCursorX, m_nSnap);
			CTracker::snap(m_nCursorY, m_nSnap);
		}

		// CMW 26.10.99 Changed the move logic below which caused resizing
		//				to go wrong for widgets within frames

		// Is window a child of m_pTrackingWindow?
		FXWindow* w = window;

		do
		{
			if ( w == m_pTrackingWindow )
			{
				for ( FXint i = 0; i < m_Trackers.no(); i++ )
				{
					m_Trackers[i]->move((nCol - m_nCursorX), (nRow - m_nCursorY));
				}
			}
			w = w->getParent();
		}
		while (w);

		if ( m_bSnapToGrid && m_pTrackingWindow != m_pAppFrame )
		{
			snap(m_nSnap);
		}

		if ( m_bOutline )
		{
			drawBoundary(dc);
		}
		else
		{
//			clearGrabbers();
			release();
			position();
		}

		m_nCursorX = nCol;
		m_nCursorY = nRow;

		return TRUE;
	}

	FXshort nCol = x;
	FXshort nRow  = y;

	CTracker::getPosition(m_pTrackingWindow, window, nCol, nRow);
	
	eGrabber grabber;
	FXint nIndex = 0;
	if ( isGrabberAtPoint2(window, (FXshort&)x, (FXshort&)y, nIndex, grabber) )
	{
		switch ( grabber )
		{
		case TOP_LEFT:
		case BOTTOM_RIGHT:
			window->setDefaultCursor(window->getApp()->dragTLCursor);
			break;

		case TOP_RIGHT:
		case BOTTOM_LEFT:
			window->setDefaultCursor(window->getApp()->dragTRCursor);
			break;

		case TOP_CENTRE:
		case BOTTOM_CENTRE:
			window->setDefaultCursor(window->getApp()->dragHCursor);
			break;

		case CENTRE_LEFT:
		case CENTRE_RIGHT: 
			window->setDefaultCursor(window->getApp()->dragVCursor);
			break;

		case ALL:
			window->setDefaultCursor(window->getApp()->moveCursor);
			break;

		case NONE:
			window->setDefaultCursor(window->getApp()->arrowCursor);
			break;
		};
	}
	else
	{
		window->setDefaultCursor(window->getApp()->arrowCursor);
	}

	return TRUE;
}

FXbool CTrackManager::onLButtonUp(FXWindow* window, FXint x, FXint y) 
{
	if ( m_bRubberBand )
	{
		// Invert the rubber band coordinates if the user
		// has drawn an 'up-side-down' rubber band
		if ( m_nOriginalLeft > m_nCursorX )
		{
			FXshort temp = m_nCursorX;
			m_nCursorX = m_nOriginalLeft;
			m_nOriginalLeft = temp;
		}
		if ( m_nOriginalTop > m_nCursorY )
		{
			FXshort temp = m_nCursorY;
			m_nCursorY = m_nOriginalTop;
			m_nOriginalTop = temp;
		}

		CConnector* pParentConnector = 0;
		application->isValidWidget(m_pCreateWindow, &pParentConnector);

		if ( m_pAction )
		{
			// Destroy any trackers
			clearGrabbers();
			for ( FXint i = 0; i < m_Trackers.no(); i++ )
			{
				CTracker* pTracker = m_Trackers[i];
				delete pTracker;
			}
			m_Trackers.clear();

			//
			// CMW 06.03.00 Use the CConnectors (m_pAction is the child window under
			//				construction, pParentConnector it's parent) to:
			//
			//				1. Verify that a child will accept the parent as a parent.
			//
			//				2. Establish if the parent accepts children of the same
			//				   type as the child under construction.
			//				   
			//				   FXLabel's, for example, won't accept children at all
			//				   while an FXShutter will only accept an FXShutterItem
			//				   as a child.
			//
			//				These two checks are essentially the same, one from
			//				the child's perspective and one from the parent's.
			//				I've done this deliberately to provide flexibility
			//

			if ( m_pTrackingWindow == m_pAppFrame || !m_pCurrent->pDispenser->canHaveChildren() )
				m_pCreateWindow = m_pPane;
			else
				m_pCreateWindow = m_pCurrent->pWidget;

			if	(
				m_pAction->pDispenser->allowParent(m_pCreateWindow) &&
				pParentConnector->pDispenser->allowChild(m_pAction->pDispenser->getClassInfo())
				)
			{
				addWidget(/*pParentConnector*/m_pCurrent->pDispenser->getCreateParent(m_pCreateWindow), TRUE, TRUE,
						m_nOriginalLeft, m_nOriginalTop, m_nCursorX - m_nOriginalLeft, m_nCursorY - m_nOriginalTop);
			}
			else
			{
				m_nPrimaryTracker = m_Trackers.no()-1;
				m_pAction->releaseButtons();
				m_pAction = NULL;
			}
			
			clearGrabbers();
		}
		
		if ( !m_pAction )
		{
			FXshort wx = 0;
			FXshort wy = 0;
			FXshort wh = 0;
			FXshort ww = 0;

			FXbool bSelected = FALSE;

			FXWindow* pParent = 0;

			// Select the widgets bounded by the tracker
			// CMW 28.10.99 Start at 3 here -- we don't need to select m_pPane etc!
			for ( FXint i = 3; i < application->m_Widgets.no(); i++ )
			{
				FXWindow* pWindow = application->m_Widgets[i]->pWidget;

				if ( pParent == 0 )
				{
					pParent = pWindow->getParent();
				}

				if ( pWindow->getParent() != pParent )
				{
					continue;
				}

				if	(
					pWindow != m_pAppFrame &&
					pWindow != m_pFrame &&
					pWindow != m_pPane
					)
				{
					wx = pWindow->getX();
					wy = pWindow->getY();
					wh = pWindow->getHeight();
					ww = pWindow->getWidth();

					if	(
						(m_nOriginalLeft < wx    && m_nCursorX   > wx) &&
						(m_nOriginalTop  < wy    && m_nCursorY   > wy) &&
						(m_nOriginalLeft < wx+ww && m_nCursorX > wx+ww) &&
						(m_nOriginalTop  < wy+wh && m_nCursorY > wy+wh)
						)
					{
						CTracker* pTracker = new CTracker;
						m_Trackers.append(pTracker);
						m_nPrimaryTracker = m_Trackers.no()-1;
						m_Trackers[m_nPrimaryTracker]->track(m_pPane, pWindow, GS_ALL);
						track(m_nPrimaryTracker);
						m_pTrackingWindow = m_pPane;

						bSelected = TRUE;
					}
				}
			}

			// Destroy the rubber band tracker
			if ( m_Trackers.no() )
			{
				CTracker* pTracker = m_Trackers[1];
				delete pTracker;
				m_Trackers.remove(1);
			}

			// Destroy dialog tracker?
			if ( bSelected )
			{
				CTracker* pTracker = m_Trackers[0];
				delete pTracker;
				m_Trackers.remove(0);
			}

			m_nPrimaryTracker = m_Trackers.no()-1;
		}

		m_bRubberBand = FALSE;
	}

	if ( m_bMouseDown == TRUE )
	{
		clearGrabbers();

		m_bMouseDown = FALSE; 

		release();
		position();

		// Reestablish m_pCurrent
		if ( m_nPrimaryTracker == -1 )
			trackDialog();

		application->isValidWidget(m_Trackers[m_nPrimaryTracker]->window(), &m_pCurrent);

		if ( m_pCurrent->pProperties )
		{
			m_pCurrent->select(m_Trackers.no() == 1 ? FALSE : TRUE);
		}
	}

	recalc();

	m_pAction = NULL;

	return TRUE;
}

void CTrackManager::drawGrabber(FXint index)
{
	if ( index == -1 )
		return;

	FXWindow* w = m_Trackers[index]->window();

	if ( !w )
		return;

	FXDCWindow dc(w->getParent());
	dc.clipChildren(FALSE);

	FXColor white = FXRGB(255, 255, 255);
	FXColor blue  = FXRGB(  0,   0, 128);

	FXshort tlx = -7; //left
	FXshort tly = -7; //top

	for ( FXint g = 0; g < 8; g++ )
	{
		switch (g)
		{
		case TOP_LEFT:
			tlx = - 7;//left
			tly = - 7;
			break;
		case TOP_CENTRE:
			tlx = w->getWidth()/2-3;
			tly = - 7;
			break;
		case TOP_RIGHT:
			tlx = w->getWidth();
			tly = - 7;
			break;
		case CENTRE_LEFT:
			tlx = - 7;
			tly = w->getHeight()/2-4;
			break;
		case CENTRE_RIGHT:
			tlx = w->getWidth();
			tly = w->getHeight()/2-4;
			break;
		case BOTTOM_LEFT:
			tlx = - 7;//left
			tly = w->getHeight();
			break;
		case BOTTOM_CENTRE:
			tlx = w->getWidth()/2-3;
			tly = w->getHeight();
			break;
		case BOTTOM_RIGHT:
			tlx = w->getWidth();
			tly = w->getHeight();
			break;
		};

		CTracker::getPosition(w->getParent(), w, tlx, tly);

		if ( m_Trackers[index]->getGrabber((eGrabber)g).bActive )
		{
			dc.setForeground(white);
			dc.fillRectangle(tlx, tly, 7, 7);

			dc.setForeground(blue);
			dc.fillRectangle(tlx+1, tly+1, 5, 5);
		}
		else
		{
			dc.setForeground(blue);
			dc.fillRectangle(tlx, tly, 7, 7);

			dc.setForeground(white);
			dc.fillRectangle(tlx+1, tly+1, 5, 5);
		}
	}
}

void CTrackManager::drawGrabbers(void)
{
	if ( !m_pTrackingWindow )
		return;

	for (FXint i = 0; i < m_Trackers.no(); i++ )
	{
		// Don't draw the primary tracker now...
		if ( i != m_nPrimaryTracker )
		{
			drawGrabber(i);
		}
	}

	// .. do it here so that the primary tracker is fully drawn.
	if ( m_nPrimaryTracker != 0xFFFF )
	{
		drawGrabber(m_nPrimaryTracker);
	}
}

void CTrackManager::onPaint(FXWindow* window)
{
	if ( m_bSnapToGrid && m_pPane && window == m_pPane /*&& !m_bMouseDown*/ )
	{
		FXDCWindow dc(m_pPane);

//		dc.setForeground(m_pFrame->getBackColor());
//		dc.fillRectangle(0, 0, m_pPane->getWidth(), m_pPane->getHeight());

		dc.setForeground(FXRGB(0,0,0));

		for ( FXint nRow = 0; nRow < m_pPane->getWidth(); nRow += m_nSnap ) 
		{
			for ( FXint nCol = 0; nCol < m_pPane->getHeight(); nCol += m_nSnap ) 
			{
				dc.fillRectangle(nRow, nCol, 1, 1);
			}
		}
	}

	if ( window == m_pTrackingWindow && !m_bRubberBand && m_bMouseDown && m_bOutline )
	{
		FXDCWindow dc(m_pTrackingWindow);
		drawBoundary(dc);
	}

	if ( !m_bMouseDown )
	{
		drawGrabbers();
	}
}

FXbool CTrackManager::OnKeyDown(FXWindow* window, FXuint key, FXuint code)
{
	if ( key == KEY_Delete || key == KEY_KP_Delete )
	{
		for (FXint i = m_Trackers.no()-1; i >= 0; i-- )
		{
			FXWindow* pWindow = m_Trackers[i]->window();
			
			CConnector* pConnector = 0;
			application->getWidgetIndex(pWindow, &pConnector);

			application->clearWidget(*pConnector);
		}

		application->refreshWidgetTree();

		clearTrackers();
	
		trackDialog();
	}

	return 0;
}

FXbool CTrackManager::OnKeyUp(FXWindow* window, FXuint key, FXuint code)
{
//	CConnector* pDummy = 0;
//	if ( application->isValidWidget(window, &pDummy) )
	if ( !isMultipleSelected() )
	{
		if ( m_pCurrent )
		{
			m_pCurrent->select();
		}
	}

//	if ( !application->isCtrlPressed() )
//		m_pCurrent->update("Text");

	return 0;
}

void CTrackManager::resetAction(void)
{
	if ( m_pAction )
	{
		m_pAction->pButton->setState(STATE_UP);
		m_pAction = NULL;
	}
}

long CTrackManager::onAddWidget(FXObject* o,FXSelector s,void* ptr)
{
	m_pAction = (CConnector*)ptr;

	if ( !m_pAction->bDoubleClick )
		return 1;

	CConnector* pParentConnector = 0;
	application->isValidWidget(m_pCreateWindow, &pParentConnector);

	// Safety Check. This shouldn't be necessary but I think that
	// if you add a widget and then delete it the delete is not
	// resetting m_pCreateWindow = m_pPane somewhere.
	// No. You definately need it. If the app is started and the
	// user immediately double clicks a widget then there is no
	// selected widget. m_pCreateWindow, m_pTrackingWindow and
	// m_pCurrent are all NULL.
	//
	if ( !pParentConnector )
	{
		m_pTrackingWindow = m_pPane;
		m_pCreateWindow = m_pPane;
		application->isValidWidget(m_pCreateWindow, &pParentConnector);
	}

	// Another safety check. The user may not have selected the dialog yet
	if ( !m_pCurrent )
		application->isValidWidget(m_pPane, &m_pCurrent);

	if ( m_pTrackingWindow == m_pAppFrame || !m_pCurrent->pDispenser->canHaveChildren() )
		m_pCreateWindow = m_pPane;
	else
		m_pCreateWindow = m_pCurrent->pWidget;
//		m_pCreateWindow = m_pCurrent->pDispenser->getCreateParent(m_pCurrent->pWidget);

	if ( m_nPrimaryTracker != 0xFFFF )
	{
		//
		// CMW 14.03.00 Use the CConnectors (m_pAction is the child window under
		//				construction, pCurrent it's (potential) parent) to:
		//
		//				1. Verify that a child will accept the parent as a parent.
		//
		//				2. Establish if the parent accepts children of the same
		//				   type as the child under construction.
		//
		//				   FXLabel's, for example, won't accept children at all
		//				   while an FXShutter will only accept an FXShutterItem
		//				   as a child.
		//
		//				These two checks are essentially the same, one from
		//				the child's perspective and one from the parent's.
		//				I've done this deliberately to provide flexibility
		//

		if	(
			m_pAction->pDispenser->allowParent(m_pCurrent->pWidget) &&
			m_pCurrent->pDispenser->allowChild(m_pAction->pDispenser->getClassInfo())
			)
		{
//			addWidget(pParentConnector->pDispenser->getCreateParent(m_pCreateWindow), TRUE, TRUE);
			addWidget(m_pCurrent->pDispenser->getCreateParent(m_pCreateWindow), TRUE, TRUE);
		}
	}
	else
	{
		addWidget(m_pPane, TRUE);
	}

	m_pAction = 0;

	return 1;
}

FXbool CTrackManager::isMultipleSelected(void)
{
	FXint nCount = 0;

	for (FXint i = 0; i < m_Trackers.no(); i++ )
	{
		if	(
			m_Trackers[i]->frame() != m_pAppFrame &&
			m_Trackers[i]->window() != m_pPane &&
			m_Trackers[i]->isActive()
			)
		{
			nCount++;
		}
	}
	
	if ( nCount > 1 )
	{
		return TRUE;
	}

	return FALSE;
}

void CTrackManager::alignLeft(void)
{
	clearGrabbers();

	for ( FXint i = 0; i < m_Trackers.no(); i++ )
	{
		if ( i == m_nPrimaryTracker )
		{
			continue;
		}

		FXWindow* pWindow = m_Trackers[i]->window();

		if	(
			pWindow != m_pAppFrame &&
			pWindow != m_pFrame &&
			pWindow != m_pPane
			)
		{
			m_Trackers[i]->position(m_Trackers[i]->top(),
									m_Trackers[m_nPrimaryTracker]->left(),
									m_Trackers[i]->width(),
									m_Trackers[i]->height());
		}
	}
}

void CTrackManager::alignRight(void)
{
	clearGrabbers();

	for ( FXint i = 0; i < m_Trackers.no(); i++ )
	{
		if ( i == m_nPrimaryTracker )
		{
			continue;
		}

		FXWindow* pWindow = m_Trackers[i]->window();

		if	(
			pWindow != m_pAppFrame &&
			pWindow != m_pFrame &&
			pWindow != m_pPane
			)
		{
			m_Trackers[i]->position(m_Trackers[i]->top(),
									m_Trackers[m_nPrimaryTracker]->left() + m_Trackers[m_nPrimaryTracker]->width() - m_Trackers[i]->width(),
									m_Trackers[i]->width(),
									m_Trackers[i]->height());
		}
	}
}

void CTrackManager::alignTop(void)
{
	clearGrabbers();

	for ( FXint i = 0; i < m_Trackers.no(); i++ )
	{
		if ( i == m_nPrimaryTracker )
		{
			continue;
		}

		FXWindow* pWindow = m_Trackers[i]->window();

		if	(
			pWindow != m_pAppFrame &&
			pWindow != m_pFrame &&
			pWindow != m_pPane
			)
		{
			m_Trackers[i]->position(m_Trackers[m_nPrimaryTracker]->top(),
									m_Trackers[i]->left(),
									m_Trackers[i]->width(),
									m_Trackers[i]->height());
		}
	}
}

void CTrackManager::alignBottom(void)
{
	clearGrabbers();

	for ( FXint i = 0; i < m_Trackers.no(); i++ )
	{
		if ( i == m_nPrimaryTracker )
		{
			continue;
		}

		FXWindow* pWindow = m_Trackers[i]->window();

		if	(
			pWindow != m_pAppFrame &&
			pWindow != m_pFrame &&
			pWindow != m_pPane
			)
		{
			m_Trackers[i]->position(m_Trackers[m_nPrimaryTracker]->top() + m_Trackers[m_nPrimaryTracker]->height() - m_Trackers[i]->height(),
									m_Trackers[i]->left(),
									m_Trackers[i]->width(),
									m_Trackers[i]->height());
		}
	}
}

void CTrackManager::sizeWidth(void)
{
	clearGrabbers();

	for ( FXint i = 0; i < m_Trackers.no(); i++ )
	{
		if ( i == m_nPrimaryTracker )
		{
			continue;
		}

		FXWindow* pWindow = m_Trackers[i]->window();

		if	(
			pWindow != m_pAppFrame &&
			pWindow != m_pFrame &&
			pWindow != m_pPane
			)
		{
			m_Trackers[i]->position(m_Trackers[i]->top(),
									m_Trackers[i]->left(),
									m_Trackers[m_nPrimaryTracker]->width(),
									m_Trackers[i]->height());
		}
	}
}

void CTrackManager::sizeHeight(void)
{
	clearGrabbers();

	for ( FXint i = 0; i < m_Trackers.no(); i++ )
	{
		if ( i == m_nPrimaryTracker )
		{
			continue;
		}

		FXWindow* pWindow = m_Trackers[i]->window();

		if	(
			pWindow != m_pAppFrame &&
			pWindow != m_pFrame &&
			pWindow != m_pPane
			)
		{
			m_Trackers[i]->position(m_Trackers[i]->top(),
									m_Trackers[i]->left(),
									m_Trackers[i]->width(),
									m_Trackers[m_nPrimaryTracker]->height());
		}
	}
}

void CTrackManager::sizeBoth(void)
{
	clearGrabbers();

	for ( FXint i = 0; i < m_Trackers.no(); i++ )
	{
		if ( i == m_nPrimaryTracker )
		{
			continue;
		}

		FXWindow* pWindow = m_Trackers[i]->window();

		if	(
			pWindow != m_pAppFrame &&
			pWindow != m_pFrame &&
			pWindow != m_pPane
			)
		{
			m_Trackers[i]->position(m_Trackers[i]->top(),
									m_Trackers[i]->left(),
									m_Trackers[m_nPrimaryTracker]->width(),
									m_Trackers[m_nPrimaryTracker]->height());
		}
	}
}

void CTrackManager::centreHorizontal(void)
{
	clearGrabbers();

	FXint nLeft   = 0xFFFF;
	FXint nRight  = 0;

	FXint i = 0;
	for ( i = 0; i < m_Trackers.no(); i++ )
	{
		// Only operate on widgets that are children of the frame
		if	(
			m_Trackers[i]->frame() != m_pAppFrame &&
			m_Trackers[i]->window() != m_pPane &&
			m_Trackers[i]->isActive()
			)
		{
			nLeft   = FXMIN(nLeft, m_Trackers[i]->left());
			nRight  = FXMAX(nRight, m_Trackers[i]->left()+m_Trackers[i]->width());
		}
	}

	FXint nXOffset = (m_pPane->getWidth() - (nRight - nLeft))/2;

	for ( i = 0; i < m_Trackers.no(); i++ )
	{
		// Only operate on widgets that are children of the frame
		if	(
			m_Trackers[i]->frame() != m_pAppFrame &&
			m_Trackers[i]->window() != m_pPane &&
			m_Trackers[i]->isActive()
			)
		{
			m_Trackers[i]->position(m_Trackers[i]->top(),
									m_Trackers[i]->left()+nXOffset-nLeft,
									m_Trackers[i]->width(),
									m_Trackers[i]->height());
		}
	}
}

void CTrackManager::centreVertical(void)
{
	clearGrabbers();

	FXint nTop    = 0xFFFF;
	FXint nBottom = 0;

	FXint i = 0;
	for ( i = 0; i < m_Trackers.no(); i++ )
	{
		// Only operate on widgets that are children of the frame
		if	(
			m_Trackers[i]->frame() != m_pAppFrame &&
			m_Trackers[i]->window() != m_pPane &&
			m_Trackers[i]->isActive()
			)
		{
			nTop    = FXMIN(nTop, m_Trackers[i]->top());
			nBottom = FXMAX(nBottom, m_Trackers[i]->top()+m_Trackers[i]->height());
		}
	}

	FXint nYOffset = (m_pPane->getHeight() - (nBottom - nTop))/2;

	for ( i = 0; i < m_Trackers.no(); i++ )
	{
		// Only operate on widgets that are children of the frame
		if	(
			m_Trackers[i]->frame() != m_pAppFrame &&
			m_Trackers[i]->window() != m_pPane &&
			m_Trackers[i]->isActive()
			)
		{
			m_Trackers[i]->position(m_Trackers[i]->top()+nYOffset-nTop,
									m_Trackers[i]->left(),
									m_Trackers[i]->width(),
									m_Trackers[i]->height());
		}
	}
}

FXint CTrackManager::getTrackerCount(void)
{
	return m_Trackers.no();
}

CConnector* CTrackManager::getConnector(FXint nIndex)
{
	CConnector* pConnector = 0;
	application->isValidWidget(m_Trackers[nIndex]->window(), &pConnector);
	return pConnector;
}

void CTrackManager::recalc()
{
	/* CMW 09.03.00 Force recalc -- FXShutterItem */
	if ( m_pCurrent )
	{
		m_pCurrent->pWidget->recalc();
		m_pCurrent->pWidget->getParent()->recalc();
	}

	//	CMW 16.11.99 - This recalc causes the CFrame to flicker 
	m_pPane->recalc();
	application->forceRefresh();

//	m_pPane->update();
}

FXWindow* CTrackManager::addWidget(FXWindow* pParent, FXbool bCreate, FXbool bSetCurrent, FXshort x, FXshort y, FXshort w, FXshort h)
{
	FXWindow* pWnd = m_pAction->addWidget(pParent, bCreate, bSetCurrent, x, y, w, h);

	if ( !application->isCtrlPressed() )
	{
		m_pCurrent = m_pAction;

		if ( m_pTrackingWindow == m_pAppFrame || !m_pCurrent->pDispenser->canHaveChildren() )
			m_pCreateWindow = m_pPane;
		else
			m_pCreateWindow = m_pCurrent->pWidget;

		m_pTrackingWindow = m_pCurrent->pWidget;
	}

	m_pCurrent->select();

	application->setDirty();

	return pWnd;
}
